@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/aclock.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/aclock.cpp   3.3.4   edited Dec 10 2003 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -21,7 +21,7 @@ AnalogClock::AnalogClock( QWidget *parent, const char *name )
     : QWidget( parent, name )
 {
     time = QTime::currentTime();		// get current time
-    QTimer *internalTimer = new QTimer( this );	// create internal timer
+    internalTimer = new QTimer( this );	// create internal timer
     connect( internalTimer, SIGNAL(timeout()), SLOT(timeout()) );
     internalTimer->start( 5000 );		// emit signal every 5 seconds
 }
@@ -38,21 +38,31 @@ void AnalogClock::mouseMoveEvent( QMouseEvent *e )
 	move( e->globalPos() - clickPos );
 }
 
-void AnalogClock::setTime( const QTime & t )
-{
-    time = t;
-    timeout();
-}
-
 //
 // The QTimer::timeout() signal is received by this slot.
 //
 
+//
+// When we set an explicit time we don't want the timeout() slot to be 
+// called anymore as this relies on currentTime()
+//
+void AnalogClock::setTime( const QTime & t )
+{
+    time = t;
+    disconnect( internalTimer, SIGNAL(timeout()), this, SLOT(timeout()) );
+    if (autoMask())
+	updateMask();
+    else
+	update();
+}
+
+
 void AnalogClock::timeout()
 {
-    QTime new_time = QTime::currentTime();	// get the current time
-    time = time.addSecs( 5 );
-    if ( new_time.minute() != time.minute() ) {	// minute has changed
+    QTime old_time = time;
+    time = QTime::currentTime();
+    if ( old_time.minute() != time.minute() 
+	|| old_time.hour() != time.hour() ) {	// minute or hour has changed
 	if (autoMask())
 	    updateMask();
 	else
@@ -104,8 +114,7 @@ void AnalogClock::drawClock( QPainter *paint )
     int d = QMIN( v.width(), v.height() );
     paint->setViewport( v.left() + (v.width()-d)/2,
 			v.top() + (v.height()-d)/2, d, d );
-
-    // time = QTime::currentTime();
+    
     QPointArray pts;
 
     paint->save();
