@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/main.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/main.cpp   3.3.4   edited Jul 14 2003 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -10,8 +10,7 @@
 
 #include "helpwindow.h"
 #include <qapplication.h>
-#include <qwindowsstyle.h>
-#include <qstylesheet.h>
+#include <qdir.h>
 #include <stdlib.h>
 
 
@@ -21,12 +20,13 @@ int main( int argc, char ** argv )
     QApplication a(argc, argv);
 
     QString home;
-    if (argc > 1)
+    if (argc > 1) {
         home = argv[1];
-    else
-        home = QString(getenv("QTDIR")) + "/doc/html/index.html";
+    } else {
+	// Use a hard coded path. It is only an example.
+	home = QDir( "../../doc/html/index.html" ).absPath();
+    }
 
-    
     HelpWindow *help = new HelpWindow(home, ".", 0, "help viewer");
     help->setCaption("Qt Example - Helpviewer");
     if ( QApplication::desktop()->width() > 400
