@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/main.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/main.cpp   3.3.4   edited Sep 10 2003 $
 **
 ** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
@@ -22,12 +22,12 @@ int main( int argc, char *argv[] )
 	// Copy data from the oracle database to the ODBC (default)
 	// database
 	QSqlQuery target;
-	QSqlQuery query( "SELECT id, name FROM people;", oracledb );
+	QSqlQuery query( "SELECT id, name FROM people", oracledb );
 	if ( query.isActive() ) {
 	    while ( query.next() ) {
 		target.exec( "INSERT INTO people ( id, name ) VALUES ( " + 
 			      query.value(0).toString() + 
-			      ", '" + query.value(1).toString() +  "' );" );
+			      ", '" + query.value(1).toString() +  "' )" );
 	    }
 	}
     }
