@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/main.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/main.cpp   3.3.4   edited Sep 10 2003 $
 **
 ** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
@@ -18,7 +18,7 @@ int main( int argc, char *argv[] )
     QApplication app( argc, argv, FALSE );
 
     if ( createConnections() ) {
-	QSqlQuery query( "SELECT id, surname FROM staff;" );
+	QSqlQuery query( "SELECT id, surname FROM staff" );
 	if ( query.isActive() ) {
 	    while ( query.next() ) {
 		qDebug( query.value(0).toString() + ": " +
