@@ -1,12 +1,12 @@
 /**********************************************************************
-** $Id:  qt/qpsprinter_p.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qpsprinter_p.h   3.3.4   edited May 27 2003 $
 **
 ** Definition of internal QPSPrinter class.
 ** QPSPrinter implements PostScript (tm) output via QPrinter.
 **
 ** Created : 940927
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -61,7 +61,7 @@
 
 #ifndef QT_NO_PRINTER
 
-struct QPSPrinterPrivate;
+class QPSPrinterPrivate;
 
 class Q_EXPORT QPSPrinter : public QPaintDevice
 {
