@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qasciicache.h   3.2.1   edited May 13 09:08 $
+** $Id: qt/qasciicache.h   3.3.4   edited Apr 6 2004 $
 **
 ** Definition of QAsciiCache template/macro class
 **
@@ -44,7 +44,12 @@
 
 
 template<class type> 
-class QAsciiCache : public QGCache
+class QAsciiCache
+#ifdef Q_QDOC
+	: public QPtrCollection
+#else
+	: public QGCache
+#endif
 {
 public:
     QAsciiCache( const QAsciiCache<type> &c ) : QGCache(c) {}
