@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qcache.h   3.3.4   edited May 27 2003 $
 **
 ** Definition of QCache template class
 **
@@ -91,8 +91,7 @@ template<class type> inline void QCache<type>::deleteItem( QPtrCollection::Item
 }
 
 template<class type>
-class QCacheIterator
-: public QGCacheIterator
+class QCacheIterator : public QGCacheIterator
 {
 public:
     QCacheIterator( const QCache<type> &c ):QGCacheIterator((QGCache &)c) {}
