@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qftp.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qftp.h   3.3.4   edited May 27 2003 $
 **
 ** Definition of QFtp class.
 **
@@ -54,19 +54,97 @@
 
 
 class QSocket;
-
+class QFtpCommand;
 
 class QM_EXPORT_FTP QFtp : public QNetworkProtocol
 {
     Q_OBJECT
 
 public:
-    QFtp();
+    QFtp(); // ### Qt 4.0: get rid of this overload
+    QFtp( QObject *parent, const char *name=0 );
     virtual ~QFtp();
+
     int supportedOperations() const;
 
+    // non-QNetworkProtocol functions:
+    enum State {
+	Unconnected,
+	HostLookup,
+	Connecting,
+	Connected,
+	LoggedIn,
+	Closing
+    };
+    enum Error {
+	NoError,
+	UnknownError,
+	HostNotFound,
+	ConnectionRefused,
+	NotConnected
+    };
+    enum Command {
+	None,
+	ConnectToHost,
+	Login,
+	Close,
+	List,
+	Cd,
+	Get,
+	Put,
+	Remove,
+	Mkdir,
+	Rmdir,
+	Rename,
+	RawCommand
+    };
+
+    int connectToHost( const QString &host, Q_UINT16 port=21 );
+    int login( const QString &user=QString::null, const QString &password=QString::null );
+    int close();
+    int list( const QString &dir=QString::null );
+    int cd( const QString &dir );
+    int get( const QString &file, QIODevice *dev=0 );
+    int put( const QByteArray &data, const QString &file );
+    int put( QIODevice *dev, const QString &file );
+    int remove( const QString &file );
+    int mkdir( const QString &dir );
+    int rmdir( const QString &dir );
+    int rename( const QString &oldname, const QString &newname );
+
+    int rawCommand( const QString &command );
+
+    Q_ULONG bytesAvailable() const;
+    Q_LONG readBlock( char *data, Q_ULONG maxlen );
+    QByteArray readAll();
+
+    int currentId() const;
+    QIODevice* currentDevice() const;
+    Command currentCommand() const;
+    bool hasPendingCommands() const;
+    void clearPendingCommands();
+
+    State state() const;
+
+    Error error() const;
+    QString errorString() const;
+
+public slots:
+    void abort();
+
+signals:
+    void stateChanged( int );
+    void listInfo( const QUrlInfo& );
+    void readyRead();
+    void dataTransferProgress( int, int );
+    void rawCommandReply( int, const QString& );
+
+    void commandStarted( int );
+    void commandFinished( int, bool );
+    void done( bool );
+
 protected:
-    void parseDir( const QString &buffer, QUrlInfo &info );
+    void parseDir( const QString &buffer, QUrlInfo &info ); // ### Qt 4.0: delete this? (not public API)
     void operationListChildren( QNetworkOperation *op );
     void operationMkDir( QNetworkOperation *op );
     void operationRemove( QNetworkOperation *op );
@@ -74,6 +152,8 @@ protected:
     void operationGet( QNetworkOperation *op );
     void operationPut( QNetworkOperation *op );
 
+    // ### Qt 4.0: delete these
+    // unused variables:
     QSocket *commandSocket, *dataSocket;
     bool connectionReady, passiveMode;
     int getTotalSize, getDoneSize;
@@ -82,27 +162,36 @@ protected:
     bool errorInListChildren;
 
 private:
+    void init();
+    int addCommand( QFtpCommand * );
+
     bool checkConnection( QNetworkOperation *op );
-    void close();
-    void reinitCommandSocket();
-    void okButTryLater( int code, const QCString &data );
-    void okGoOn( int code, const QCString &data );
-    void okButNeedMoreInfo( int code, const QCString &data );
-    void errorForNow( int code, const QCString &data );
-    void errorForgetIt( int code, const QCString &data );
+
+private slots:
+    void startNextCommand();
+    void piFinished( const QString& );
+    void piError( int, const QString& );
+    void piConnectState( int );
+    void piFtpReply( int, const QString& );
+
+private slots:
+    void npListInfo( const QUrlInfo & );
+    void npDone( bool );
+    void npStateChanged( int );
+    void npDataTransferProgress( int, int );
+    void npReadyRead();
 
 protected slots:
+    // ### Qt 4.0: delete these
     void hostFound();
     void connected();
     void closed();
-    void readyRead();
     void dataHostFound();
     void dataConnected();
     void dataClosed();
     void dataReadyRead();
     void dataBytesWritten( int nbytes );
     void error( int );
-
 };
 
 #endif // QT_NO_NETWORKPROTOCOL_FTP
