@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qinputdialog.h   3.2.3   edited May 13 09:08 $
+** $Id: qt/qinputdialog.h   3.3.4   edited Jan 21 2004 $
 **
 ** Definition of QInputDialog class
 **
@@ -52,9 +52,6 @@ class QInputDialogPrivate;
 
 class Q_EXPORT QInputDialog : public QDialog
 {
-#if defined(Q_CC_MSVC)
-    friend class QInputDialog;
-#endif
     Q_OBJECT
 
 private:
