@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qmotifstyle.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qmotifstyle.h   3.3.4   edited Apr 2 2004 $
 **
 ** Definition of Motif-like style class
 **
@@ -42,7 +42,7 @@
 #include "qcommonstyle.h"
 #endif // QT_H
 
-#ifndef QT_NO_STYLE_MOTIF
+#if !defined(QT_NO_STYLE_MOTIF) || defined(QT_PLUGIN)
 
 class QPalette;
 
@@ -91,7 +91,11 @@ public:
 			     const QRect& r,
 			     const QColorGroup& cg,
 			     SFlags how = Style_Default,
+#ifdef Q_QDOC
 			     SCFlags sub = SC_All,
+#else
+			     SCFlags sub = (uint)SC_All,
+#endif
 			     SCFlags subActive = SC_None,
 			     const QStyleOption& = QStyleOption::Default ) const;
 
