@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/unixmake.cpp   3.2.3   edited Sep 1 04:40 $
+** $Id: qt/unixmake.cpp   3.3.4   edited Jul 27 01:34 $
 **
 ** Implementation of UnixMakefileGenerator class.
 **
@@ -76,10 +76,22 @@ UnixMakefileGenerator::init()
 	    project->variables()["QMAKE_EXTENSION_SHLIB"].append( "dll" );
 	}
     }
+    if( project->isEmpty("QMAKE_CFLAGS_PRECOMPILE"))
+	project->variables()["QMAKE_CFLAGS_PRECOMPILE"].append("-x c-header -c");
+    if( project->isEmpty("QMAKE_CXXFLAGS_PRECOMPILE"))
+	project->variables()["QMAKE_CXXFLAGS_PRECOMPILE"].append("-x c++-header -c");
+    if( project->isEmpty("QMAKE_CFLAGS_USE_PRECOMPILE"))
+	project->variables()["QMAKE_CFLAGS_USE_PRECOMPILE"].append("-include");
+    if( project->isEmpty("QMAKE_EXTENSION_PLUGIN") )
+	project->variables()["QMAKE_EXTENSION_PLUGIN"].append(project->first("QMAKE_EXTENSION_SHLIB"));
     if( project->isEmpty("QMAKE_COPY_FILE") )
 	project->variables()["QMAKE_COPY_FILE"].append( "$(COPY)" );
     if( project->isEmpty("QMAKE_COPY_DIR") )
 	project->variables()["QMAKE_COPY_DIR"].append( "$(COPY) -R" );
+    if( project->isEmpty("QMAKE_INSTALL_FILE") )
+	project->variables()["QMAKE_INSTALL_FILE"].append( "$(COPY_FILE)" );
+    if( project->isEmpty("QMAKE_INSTALL_DIR") )
+	project->variables()["QMAKE_INSTALL_DIR"].append( "$(COPY_DIR)" );
     if( project->isEmpty("QMAKE_LIBTOOL") )
 	project->variables()["QMAKE_LIBTOOL"].append( "libtool --silent" );
     //If the TARGET looks like a path split it into DESTDIR and the resulting TARGET
@@ -179,7 +191,7 @@ UnixMakefileGenerator::init()
 		project->variables()["QMAKE_LIBS"] += project->variables()["QMAKE_LIBS_QT"];
 	}
     }
-    if ( project->isActiveConfig("opengl") ) {
+    if ( project->isActiveConfig("opengl") && !project->isActiveConfig("dlopen_opengl")) {
 	project->variables()["INCLUDEPATH"] += project->variables()["QMAKE_INCDIR_OPENGL"];
 	if(!project->isEmpty("QMAKE_LIBDIR_OPENGL"))
 	    project->variables()["QMAKE_LIBDIR_FLAGS"] += varGlue("QMAKE_LIBDIR_OPENGL", "-L", " -L", "");
@@ -229,22 +241,31 @@ UnixMakefileGenerator::init()
     }
     if ( project->isActiveConfig("moc") )
 	setMocAware(TRUE);
-    if ( project->isEmpty("QMAKE_RUN_CC") )
-	project->variables()["QMAKE_RUN_CC"].append("$(CC) -c $(CFLAGS) $(INCPATH) -o $obj $src");
-    if ( project->isEmpty("QMAKE_RUN_CC_IMP") )
-	project->variables()["QMAKE_RUN_CC_IMP"].append("$(CC) -c $(CFLAGS) $(INCPATH) -o $@ $<");
-    if ( project->isEmpty("QMAKE_RUN_CXX") )
-	project->variables()["QMAKE_RUN_CXX"].append("$(CXX) -c $(CXXFLAGS) $(INCPATH) -o $obj $src");
-    if ( project->isEmpty("QMAKE_RUN_CXX_IMP") )
-	project->variables()["QMAKE_RUN_CXX_IMP"].append("$(CXX) -c $(CXXFLAGS) $(INCPATH) -o $@ $<");
-    project->variables()["QMAKE_FILETAGS"] += QStringList::split("HEADERS SOURCES TARGET DESTDIR", " ");
-    if ( !project->isEmpty("PRECOMPH") ) {
-	initOutPaths(); 	// Need to fix MOC_DIR since we do this before init()
+    QString compile_flag = var("QMAKE_COMPILE_FLAG");
+    if(compile_flag.isEmpty())
+	compile_flag = "-c";
+    if(doPrecompiledHeaders() && !project->isEmpty("PRECOMPILED_HEADER")) {
+	QString prefix_flags = project->first("QMAKE_CFLAGS_PREFIX_INCLUDE");
+	if(prefix_flags.isEmpty())
+	    prefix_flags = "-include";
+	compile_flag += " " + prefix_flags + " " + project->first("QMAKE_ORIG_TARGET");
+    }
+    if(!project->isEmpty("ALLMOC_HEADER")) {
+	initOutPaths(); 	// Need to fix outdirs since we do this before init() (because we could add to SOURCES et al)
 	QString allmoc = fileFixify(project->first("MOC_DIR") + "/allmoc.cpp", QDir::currentDirPath(), Option::output_dir);
 	project->variables()["SOURCES"].prepend(allmoc);
 	project->variables()["HEADERS_ORIG"] = project->variables()["HEADERS"];
 	project->variables()["HEADERS"].clear();
     }
+    if ( project->isEmpty("QMAKE_RUN_CC") )
+	project->variables()["QMAKE_RUN_CC"].append("$(CC) " + compile_flag + " $(CFLAGS) $(INCPATH) -o $obj $src");
+    if ( project->isEmpty("QMAKE_RUN_CC_IMP") )
+	project->variables()["QMAKE_RUN_CC_IMP"].append("$(CC) " + compile_flag + " $(CFLAGS) $(INCPATH) -o $@ $<");
+    if ( project->isEmpty("QMAKE_RUN_CXX") )
+	project->variables()["QMAKE_RUN_CXX"].append("$(CXX) " + compile_flag + " $(CXXFLAGS) $(INCPATH) -o $obj $src");
+    if ( project->isEmpty("QMAKE_RUN_CXX_IMP") )
+	project->variables()["QMAKE_RUN_CXX_IMP"].append("$(CXX) " + compile_flag + " $(CXXFLAGS) $(INCPATH) -o $@ $<");
+    project->variables()["QMAKE_FILETAGS"] += QStringList::split("HEADERS SOURCES TARGET DESTDIR", " ");
     if( project->isActiveConfig("GNUmake") && !project->isEmpty("QMAKE_CFLAGS_DEPS"))
 	include_deps = TRUE; //do not generate deps
     if(project->isActiveConfig("compile_libtool"))
@@ -260,10 +281,12 @@ UnixMakefileGenerator::init()
 	    project->variables()["QMAKE_PKGINFO"].append(project->first("DESTDIR") + "../PkgInfo");
 	    project->variables()["ALL_DEPS"] += project->first("QMAKE_PKGINFO");
 
-	    QString plist = specdir() + QDir::separator() + "Info.plist." +
-			    project->first("TEMPLATE");
+	    QString plist = fileFixify(project->first("QMAKE_INFO_PLIST"));
+	    if(plist.isEmpty())
+		plist = specdir() + QDir::separator() + "Info.plist." + project->first("TEMPLATE");
 	    if(QFile::exists(Option::fixPathToLocalOS(plist))) {
-		project->variables()["QMAKE_INFO_PLIST"].append(plist);
+		if(project->isEmpty("QMAKE_INFO_PLIST"))
+		    project->variables()["QMAKE_INFO_PLIST"].append(plist);
 		project->variables()["QMAKE_INFO_PLIST_OUT"].append(project->first("DESTDIR") +
 								    "../Info.plist");
 		project->variables()["ALL_DEPS"] += project->first("QMAKE_INFO_PLIST_OUT");
@@ -351,6 +374,9 @@ UnixMakefileGenerator::init()
 QStringList
 UnixMakefileGenerator::combineSetLFlags(const QStringList &list1, const QStringList &list2)
 {
+    if(project->isActiveConfig("no_smart_library_merge"))
+	return list1 + list2;
+
     QStringList ret;
     for(int i = 0; i < 2; i++) {
 	const QStringList *lst = i ? &list2 : &list1;
@@ -434,7 +460,7 @@ UnixMakefileGenerator::combineSetLFlags(const QStringList &list1, const QStringL
 void
 UnixMakefileGenerator::processPrlVariable(const QString &var, const QStringList &l)
 {
-    if(var == "QMAKE_PRL_LIBS") 
+    if(var == "QMAKE_PRL_LIBS")
 	project->variables()["QMAKE_CURRENT_PRL_LIBS"] = combineSetLFlags(project->variables()["QMAKE_CURRENT_PRL_LIBS"] +
 									  project->variables()["QMAKE_LIBS"], l);
     else
@@ -451,7 +477,7 @@ UnixMakefileGenerator::findDependency(const QString &dep)
 	    QString targ = var((*it) + ".target");
 	    if(targ.isEmpty())
 		targ = (*it);
-	    if(targ.endsWith(dep)) 
+	    if(targ.endsWith(dep))
 		return targ;
 	}
     }
@@ -469,8 +495,8 @@ UnixMakefileGenerator::findDependency(const QString &dep)
 		    QString out = tmp_out;
 		    QFileInfo fi(Option::fixPathToLocalOS((*input)));
 		    out.replace("${QMAKE_FILE_BASE}", fi.baseName());
-		    out.replace("${QMAKE_FILE_NAME}", fi.fileName());
-		    if(out.endsWith(dep)) 
+		    out.replace("${QMAKE_FILE_NAME}", fi.filePath());
+		    if(out.endsWith(dep))
 			return out;
 		}
 	    }
@@ -479,6 +505,34 @@ UnixMakefileGenerator::findDependency(const QString &dep)
     return MakefileGenerator::findDependency(dep);
 }
 
+QStringList
+&UnixMakefileGenerator::findDependencies(const QString &file)
+{
+    QStringList &ret = MakefileGenerator::findDependencies(file);
+    // Note: The QMAKE_IMAGE_COLLECTION file have all images
+    // as dependency, so don't add precompiled header then
+    if(doPrecompiledHeaders() && !project->isEmpty("PRECOMPILED_HEADER")
+       && file != project->first("QMAKE_IMAGE_COLLECTION")) {
+	QString header_prefix = project->first("QMAKE_ORIG_TARGET") + ".gch" + Option::dir_sep;
+	header_prefix += project->first("QMAKE_PRECOMP_PREFIX");
+	if(file.endsWith(".c")) {
+	    QString precomp_h = header_prefix + "c";
+	    if(!ret.contains(precomp_h))
+		ret += precomp_h;
+	} else {
+	    for(QStringList::Iterator it = Option::cpp_ext.begin(); it != Option::cpp_ext.end(); ++it) {
+		if(file.endsWith(*it)) {
+		    QString precomp_h = header_prefix + "c++";
+		    if(!ret.contains(precomp_h))
+			ret += precomp_h;
+		    break;
+		}
+	    }
+	}
+    }
+    return ret;
+}
+
 bool
 UnixMakefileGenerator::findLibraries()
 {
@@ -656,7 +710,7 @@ UnixMakefileGenerator::processPrlFiles()
 	    }
 	    if(ret && l != l_out)
 		l = l_out;
-	    else 
+	    else
 		break;
 	}
     }
@@ -696,7 +750,7 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
 	    if(slsh != -1)
 		dst_prl = dst_prl.right(dst_prl.length() - slsh - 1);
 	    dst_prl = root + targetdir + dst_prl;
-	    ret += "-$(COPY) \"" + project->first("QMAKE_INTERNAL_PRL_FILE") + "\" \"" + dst_prl + "\"";
+	    ret += "-$(INSTALL_FILE) \"" + project->first("QMAKE_INTERNAL_PRL_FILE") + "\" \"" + dst_prl + "\"";
 	    if(!uninst.isEmpty())
 		uninst.append("\n\t");
 	    uninst.append("-$(DEL_FILE) \"" + dst_prl + "\"");
@@ -719,7 +773,7 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
 	    }
 	    if(!ret.isEmpty())
 		ret += "\n\t";
-	    ret += "-$(COPY) \"" + src_lt + "\" \"" + dst_lt + "\"";
+	    ret += "-$(INSTALL_FILE) \"" + src_lt + "\" \"" + dst_lt + "\"";
 	    if(!uninst.isEmpty())
 		uninst.append("\n\t");
 	    uninst.append("-$(DEL_FILE) \"" + dst_lt + "\"");
@@ -743,7 +797,7 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
 	    if(!ret.isEmpty())
 		ret += "\n\t";
 	    ret += mkdir_p_asstring(d) + "\n\t";
-	    ret += "-$(COPY) \"" + src_pc + "\" \"" + dst_pc + "\"";
+	    ret += "-$(INSTALL_FILE) \"" + src_pc + "\" \"" + dst_pc + "\"";
 	    if(!uninst.isEmpty())
 		uninst.append("\n\t");
 	    uninst.append("-$(DEL_FILE) \"" + dst_pc + "\"");
@@ -772,14 +826,14 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
 	QString src_targ = target;
 	if(!destdir.isEmpty())
 	    src_targ = Option::fixPathToTargetOS(destdir + target, FALSE);
-	QString dst_targ = root + fileFixify(targetdir + target);
+	QString dst_targ = root + targetdir + target;
 	if(!ret.isEmpty())
 	    ret += "\n\t";
 	if(resource)
 	    ret += "$(DEL_FILE) -r \"" + dst_targ + "\"" + "\n\t";
 	if(!ret.isEmpty())
 	    ret += "\n\t";
-	ret += QString(resource ? "-$(COPY_DIR)" : "-$(COPY)") + " \"" +
+	ret += QString(resource ? "-$(INSTALL_DIR)" : "-$(INSTALL_FILE)") + " \"" +
 	       src_targ + "\" \"" + dst_targ + "\"";
 	if(!project->isActiveConfig("debug") && !project->isEmpty("QMAKE_STRIP") &&
 	   (project->first("TEMPLATE") != "lib" || !project->isActiveConfig("staticlib"))) {
@@ -809,7 +863,7 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
 		    int lslash = link.findRev(Option::dir_sep);
 		    if(lslash != -1)
 			link = link.right(link.length() - (lslash + 1));
-		    QString dst_link = root + fileFixify(targetdir + link);
+		    QString dst_link = root + targetdir + link;
 		    ret += "\n\t-$(SYMLINK) \"$(TARGET)\" \"" + dst_link + "\"";
 		    if(!uninst.isEmpty())
 			uninst.append("\n\t");
