@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/msvc_objectmodel.cpp   3.2.3   edited Nov 5 09:12 $
+** $Id: qt/msvc_objectmodel.cpp   3.3.4   edited Dec 29 16:35 $
 **
 ** Implementation of VCProject class.
 **
@@ -37,21 +37,7 @@
 #include "msvc_vcproj.h"
 #include <qtextstream.h>
 #include <qstringlist.h>
-#include <quuid.h>
-
-#if defined(Q_OS_WIN32)
-#include <objbase.h>
-#ifndef GUID_DEFINED
-#define GUID_DEFINED
-typedef struct _GUID
-{
-    ulong   Data1;
-    ushort  Data2;
-    ushort  Data3;
-    uchar   Data4[8];
-} GUID;
-#endif
-#endif
+#include <qfileinfo.h>
 
 // XML Tags ---------------------------------------------------------
 const char* _xmlInit				= "<?xml version=\"1.0\" encoding = \"Windows-1252\"?>";
@@ -323,8 +309,10 @@ QTextStream &operator<<( QTextStream &strm, const SPair &prop )
 
 QTextStream &operator<<( QTextStream &strm, const XPair &prop )
 {
-    if ( !prop.value.isEmpty() )
-	strm << prop.name << prop.value.join(prop.sep).latin1() << "\"";
+    if ( !prop.value.isEmpty() ) {
+        QString outText = prop.value.join(prop.sep);
+        strm << prop.name << outText.replace('\"', "&quot;") << "\"";
+    }
     return strm;
 }
 
@@ -360,7 +348,8 @@ VCCLCompilerTool::VCCLCompilerTool()
 	Optimization( optimizeCustom ),
 	OptimizeForProcessor( procOptimizeBlended ),
 	OptimizeForWindowsApplication( unset ),
-	RuntimeLibrary( rtMultiThreaded ),
+	ProgramDataBaseFileName( "" ),
+	RuntimeLibrary( rtSingleThreaded ),
 	RuntimeTypeInfo( unset ),
 	ShowIncludes( unset ),
 	SmallerTypeCheck( unset ),
@@ -394,7 +383,7 @@ QTextStream &operator<<( QTextStream &strm, const VCCLCompilerTool &tool )
     if ( tool.CompileAs != compileAsDefault )		    strm << EPair( _CompileAs, tool.CompileAs );
     if ( tool.CompileAsManaged != managedDefault )	    strm << EPair( _CompileAsManaged, tool.CompileAsManaged );
     strm << TPair( _CompileOnly, tool.CompileOnly );
-    strm << EPair( _DebugInformationFormat, tool.DebugInformationFormat );
+    if ( tool.DebugInformationFormat != debugUnknown )	    strm << EPair( _DebugInformationFormat, tool.DebugInformationFormat );
     strm << TPair( _DefaultCharIsUnsigned, tool.DefaultCharIsUnsigned );
     strm << TPair( _Detect64BitPortabilityProblems, tool.Detect64BitPortabilityProblems );
     strm << TPair( _DisableLanguageExtensions, tool.DisableLanguageExtensions );
@@ -408,7 +397,7 @@ QTextStream &operator<<( QTextStream &strm, const VCCLCompilerTool &tool )
     strm << TPair( _ForceConformanceInForLoopScope, tool.ForceConformanceInForLoopScope );
     strm << XPair( _ForcedIncludeFiles, tool.ForcedIncludeFiles );
     strm << XPair( _ForcedUsingFiles, tool.ForcedUsingFiles );
-    strm << EPair( _GeneratePreprocessedFile, tool.GeneratePreprocessedFile );
+    if ( tool.GeneratePreprocessedFile != preprocessUnknown)strm << EPair( _GeneratePreprocessedFile, tool.GeneratePreprocessedFile );
     strm << TPair( _GlobalOptimizations, tool.GlobalOptimizations );
     strm << TPair( _IgnoreStandardIncludePath, tool.IgnoreStandardIncludePath );
     strm << TPair( _ImproveFloatingPointConsistency, tool.ImproveFloatingPointConsistency );
@@ -424,8 +413,8 @@ QTextStream &operator<<( QTextStream &strm, const VCCLCompilerTool &tool )
     strm << SPair( _PrecompiledHeaderFile, tool.PrecompiledHeaderFile );
     strm << SPair( _PrecompiledHeaderThrough, tool.PrecompiledHeaderThrough );
     strm << XPair( _PreprocessorDefinitions, tool.PreprocessorDefinitions );
-    strm << _ProgramDataBaseFileName << tool.ProgramDataBaseFileName.latin1() << "\"";
-    strm << EPair( _RuntimeLibrary, tool.RuntimeLibrary );
+    if ( !tool.ProgramDataBaseFileName.isNull() ) strm << _ProgramDataBaseFileName << tool.ProgramDataBaseFileName.latin1() << "\"";
+    if ( tool.RuntimeLibrary != rtUnknown ) strm << EPair( _RuntimeLibrary, tool.RuntimeLibrary );
     strm << TPair( _RuntimeTypeInfo, tool.RuntimeTypeInfo );
     strm << TPair( _ShowIncludes, tool.ShowIncludes );
     strm << TPair( _SmallerTypeCheck, tool.SmallerTypeCheck );
@@ -440,7 +429,7 @@ QTextStream &operator<<( QTextStream &strm, const VCCLCompilerTool &tool )
 	 !tool.PrecompiledHeaderThrough.isEmpty() )
 	strm << EPair( _UsePrecompiledHeader, tool.UsePrecompiledHeader );
     strm << TPair( _WarnAsError, tool.WarnAsError );
-    strm << EPair( _WarningLevel, tool.WarningLevel );
+    if ( tool.WarningLevel != warningLevelUnknown ) strm << EPair( _WarningLevel, tool.WarningLevel );
     strm << TPair( _WholeProgramOptimization, tool.WholeProgramOptimization );
     strm << "/>";
 return strm;
@@ -473,13 +462,13 @@ bool VCCLCompilerTool::parseOption( const char* option )
 	if ( second != 'I' ) {
 	    found = FALSE; break;
 	}
-	AdditionalUsingDirectories += option+2;
+	AdditionalUsingDirectories += option+3;
 	break;
     case 'C':
 	KeepComments = _True;
 	break;
     case 'D':
-	PreprocessorDefinitions += option+1;
+	PreprocessorDefinitions += option+2;
 	break;
     case 'E':
 	if ( second == 'H' ) {
@@ -495,6 +484,7 @@ bool VCCLCompilerTool::parseOption( const char* option )
 		     || (third == 's' && fourth == 'c') ) {
 		ExceptionHandling = _True;
 		AdditionalOptions += option;
+		break;
 	    }
 	    found = FALSE; break;
 	}
@@ -962,7 +952,7 @@ VCLinkerTool::VCLinkerTool()
 	HeapReserveSize( -1 ),
 	IgnoreAllDefaultLibraries( unset ),
 	IgnoreEmbeddedIDL( unset ),
-	IgnoreImportLibrary( unset ),
+	IgnoreImportLibrary( _True ),
 	LargeAddressAware( addrAwareDefault ),
 	LinkDLL( unset ),
 	LinkIncremental( linkIncrementalDefault ),
@@ -1715,9 +1705,9 @@ QTextStream &operator<<( QTextStream &strm, const VCCustomBuildTool &tool )
     strm << _begTool3;
     strm << SPair( _ToolName, tool.ToolName );
     strm << XPair( _AdditionalDependencies4, tool.AdditionalDependencies, ";" );
-    strm << SPair( _CommandLine4, tool.CommandLine );
+    strm << XPair( _CommandLine4, tool.CommandLine, "\n" );
     strm << SPair( _Description4, tool.Description );
-    strm << SPair( _Outputs4, tool.Outputs );
+    strm << XPair( _Outputs4, tool.Outputs, ";" );
     strm << SPair( _ToolPath, tool.ToolPath );
     strm << "/>";
    return strm;
@@ -1834,43 +1824,39 @@ QTextStream &operator<<( QTextStream &strm, const VCConfiguration &tool )
 VCFilter::VCFilter()
     :   ParseFiles( unset )
 {
+    useCustomBuildTool = FALSE;
+    useCompilerTool = FALSE;
 }
 
-void VCFilter::generateMOC( QTextStream &strm, QString str ) const
+void VCFilter::addMOCstage( QTextStream & /*strm*/, QString filename )
 {
-    QString mocOutput = Project->findMocDestination( str );
+    QString mocOutput = Project->findMocDestination( filename );
     QString mocApp = Project->var( "QMAKE_MOC" );
 
-    if( mocOutput.isEmpty() ) {
+    if( mocOutput.isEmpty() && filename.endsWith(".moc") ) {
         // In specialcases we DO moc .cpp files
 	// when the result is an .moc file
-	if ( !str.endsWith(".moc") )
-	    return;
-	mocOutput = str;
-	str = Project->findMocSource( mocOutput );
+	mocOutput = filename;
+	filename = Project->findMocSource( mocOutput );
     }
 
-    strm << _begFileConfiguration;
-    strm << _Name5;
-    strm << Config->Name;
-    strm << "\">";
-    strm << _begTool5;
-    strm << _VCCustomBuildTool;
-    strm << _Description6;
-    strm << "Moc'ing " << str << "...\"";
-    strm << _CommandLine6;
-    strm << mocApp;
-    strm << " " << str << " -o " << mocOutput << "\"";
-    strm << _AdditionalDependencies6;
-    strm << mocApp << "\"";
-    strm << _Outputs6;
-    strm << mocOutput << "\"";
-    strm << "/>";
-    strm << _endFileConfiguration;
+    if (mocOutput.isEmpty())
+	return;
+
+    CustomBuildTool = VCCustomBuildTool();
+    useCustomBuildTool = TRUE;
+    CustomBuildTool.Description = "Moc&apos;ing " + filename + "...";
+    CustomBuildTool.CommandLine += (mocApp + " "
+				+ filename + " -o " + mocOutput);
+    CustomBuildTool.AdditionalDependencies = mocApp;
+    CustomBuildTool.Outputs += mocOutput;
 }
 
-void VCFilter::generateUIC( QTextStream &strm, const QString& str ) const
+void VCFilter::addUICstage( QTextStream & /*strm*/, QString str )
 {
+    CustomBuildTool = VCCustomBuildTool();
+    useCustomBuildTool = TRUE;
+
     QString uicApp = Project->var("QMAKE_UIC");
     QString mocApp = Project->var( "QMAKE_MOC" );
     QString fname = str.section( '\\', -1 );
@@ -1878,7 +1864,7 @@ void VCFilter::generateUIC( QTextStream &strm, const QString& str ) const
     QString uiDir = Project->var( "UI_DIR" );
     QString uiHeaders;
     QString uiSources;
-    
+
     // Determining the paths for the output files.
     int slash = str.findRev( '\\' );
     QString pname = ( slash != -1 ) ? str.left( slash+1 ) : QString( ".\\" );
@@ -1906,27 +1892,152 @@ void VCFilter::generateUIC( QTextStream &strm, const QString& str ) const
     if ( mocDir.isEmpty() )
 	mocDir = pname;
 
-    strm << _begFileConfiguration;
-    strm << _Name5;
-    strm << Config->Name;
-    strm << "\">";
-    strm << _begTool5;
-    strm << _VCCustomBuildTool;
-    strm << _Description6;
-    strm << "Uic'ing " << str << "...\"";
-    strm << _CommandLine6;
-    strm << uicApp << " " << str << " -o " << uiHeaders << fname << ".h &amp;&amp; ";				// Create .h from .ui file
-    strm << uicApp << " " << str << " -i " << fname << ".h -o " << uiSources << fname << ".cpp &amp;&amp; ";	// Create .cpp from .ui file
-    strm << mocApp << " " << uiHeaders << fname << ".h -o " << mocDir << Option::h_moc_mod << fname << Option::h_moc_ext << "\"";
-    strm << _AdditionalDependencies6;
-    strm << mocApp << ";" << uicApp << "\"";
-    strm << _Outputs6;
-    strm << uiHeaders << fname << ".h;" << uiSources << fname << ".cpp;" << mocDir << Option::h_moc_mod << fname << Option::h_moc_ext << "\"";
-    strm << "/>";
-    strm << _endFileConfiguration;
+    CustomBuildTool.Description = ("Uic'ing " + str + "...\"");
+    CustomBuildTool.CommandLine += // Create .h from .ui file
+	uicApp + " " + str + " -o " + uiHeaders + fname + ".h";
+    CustomBuildTool.CommandLine += // Create .cpp from .ui file
+	uicApp + " " + str + " -i " + fname + ".h -o " + uiSources + fname + ".cpp";
+    CustomBuildTool.CommandLine += // Moc the headerfile
+	mocApp + " " + uiHeaders + fname + ".h -o " + mocDir + Option::h_moc_mod + fname + Option::h_moc_ext;
+
+    CustomBuildTool.AdditionalDependencies += mocApp;
+    CustomBuildTool.AdditionalDependencies += uicApp;
+    CustomBuildTool.Outputs +=
+	uiHeaders + fname + ".h;" + uiSources + fname + ".cpp;" + mocDir + Option::h_moc_mod + fname + Option::h_moc_ext;
 }
 
-QTextStream &operator<<( QTextStream &strm, const VCFilter &tool )
+void VCFilter::modifyPCHstage( QTextStream &/*strm*/, QString str )
+{
+    bool isCFile = str.endsWith(".c");
+    bool isHFile = (str.endsWith(".h") && str == Project->precompH);
+
+    if (!isCFile && !isHFile)
+	return;
+
+    CompilerTool = VCCLCompilerTool();
+    useCompilerTool = TRUE;
+
+    // Unset some default options
+    CompilerTool.BufferSecurityCheck = unset;
+    CompilerTool.DebugInformationFormat = debugUnknown;
+    CompilerTool.ExceptionHandling = unset;
+    CompilerTool.GeneratePreprocessedFile = preprocessUnknown;
+    CompilerTool.Optimization = optimizeDefault;
+    CompilerTool.ProgramDataBaseFileName = QString::null;
+    CompilerTool.RuntimeLibrary = rtUnknown;
+    CompilerTool.WarningLevel = warningLevelUnknown;
+
+    // Setup PCH options
+    CompilerTool.UsePrecompiledHeader     = (isCFile ? pchNone : pchCreateUsingSpecific);
+    CompilerTool.PrecompiledHeaderThrough = "$(NOINHERIT)";
+    CompilerTool.ForcedIncludeFiles       = "$(NOINHERIT)";
+}
+
+bool VCFilter::addIMGstage( QTextStream &/*strm*/, QString str )
+{
+    bool isCorH = FALSE;
+    if (str.endsWith(".c") || str.endsWith(".rc"))
+	isCorH = TRUE;
+    QStringList::Iterator it;
+    for(it = Option::cpp_ext.begin(); it != Option::cpp_ext.end(); ++it)
+	if(str.endsWith(*it))
+	    isCorH = TRUE;
+    for(it = Option::h_ext.begin(); it != Option::h_ext.end(); ++it)
+	if(str.endsWith(*it))
+	    isCorH = TRUE;
+
+    QString collectionName = Project->project->first("QMAKE_IMAGE_COLLECTION");
+    if (str.isEmpty() || isCorH || collectionName.isEmpty())
+	return FALSE;
+
+    CustomBuildTool = VCCustomBuildTool();
+    useCustomBuildTool = TRUE;
+
+    // Some projects (like designer core) may have too many images to
+    // call uic directly. Therefor we have to create a temporary
+    // file, with the image list, and call uic with the -f option.
+    QString tmpFileCmd = "echo ";
+    QString tmpImageFilename = ".imgcol";
+    QStringList& list = Project->project->variables()["IMAGES"];
+    bool firstOutput = TRUE;
+    it = list.begin();
+    while( it!=list.end() ) {
+	tmpFileCmd += (*it) + " ";
+	++it;
+	if (tmpFileCmd.length()>250 || it==list.end()) {
+	    CustomBuildTool.CommandLine += tmpFileCmd
+					  + (firstOutput?"> ":">> ")
+					  + tmpImageFilename;
+	    tmpFileCmd = "echo ";
+	    firstOutput = FALSE;
+	}
+    }
+
+    QString uicApp = Project->var("QMAKE_UIC");
+    QString commandLine = uicApp + " -embed " + Project->project->first("QMAKE_ORIG_TARGET")
+	+ " -f .imgcol -o " + collectionName;
+
+    // The loop below is to avoid the resulting CommandLine buffer
+    // from being a size of between 2071-#char_replaced and 2102,
+    // as this triggers a buffer overflow bug in VS2003. As we only
+    // the only replacement we use in this buffer is one $(QTDIR),
+    // we assume this can be upto 256 characters long, making the
+    // lower-bound to be 1814 characters. So, if the buffer is
+    // between 1814 and 2103 bytes, the buffer is "padded" til it's
+    // over 2103 bytes in size.
+    int totalSize = CustomBuildTool.CommandLine.join("  ").length();
+    while (totalSize > 1814 && totalSize < 2103) {
+        CustomBuildTool.CommandLine += 
+            "echo \"Padding\" the Custom Build Step buffer to avoid a potentional "
+            "buffer overflow issue with VS2003 for buffer of 1814-2103 bytes > nul";
+        totalSize = CustomBuildTool.CommandLine.join("  ").length();
+    }
+
+    CustomBuildTool.Description = ("Generate imagecollection");
+    CustomBuildTool.CommandLine += commandLine;
+    CustomBuildTool.AdditionalDependencies += uicApp;
+    CustomBuildTool.AdditionalDependencies += list;
+    CustomBuildTool.Outputs = collectionName;
+    CustomBuildTool.Outputs += tmpImageFilename;
+    return TRUE;
+}
+
+bool VCFilter::addLexYaccStage( QTextStream &/*strm*/, QString str )
+{
+    bool doLex = str.endsWith(".l");
+    if (!doLex && !str.endsWith(".y"))
+        return FALSE;
+
+    QString fname = "$(InputName)";
+    if (doLex) {
+        fname += Option::lex_mod + Option::cpp_ext.first();
+	QString lexpath = Project->var("QMAKE_LEX")
+                        + Project->varGlue("QMAKE_LEXFLAGS", " ", " ", "") + " ";
+
+        CustomBuildTool.Description = "Lex'ing $(InputFileName)";
+        CustomBuildTool.CommandLine = lexpath + " $(InputFileName)";
+        CustomBuildTool.CommandLine += "del " + fname;
+        CustomBuildTool.CommandLine += "move lex.yy.c " + fname;
+        CustomBuildTool.Outputs = fname;
+    } else {
+        fname +=Option::yacc_mod;
+	QString yaccpath = Project->var("QMAKE_YACC")
+                         + Project->varGlue("QMAKE_YACCFLAGS", " ", " ", "") + " ";
+
+        CustomBuildTool.Description = "Yacc'ing $(InputFileName)";
+        CustomBuildTool.CommandLine = yaccpath + " $(InputFileName)";
+        CustomBuildTool.CommandLine += "del " + fname + Option::h_ext.first();
+        CustomBuildTool.CommandLine += "move y.tab.h " + fname + Option::h_ext.first();
+        CustomBuildTool.Outputs = fname + Option::h_ext.first();
+        CustomBuildTool.CommandLine += "del " + fname + Option::cpp_ext.first();
+        CustomBuildTool.CommandLine += "move y.tab.c " + fname + Option::cpp_ext.first();
+        CustomBuildTool.Outputs += fname + Option::cpp_ext.first();
+    }
+    useCustomBuildTool = TRUE;
+    return TRUE;
+}
+
+QTextStream &operator<<( QTextStream &strm, VCFilter &tool )
 {
     if ( tool.Files.count() == 0 )
 	return strm;
@@ -1936,17 +2047,79 @@ QTextStream &operator<<( QTextStream &strm, const VCFilter &tool )
     strm << TPair( _ParseFiles, tool.ParseFiles );
     strm << SPair( _Filter, tool.Filter );
     strm << ">";
+
+    bool resourceBuild = FALSE;
+    int currentLevels = 0;
+    QStringList currentDirs;
     for ( QStringList::ConstIterator it = tool.Files.begin(); it != tool.Files.end(); ++it ) {
+	if ( !tool.flat_files ) {
+	    QStringList newDirs = QStringList::split('\\',(*it));
+	    newDirs.pop_back(); // Skip the filename
+
+	    int newLevels = int(newDirs.count());
+	    int equalLevels = 0;
+	    for (int i = 0; i<currentLevels; i++, equalLevels++ )
+		if (currentDirs[i] != newDirs[i])
+		    break;
+	    int closeFilters = currentLevels - equalLevels;
+	    int openFilters  = newLevels - equalLevels;
+
+	    // close previous non-equal filter
+	    while ( closeFilters-- )
+		strm << _endFilter;
+
+	    // open new non-equal filters
+	    newLevels = 0;
+	    while ( openFilters-- ) {
+		strm << _begFilter;
+		strm << SPair( _Name3, newDirs[equalLevels + newLevels] );
+		strm << _Filter << "\">"; // Blank filter
+		++newLevels;
+	    }
+	    currentDirs = newDirs;
+	    currentLevels = int(newDirs.count());
+	}
+
+	tool.useCustomBuildTool = FALSE;
+	tool.useCompilerTool = FALSE;
+	// Add UIC, MOC and PCH stages to file
+	if ( tool.CustomBuild == moc )
+	    tool.addMOCstage( strm, *it );
+	else if ( tool.CustomBuild == uic )
+	    tool.addUICstage( strm, *it );
+	else if ( tool.CustomBuild == lexyacc )
+	    tool.addLexYaccStage( strm, *it );
+	else if ( tool.CustomBuild == resource ) {
+	    if (!resourceBuild)
+		resourceBuild = tool.addIMGstage(strm, *it);
+	}
+	if (tool.Project->usePCH)
+	    tool.modifyPCHstage( strm, *it );
+
 	strm << _begFile;
 	strm << SPair( _RelativePath, *it );
 	strm << ">";
-	if ( tool.CustomBuild == moc )
-	    tool.generateMOC( strm, *it );
-	else if ( tool.CustomBuild == uic )
-	    tool.generateUIC( strm, *it );
+	// Output custom build and compiler options
+	// for all configurations
+	if (tool.useCustomBuildTool || tool.useCompilerTool) {
+	    for ( uint i = 0; i < tool.Config->count(); i++ ) {
+		strm << _begFileConfiguration;
+		strm << _Name5;
+		strm << (*tool.Config)[i].Name;
+		strm << "\">";
+		if (tool.useCustomBuildTool)
+		    strm <<  tool.CustomBuildTool;
+		if (tool.useCompilerTool)
+		    strm <<  tool.CompilerTool;
+		strm << _endFileConfiguration;
+	    }
+	}
 	strm << _endFile;
     }
-
+    // close remaining open filters, in non-flat mode
+    while ( !tool.flat_files && currentLevels-- ) {
+	strm << _endFilter;
+    }
     strm << _endFilter;
     return strm;
 }
@@ -1954,17 +2127,9 @@ QTextStream &operator<<( QTextStream &strm, const VCFilter &tool )
 // VCProject --------------------------------------------------------
 VCProject::VCProject()
 {
-#if defined(Q_WS_WIN32)
-    GUID guid;
-    QUuid uniqueId;
-    HRESULT h = CoCreateGuid( &guid );
-    if ( h == S_OK )
-	uniqueId = QUuid( guid );
-    ProjectGUID = uniqueId.toString();
-#else
-    // Qt doesn't support GUID on other platforms yet
-    ProjectGUID = "";
-#endif
+    VCConfiguration conf;
+    Configuration += conf ; // Release
+    //Configuration += conf ; // Debug added later, after Release init
 }
 
 QTextStream &operator<<( QTextStream &strm, const VCProject &tool )
@@ -1984,17 +2149,18 @@ QTextStream &operator<<( QTextStream &strm, const VCProject &tool )
     strm << "/>";
     strm << _endPlatforms;
     strm << _begConfigurations;
-    strm << tool.Configuration;
+    for ( uint i = 0; i < tool.Configuration.count(); i++ )
+	strm << tool.Configuration[i];
     strm << _endConfigurations;
     strm << _begFiles;
-    strm << tool.SourceFiles;
-    strm << tool.HeaderFiles;
-    strm << tool.MOCFiles;
-    strm << tool.UICFiles;
-    strm << tool.FormFiles;
-    strm << tool.TranslationFiles;
-    strm << tool.LexYaccFiles;
-    strm << tool.ResourceFiles;
+    strm << (VCFilter&)tool.SourceFiles;
+    strm << (VCFilter&)tool.HeaderFiles;
+    strm << (VCFilter&)tool.MOCFiles;
+    strm << (VCFilter&)tool.UICFiles;
+    strm << (VCFilter&)tool.FormFiles;
+    strm << (VCFilter&)tool.TranslationFiles;
+    strm << (VCFilter&)tool.LexYaccFiles;
+    strm << (VCFilter&)tool.ResourceFiles;
     strm << _endFiles;
     strm << _begGlobals;
     strm << _endGlobals;
