@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/winmakefile.cpp   3.2.3   edited Sep 15 07:13 $
+** $Id: qt/winmakefile.cpp   3.3.4   edited Aug 3 11:22 $
 **
 ** Implementation of Win32MakefileGenerator class.
 **
@@ -141,6 +141,16 @@ Win32MakefileGenerator::writeSubDirs(QTextStream &t)
 	t << endl << endl;
     }
 
+    if (project->isActiveConfig("ordered")) {   // generate dependencies
+        for( it.toFirst(); it.current(); ) {
+            QString tar = it.current()->target;
+            ++it;
+            if (it.current())
+                t << it.current()->target << ": " << tar << endl;
+        }
+        t << endl;
+    }
+
     if(project->variables()["QMAKE_INTERNAL_QMAKE_DEPS"].findIndex("qmake_all") == -1)
 	project->variables()["QMAKE_INTERNAL_QMAKE_DEPS"].append("qmake_all");
     writeMakeQmake(t);
@@ -230,6 +240,7 @@ Win32MakefileGenerator::writeSubDirs(QTextStream &t)
 	t << "\n\n" << targ << ":" << deps << "\n\t"
 	  << cmd;
     }
+    t << endl << endl;
 
     if(project->variables()["QMAKE_NOFORCE"].isEmpty())
 	t << "FORCE:" << endl << endl;
@@ -276,6 +287,27 @@ Win32MakefileGenerator::findDependency(const QString &dep)
 		return targ;
 	}
     }
+    {
+	QStringList &quc = project->variables()["QMAKE_EXTRA_WIN_COMPILERS"];
+	for(QStringList::Iterator it = quc.begin(); it != quc.end(); ++it) {
+	    QString tmp_out = project->variables()[(*it) + ".output"].first();
+	    QString tmp_cmd = project->variables()[(*it) + ".commands"].join(" ");
+	    if(tmp_out.isEmpty() || tmp_cmd.isEmpty())
+		continue;
+	    QStringList &tmp = project->variables()[(*it) + ".input"];
+	    for(QStringList::Iterator it2 = tmp.begin(); it2 != tmp.end(); ++it2) {
+		QStringList &inputs = project->variables()[(*it2)];
+		for(QStringList::Iterator input = inputs.begin(); input != inputs.end(); ++input) {
+		    QString out = tmp_out;
+		    QFileInfo fi(Option::fixPathToLocalOS((*input)));
+		    out.replace("${QMAKE_FILE_BASE}", fi.baseName());
+		    out.replace("${QMAKE_FILE_NAME}", fi.filePath());
+		    if(out.endsWith(dep)) 
+			return out;
+		}
+	    }
+	}
+    }
     return MakefileGenerator::findDependency(dep);
 }
 
