@@ -1,11 +1,11 @@
 /**********************************************************************
-** $Id:  qt/qcanvas.h   3.0.5   edited Jan 30 17:06 $
+** $Id: qt/qcanvas.h   3.3.4   edited Jan 21 2004 $
 **
 ** Definition of QCanvas classes
 **
 ** Created : 991211
 **
-** Copyright (C) 1999-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1999-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the canvas module of the Qt GUI Toolkit.
 **
@@ -42,7 +42,6 @@
 #include "qscrollview.h"
 #include "qpixmap.h"
 #include "qptrlist.h"
-#include "qptrdict.h"
 #include "qbrush.h"
 #include "qpen.h"
 #include "qvaluelist.h"
@@ -51,8 +50,10 @@
 
 #if !defined( QT_MODULE_CANVAS ) || defined( QT_LICENSE_PROFESSIONAL ) || defined( QT_INTERNAL_CANVAS )
 #define QM_EXPORT_CANVAS
+#define QM_TEMPLATE_EXTERN_CANVAS
 #else
 #define QM_EXPORT_CANVAS Q_EXPORT
+#define QM_TEMPLATE_EXTERN_CANVAS Q_TEMPLATE_EXTERN
 #endif
 
 #ifndef QT_NO_CANVAS
@@ -71,20 +72,18 @@ class QCanvasItem;
 class QCanvasView;
 class QCanvasPixmap;
 
-
-#if defined(Q_TEMPLATEDLL)
+#if defined(Q_TEMPLATEDLL) && ( !defined(Q_CC_BOR) || !defined(QT_MAKEDLL) || defined(Q_EXPORT_TEMPLATES) )
 // MOC_SKIP_BEGIN
-template class QM_EXPORT_CANVAS QPtrList< QCanvasItem >;
-template class QM_EXPORT_CANVAS QPtrList< QCanvasView >;
-template class QM_EXPORT_CANVAS QValueList< QCanvasItem* >;
+QM_TEMPLATE_EXTERN_CANVAS template class QM_EXPORT_CANVAS QValueListIterator< QCanvasItem* >;
+QM_TEMPLATE_EXTERN_CANVAS template class QM_EXPORT_CANVAS QValueList< QCanvasItem* >;
 // MOC_SKIP_END
 #endif
 
-
 class QM_EXPORT_CANVAS QCanvasItemList : public QValueList<QCanvasItem*> {
 public:
     void sort();
     void drawUnique( QPainter& painter );
+    QCanvasItemList operator+(const QCanvasItemList &l) const;
 };
 
 
@@ -205,6 +204,7 @@ private:
     QCanvasItemExtra& extra();
     uint ani:1;
     uint vis:1;
+    uint val:1;
     uint sel:1;
     uint ena:1;
     uint act:1;
@@ -293,9 +293,7 @@ public:
     virtual void addView(QCanvasView*);
     virtual void removeView(QCanvasView*);
     void drawCanvasArea(const QRect&, QPainter* p=0, bool double_buffer=TRUE);
-#ifndef QT_NO_TRANSFORMATIONS
     void drawViewArea( QCanvasView* view, QPainter* p, const QRect& r, bool dbuf );
-#endif
 
     // These are for QCanvasItem to call
     virtual void addItem(QCanvasItem*);
@@ -352,6 +350,11 @@ private:
     bool dblbuf;
 
     friend void qt_unview(QCanvas* c);
+
+#if defined(Q_DISABLE_COPY) // Disabled copy constructor and operator=
+    QCanvas( const QCanvas & );
+    QCanvas &operator=( const QCanvas & );
+#endif
 };
 
 class QCanvasViewData;
@@ -386,6 +389,12 @@ private:
 private slots:
     void cMoving(int,int);
     void updateContentsSize();
+
+private:
+#if defined(Q_DISABLE_COPY) // Disabled copy constructor and operator=
+    QCanvasView( const QCanvasView & );
+    QCanvasView &operator=( const QCanvasView & );
+#endif
 };
 
 
@@ -406,6 +415,10 @@ public:
     void setOffset(int x, int y) { hotx = x; hoty = y; }
 
 private:
+#if defined(Q_DISABLE_COPY)
+    QCanvasPixmap( const QCanvasPixmap & );
+    QCanvasPixmap &operator=( const QCanvasPixmap & );
+#endif
     void init(const QImage&);
     void init(const QPixmap& pixmap, int hx, int hy);
 
@@ -448,6 +461,10 @@ public:
 	{ return (uint)framecount; }
 
 private:
+#if defined(Q_DISABLE_COPY)
+    QCanvasPixmapArray( const QCanvasPixmapArray & );
+    QCanvasPixmapArray &operator=( const QCanvasPixmapArray & );
+#endif
 #ifndef QT_NO_IMAGEIO
     bool readPixmaps(const QString& datafilenamepattern, int framecount, bool maskonly);
 #endif
@@ -510,6 +527,10 @@ public:
     void draw(QPainter& painter);
 
 private:
+#if defined(Q_DISABLE_COPY)
+    QCanvasSprite( const QCanvasSprite & );
+    QCanvasSprite &operator=( const QCanvasSprite & );
+#endif
     void addToChunks();
     void removeFromChunks();
     void changeChunks();
@@ -562,6 +583,10 @@ protected:
     bool winding() const;
     void setWinding(bool);
 
+    void invalidate();
+    bool isValid() const
+	{ return (bool)val; }
+
 private:
     void scanPolygon( const QPointArray& pa, int winding,
 		      QPolygonalProcessor& process ) const;
@@ -673,6 +698,7 @@ public:
     static int RTTI;
 
     void setPen(QPen p);
+    void moveBy(double dx, double dy);
 
 protected:
     void drawShape(QPainter &);
@@ -758,6 +784,10 @@ protected:
     virtual void draw(QPainter&);
 
 private:
+#if defined(Q_DISABLE_COPY)
+    QCanvasText( const QCanvasText & );
+    QCanvasText &operator=( const QCanvasText & );
+#endif
     void addToChunks();
     void removeFromChunks();
     void changeChunks();
@@ -777,7 +807,8 @@ private:
 			 const QCanvasText* ) const;
 };
 
-
+#define Q_DEFINED_QCANVAS
+#include "qwinexport.h"
 #endif // QT_NO_CANVAS
 
 #endif // QCANVAS_H
