@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qsjiscodec.cpp   3.0.5   edited Apr 2 21:10 $
+** $Id: qt/qsjiscodec.cpp   3.3.4   edited May 27 2003 $
 **
 ** Implementation of QSjisCodec class
 **
 ** Created : 990225
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -36,10 +36,11 @@
 **********************************************************************/
 
 // Most of the code here was originally written by Serika Kurusugawa
-// a.k.a. Junji Takagi, and is include in Qt with the author's permission,
+// a.k.a. Junji Takagi, and is included in Qt with the author's permission,
 // and the grateful thanks of the Trolltech team.
 
 /*! \class QSjisCodec qsjiscodec.h
+    \reentrant
     \ingroup i18n
   \brief The QSjisCodec class provides conversion to and from Shift-JIS.
 
@@ -53,12 +54,14 @@
 
   Most of the code here was written by Serika Kurusugawa,
   a.k.a. Junji Takagi, and is included in Qt with the author's
-  permission and the grateful thanks of the Trolltech team. Here is
-  the copyright statement for that code:
+  permission and the grateful thanks of the Trolltech team.
+  Here is the copyright statement for the code as it was at the
+  point of contribution. Trolltech's subsequent modifications
+  are covered by the usual copyright for Qt.
 
   \legalese
 
-  Copyright (c) 1999 Serika Kurusugawa. All rights reserved.
+  Copyright (C) 1999 Serika Kurusugawa. All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -137,9 +140,9 @@ int QSjisCodec::mibEnum() const
 /*!
   \reimp
 */
-QCString QSjisCodec::fromUnicode(const QString& uc, int& len_in_out) const
+QCString QSjisCodec::fromUnicode(const QString& uc, int& lenInOut) const
 {
-    int l = QMIN((int)uc.length(),len_in_out);
+    int l = QMIN((int)uc.length(),lenInOut);
     int rlen = l*2+1;
     QCString rstr(rlen);
     uchar* cursor = (uchar*)rstr.data();
@@ -165,8 +168,8 @@ QCString QSjisCodec::fromUnicode(const QString& uc, int& len_in_out) const
 	    *cursor++ = '?';	// unknown char
 	}
     }
-    len_in_out = cursor - (uchar*)rstr.data();
-    rstr.truncate(len_in_out);
+    lenInOut = cursor - (uchar*)rstr.data();
+    rstr.truncate(lenInOut);
     return rstr;
 }
 
