@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qutfcodec.cpp   3.2.3   edited Jul 4 09:59 $
+** $Id: qt/qutfcodec.cpp   3.3.4   edited Nov 29 2003 $
 **
 ** Implementation of QUtf{8,16}Codec class
 **
@@ -96,6 +96,11 @@ QCString QUtf8Codec::fromUnicode(const QString& uc, int& lenInOut) const
 
 QString QUtf8Codec::toUnicode(const char* chars, int len) const
 {
+    if (len > 3 && (uchar)chars[0] == 0xef && (uchar)chars[1] == 0xbb && (uchar)chars[2] == 0xbf) {
+	// starts with a byte order mark
+	chars += 3;
+	len -= 3;
+    }
     return QString::fromUtf8( chars, len );
 }
 
