@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qmessagebox.cpp   3.2.3   edited Oct 14 14:11 $
+** $Id: qt/qmessagebox.cpp   3.3.4   edited Aug 10 16:43 $
 **
 ** Implementation of QMessageBox class
 **
@@ -447,7 +447,11 @@ static const int LastButton = QMessageBox::NoAll;
   NOTE: The table of button texts correspond to the button enum.
 */
 
+#ifndef Q_OS_TEMP
 static const char * const mb_texts[] = {
+#else
+const char * mb_texts[] = {
+#endif
     0,
     QT_TRANSLATE_NOOP("QMessageBox","OK"),
     QT_TRANSLATE_NOOP("QMessageBox","Cancel"),
@@ -526,8 +530,7 @@ QMessageBox::QMessageBox( QWidget *parent, const char *name )
 	"Hardware failure.\n\nDisk error detected\nDo you want to stop?",
 	QMessageBox::Question,
 	QMessageBox::Yes | QMessageBox::Default,
-	QMessageBox::No  | QMessageBox::Escape
-	QMessageBox::NoButton );
+	QMessageBox::No  | QMessageBox::Escape );
     if ( mb.exec() == QMessageBox::No )
 	// try again
     \endcode
@@ -916,8 +919,17 @@ void QMessageBox::adjustSize()
     int btn_spacing = 7;
     if ( style().styleHint(QStyle::SH_GUIStyle) == MotifStyle )
         btn_spacing = border;
+#ifndef Q_OS_TEMP
     int buttons = mbd->numButtons * bw + (n-1) * btn_spacing;
     int h = bh;
+#else
+    int visibleButtons = 0;
+    for ( int i = 0; i < mbd->numButtons; ++i )
+	visibleButtons += mbd->pb[i]->isVisible() ? 1 : 0;
+    int buttons = visibleButtons == 0 ? 0 : visibleButtons * bw + (visibleButtons-1) * btn_spacing;
+    int h = visibleButtons == 0 ? 0 : bh;
+    n = visibleButtons;
+#endif
     if ( labelSize.height() )
         h += labelSize.height() + 3*border;
     else
@@ -926,7 +938,7 @@ void QMessageBox::adjustSize()
     if ( mbd->iconLabel.pixmap() && mbd->iconLabel.pixmap()->width() )  {
         mbd->iconLabel.adjustSize();
         lmargin += mbd->iconLabel.width() + border;
-        if ( h < mbd->iconLabel.height() + 3*border + bh )
+        if ( h < mbd->iconLabel.height() + 3*border + bh && n )
             h = mbd->iconLabel.height() + 3*border + bh;
     }
     int w = QMAX( buttons, labelSize.width() + lmargin ) + 2*border;
@@ -946,6 +958,14 @@ void QMessageBox::resizeEvent( QResizeEvent * )
     int n  = mbd->numButtons;
     int bw = mbd->buttonSize.width();
     int bh = mbd->buttonSize.height();
+#ifdef Q_OS_TEMP
+    int visibleButtons = 0;
+    for ( i = 0; i < n; ++i )
+	visibleButtons += mbd->pb[i]->isVisible() ? 1 : 0;
+    n  = visibleButtons;
+    bw = visibleButtons == 0 ? 0 : bw;
+    bh = visibleButtons == 0 ? 0 : bh;
+#endif
     int border = bh / 2 - style().pixelMetric(QStyle::PM_ButtonDefaultIndicator);
     if ( border <= 0 )
         border = 10;
