@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qasyncimageio.cpp   3.3.3   edited Jul 27 15:31 $
+** $Id: qt/qasyncimageio.cpp   3.3.4   edited Dec 14 11:56 $
 **
 ** Implementation of asynchronous image/movie loading classes
 **
@@ -765,7 +765,7 @@ void QGIFFormat::disposePrevious( QImage& img, QImageConsumer* consumer )
 	    consumer->changed(QRect(l, t, r-l+1, b-t+1));
 	break;
       case RestoreImage: {
-	if ( frame > 0 ) {
+	if ( frame >= 0 ) {
 	    QRgb** line = (QRgb **)img.jumpTable();
 	    for (int ln=t; ln<=b; ln++) {
 		memcpy(line[ln]+l,
@@ -1107,7 +1107,7 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 			oldcode=incode;
 			while (sp>stack) {
 			    --sp;
-			    if (!out_of_bounds && *sp!=trans_index)
+			    if (!out_of_bounds && line && *sp!=trans_index)
 				line[y][x] = color(*sp);
 			    x++;
 			    if (x>=swidth) out_of_bounds = TRUE;
