@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qclipboard.cpp   3.2.3   edited Nov 3 16:08 $
+** $Id: qt/qclipboard.cpp   3.3.4   edited Nov 18 2003 $
 **
 ** Implementation of QClipboard class
 **
@@ -233,13 +233,16 @@ QString QClipboard::text( QCString &subtype, Mode mode ) const
     return r;
 }
 
-/*! \obsolete
-    This function uses the QClipboard::text() function which takes
-    a QClipboard::Mode argument.  The value of the mode argument is
-    determined by the return value of selectionModeEnabled().
-    If selectionModeEnabled() returns TRUE, the mode argument is
-    QClipboard::Selection, otherwise the mode argument is
-    QClipboard::Clipboard.
+/*!
+    \overload
+
+    Returns the clipboard text in subtype \a subtype, or a null string
+    if the clipboard does not contain any text. This function uses the
+    QClipboard::text() function which takes a QClipboard::Mode
+    argument.  The value of the mode argument is determined by the
+    return value of selectionModeEnabled(). If selectionModeEnabled()
+    returns TRUE, the mode argument is QClipboard::Selection,
+    otherwise the mode argument is QClipboard::Clipboard.
 */
 // ### remove 4.0
 QString QClipboard::text( QCString& subtype ) const
@@ -265,7 +268,9 @@ QString QClipboard::text( Mode mode ) const
     return text( subtype, mode );
 }
 
-/*! \obsolete
+/*!
+    \overload
+
     This function uses the QClipboard::text() function which takes
     a QClipboard::Mode argument.  The value of the mode argument is
     determined by the return value of selectionModeEnabled().
@@ -273,13 +278,13 @@ QString QClipboard::text( Mode mode ) const
     QClipboard::Selection, otherwise the mode argument is
     QClipboard::Clipboard.
 */
-// ### remove 4.0
+
 QString QClipboard::text() const
 {
     return text( selectionModeEnabled() ? Selection : Clipboard );
 }
 
-/*!
+ /*!
     Copies \a text into the clipboard as plain text.
 
     The \a mode argument is used to control which part of the system
@@ -290,12 +295,15 @@ QString QClipboard::text() const
 
     \sa text(), setData()
 */
+
 void QClipboard::setText( const QString &text, Mode mode )
 {
     setData( new QTextDrag(text), mode );
 }
 
-/*! \obsolete
+/*!
+    \overload
+
     This function uses the QClipboard::setText() function which takes
     a QClipboard::Mode argument.  The value of the mode argument is
     determined by the return value of selectionModeEnabled().
@@ -329,7 +337,9 @@ QImage QClipboard::image( Mode mode ) const
     return r;
 }
 
-/*! \obsolete
+/*!
+    \overload
+
     This function uses the QClipboard::image() function which takes
     a QClipboard::Mode argument.  The value of the mode argument is
     determined by the return value of selectionModeEnabled().
@@ -364,7 +374,9 @@ void QClipboard::setImage( const QImage &image, Mode mode )
     setData( new QImageDrag( image ), mode );
 }
 
-/*! \obsolete
+/*!
+    \overload
+
     This function uses the QClipboard::setImage() function which takes
     a QClipboard::Mode argument.  The value of the mode argument is
     determined by the return value of selectionModeEnabled().
@@ -400,7 +412,9 @@ QPixmap QClipboard::pixmap( Mode mode ) const
     return r;
 }
 
-/*! \obsolete
+/*!
+    \overload
+
     This function uses the QClipboard::pixmap() function which takes
     a QClipboard::Mode argument.  The value of the mode argument is
     determined by the return value of selectionModeEnabled().
@@ -433,7 +447,9 @@ void QClipboard::setPixmap( const QPixmap &pixmap, Mode mode )
     setData( new QImageDrag( pixmap.convertToImage() ), mode );
 }
 
-/*! \obsolete
+/*!
+    \overload
+
     This function uses the QClipboard::setPixmap() function which takes
     a QClipboard::Mode argument.  The value of the mode argument is
     determined by the return value of selectionModeEnabled().
@@ -461,7 +477,9 @@ void QClipboard::setPixmap( const QPixmap &pixmap )
     \sa setData()
 */
 
-/*! \obsolete
+/*!
+    \overload
+
     This function uses the QClipboard::data() function which takes
     a QClipboard::Mode argument.  The value of the mode argument is
     determined by the return value of selectionModeEnabled().
@@ -499,7 +517,9 @@ QMimeSource *QClipboard::data() const
     \sa data()
 */
 
-/*! \obsolete
+/*!
+    \overload
+
     This function uses the QClipboard::setData() function which takes
     a QClipboard::Mode argument.  The value of the mode argument is
     determined by the return value of selectionModeEnabled().
@@ -525,7 +545,9 @@ void QClipboard::setData( QMimeSource *src )
     \sa QClipboard::Mode, supportsSelection()
 */
 
-/*! \obsolete
+/*!
+    \overload
+
     This function uses the QClipboard::clear() function which takes
     a QClipboard::Mode argument.  The value of the mode argument is
     determined by the return value of selectionModeEnabled().
