@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qclipboard_x11.cpp   3.2.3   edited Jul 2 16:07 $
+** $Id: qt/qclipboard_x11.cpp   3.3.4   edited Jan 5 14:24 $
 **
 ** Implementation of QClipboard class for X11
 **
@@ -39,15 +39,15 @@
 // #define QCLIPBOARD_DEBUG_VERBOSE
 
 #ifdef QCLIPBOARD_DEBUG
-#  define DEBUG qDebug
+#  define QDEBUG qDebug
 #else
-#  define DEBUG if (FALSE) qDebug
+#  define QDEBUG if (FALSE) qDebug
 #endif
 
 #ifdef QCLIPBOARD_DEBUG_VERBOSE
-#  define VDEBUG qDebug
+#  define VQDEBUG qDebug
 #else
-#  define VDEBUG if (FALSE) qDebug
+#  define VQDEBUG if (FALSE) qDebug
 #endif
 
 #include "qplatformdefs.h"
@@ -175,7 +175,7 @@ public:
 
     void setSource(QMimeSource* s)
     {
-	delete src;
+	clear(TRUE);
 	src = s;
     }
 
@@ -193,7 +193,7 @@ public:
 	transferred[1] = QPixmap();
     }
 
-    void clear();
+    void clear(bool destruct=TRUE);
 
     QMimeSource *src;
     Time timestamp;
@@ -212,9 +212,10 @@ QClipboardData::QClipboardData()
 QClipboardData::~QClipboardData()
 { clear(); }
 
-void QClipboardData::clear()
+void QClipboardData::clear(bool destruct)
 {
-    delete src;
+    if(destruct)
+	delete src;
     src = 0;
     timestamp = CurrentTime;
 }
@@ -239,6 +240,12 @@ static QClipboardData *clipboardData()
     return internalCbData;
 }
 
+void qt_clipboard_cleanup_mime_source(QMimeSource *src)
+{
+    if(internalCbData && internalCbData->source() == src)
+	internalCbData->clear(FALSE);
+}
+
 static void cleanupSelectionData()
 {
     delete internalSelData;
@@ -297,21 +304,20 @@ static void qt_xclb_incr_timeout(void)
 {
     qWarning("QClipboard: timed out while sending data");
 
-    TransactionMap::Iterator it = transactions->begin(),
-			    end = transactions->end();
-    for (; it != end; ++it) delete *it;
+    while (transactions)
+        delete *transactions->begin();
 }
 
 QClipboardINCRTransaction::QClipboardINCRTransaction(Window w, Atom p, Atom t, int f,
 						     QByteArray d, unsigned int i)
     : window(w), property(p), target(t), format(f), data(d), increment(i), offset(0u)
 {
-    DEBUG("QClipboard: sending %d bytes (INCR transaction %p)", d.size(), this);
+    QDEBUG("QClipboard: sending %d bytes (INCR transaction %p)", d.size(), this);
 
     XSelectInput(QPaintDevice::x11AppDisplay(), window, PropertyChangeMask);
 
     if (! transactions) {
-	VDEBUG("QClipboard: created INCR transaction map");
+	VQDEBUG("QClipboard: created INCR transaction map");
 	transactions = new TransactionMap;
 	prev_x11_event_filter = qt_set_x11_event_filter(qt_xclb_transation_event_handler);
 
@@ -322,13 +328,13 @@ QClipboardINCRTransaction::QClipboardINCRTransaction(Window w, Atom p, Atom t, i
 
 QClipboardINCRTransaction::~QClipboardINCRTransaction(void)
 {
-    VDEBUG("QClipboard: destroyed INCR transacton %p", this);
+    VQDEBUG("QClipboard: destroyed INCR transacton %p", this);
 
     XSelectInput(QPaintDevice::x11AppDisplay(), window, NoEventMask);
 
     transactions->remove(window);
     if (transactions->isEmpty()) {
-	VDEBUG("QClipboard: no more INCR transations");
+	VQDEBUG("QClipboard: no more INCR transations");
 	delete transactions;
 	transactions = 0;
 	(void)qt_set_x11_event_filter(prev_x11_event_filter);
@@ -354,7 +360,7 @@ int QClipboardINCRTransaction::x11Event(XEvent *event)
     unsigned int bytes_left = data.size() - offset;
     if (bytes_left > 0) {
 	unsigned int xfer = QMIN(increment, bytes_left);
-	VDEBUG("QClipboard: sending %d bytes, %d remaining (INCR transaction %p)",
+	VQDEBUG("QClipboard: sending %d bytes, %d remaining (INCR transaction %p)",
 	       xfer, bytes_left - xfer, this);
 
 	XChangeProperty(QPaintDevice::x11AppDisplay(), window, property, target, format,
@@ -438,7 +444,7 @@ static int qt_xclb_event_filter(XEvent *event)
 {
     if (event->xany.type == capture_event_type &&
 	event->xany.window == capture_event_win) {
-	VDEBUG( "QClipboard: event_filter(): caught event type %d", event->type );
+	VQDEBUG( "QClipboard: event_filter(): caught event type %d", event->type );
 	has_captured_event = TRUE;
 	captured_event = *event;
 	return 1;
@@ -450,46 +456,70 @@ static int qt_xclb_event_filter(XEvent *event)
 bool qt_xclb_wait_for_event( Display *dpy, Window win, int type, XEvent *event,
 			     int timeout )
 {
-    if ( waiting_for_data )
-	qFatal( "QClipboard: internal error, qt_xclb_wait_for_event recursed" );
-
-    waiting_for_data = TRUE;
-
     QTime started = QTime::currentTime();
     QTime now = started;
 
-    has_captured_event = FALSE;
-    capture_event_win = win;
-    capture_event_type = type;
+    if (qApp->eventLoop()->inherits("QMotif")) { // yes yes, evil hack, we know
+        if ( waiting_for_data )
+            qFatal( "QClipboard: internal error, qt_xclb_wait_for_event recursed" );
 
-    QX11EventFilter old_event_filter = qt_set_x11_event_filter(qt_xclb_event_filter);
+        waiting_for_data = TRUE;
+        has_captured_event = FALSE;
+        capture_event_win = win;
+        capture_event_type = type;
 
+        QX11EventFilter old_event_filter = qt_set_x11_event_filter(qt_xclb_event_filter);
+
+        do {
+            if ( XCheckTypedWindowEvent(dpy,win,type,event) ) {
+                waiting_for_data = FALSE;
+                qt_set_x11_event_filter(old_event_filter);
+                return TRUE;
+            }
+
+            now = QTime::currentTime();
+            if ( started > now )			// crossed midnight
+                started = now;
+
+            // 0x08 == ExcludeTimers for X11 only
+            qApp->eventLoop()->processEvents( QEventLoop::ExcludeUserInput |
+                                              QEventLoop::ExcludeSocketNotifiers |
+                                              QEventLoop::WaitForMore | 0x08 );
+
+            if ( has_captured_event ) {
+                waiting_for_data = FALSE;
+                *event = captured_event;
+                qt_set_x11_event_filter(old_event_filter);
+                return TRUE;
+            }
+        } while ( started.msecsTo(now) < timeout );
+
+        waiting_for_data = FALSE;
+        qt_set_x11_event_filter(old_event_filter);
+
+        return FALSE;
+    }
+
+    bool flushed = FALSE;
     do {
-	if ( XCheckTypedWindowEvent(dpy,win,type,event) ) {
-	    waiting_for_data = FALSE;
-	    qt_set_x11_event_filter(old_event_filter);
+        if ( XCheckTypedWindowEvent(dpy,win,type,event) )
 	    return TRUE;
-	}
 
 	now = QTime::currentTime();
 	if ( started > now )			// crossed midnight
 	    started = now;
 
-	// 0x08 == ExcludeTimers for X11 only
-	qApp->eventLoop()->processEvents( QEventLoop::ExcludeUserInput |
-					  QEventLoop::ExcludeSocketNotifiers |
-					  QEventLoop::WaitForMore | 0x08 );
-
-	if ( has_captured_event ) {
-	    waiting_for_data = FALSE;
-	    *event = captured_event;
-	    qt_set_x11_event_filter(old_event_filter);
-	    return TRUE;
+	if(!flushed) {
+	    XFlush( dpy );
+	    flushed = TRUE;
 	}
-    } while ( started.msecsTo(now) < timeout );
 
-    waiting_for_data = FALSE;
-    qt_set_x11_event_filter(old_event_filter);
+	// sleep 50ms, so we don't use up CPU cycles all the time.
+	struct timeval usleep_tv;
+	usleep_tv.tv_sec = 0;
+	usleep_tv.tv_usec = 50000;
+	select(0, 0, 0, 0, &usleep_tv);
+    } while ( started.msecsTo(now) < timeout );
 
     return FALSE;
 }
@@ -530,7 +560,7 @@ bool qt_xclb_read_property( Display *dpy, Window win, Atom property,
 
     int  offset = 0, buffer_offset = 0, format_inc = 1, proplen = bytes_left;
 
-    VDEBUG("QClipboard: read_property(): initial property length: %d", proplen);
+    VQDEBUG("QClipboard: read_property(): initial property length: %d", proplen);
 
     switch (*format) {
     case 8:
@@ -551,7 +581,7 @@ bool qt_xclb_read_property( Display *dpy, Window win, Atom property,
 
     bool ok = buffer->resize( proplen + (nullterm ? 1 : 0) );
 
-    VDEBUG("QClipboard: read_property(): buffer resized to %d", buffer->size());
+    VQDEBUG("QClipboard: read_property(): buffer resized to %d", buffer->size());
 
     if ( ok ) {
 	// could allocate buffer
@@ -614,7 +644,7 @@ bool qt_xclb_read_property( Display *dpy, Window win, Atom property,
     if ( size )
 	*size = buffer_offset;
 
-    VDEBUG("QClipboard: read_property(): buffer size %d, buffer offset %d, offset %d",
+    VQDEBUG("QClipboard: read_property(): buffer size %d, buffer offset %d, offset %d",
 	   buffer->size(), buffer_offset, offset);
 
     if ( deleteProperty )
@@ -698,7 +728,7 @@ static Atom send_targets_selection(QClipboardData *d, Window window, Atom proper
     if (d->source()->provides("image/pbm")) atoms++;
     if (d->source()->provides("text/plain")) atoms+=4;
 
-    VDEBUG("QClipboard: send_targets_selection(): %d provided types", atoms);
+    VQDEBUG("QClipboard: send_targets_selection(): %d provided types", atoms);
 
     // for 64 bit cleanness... XChangeProperty expects long* for data with format == 32
     QByteArray data((atoms+3) * sizeof(long)); // plus TARGETS, MULTIPLE and TIMESTAMP
@@ -732,7 +762,7 @@ static Atom send_targets_selection(QClipboardData *d, Window window, Atom proper
 
 #if defined(QCLIPBOARD_DEBUG_VERBOSE)
     for (int index = 0; index < n; index++) {
-	VDEBUG("    atom %d: 0x%lx (%s)", index, atarget[index],
+	VQDEBUG("    atom %d: 0x%lx (%s)", index, atarget[index],
 	       qt_xdnd_atom_to_str(atarget[index]));
     }
 #endif
@@ -747,7 +777,7 @@ static Atom send_string_selection(QClipboardData *d, Atom target, Window window,
     static Atom xa_text = *qt_xdnd_str_to_atom("TEXT");
     static Atom xa_compound_text = *qt_xdnd_str_to_atom("COMPOUND_TEXT");
 
-    DEBUG("QClipboard: send_string_selection():\n"
+    QDEBUG("QClipboard: send_string_selection():\n"
 	  "    property type %lx\n"
 	  "    property name '%s'",
 	  target, qt_xdnd_atom_to_str(target));
@@ -766,7 +796,7 @@ static Atom send_string_selection(QClipboardData *d, Atom target, Window window,
 	if (list[0] != NULL
 	    && XmbTextListToTextProperty(QPaintDevice::x11AppDisplay(),
 					 list, 1, style, &textprop) == Success) {
-	    DEBUG("    textprop type %lx\n"
+	    QDEBUG("    textprop type %lx\n"
 		  "    textprop name '%s'\n"
 		  "    format %d\n"
 		  "    %ld items",
@@ -802,11 +832,9 @@ static Atom send_string_selection(QClipboardData *d, Atom target, Window window,
 
     QByteArray data = d->source()->encodedData(fmt);
 
-    DEBUG("    format 8\n    %d bytes", data.size());
+    QDEBUG("    format 8\n    %d bytes", data.size());
 
-    XChangeProperty(QPaintDevice::x11AppDisplay(), window, property, xtarget, 8,
-		    PropModeReplace, (uchar *) data.data(), data.size());
-    return property;
+    return send_selection(d, xtarget, window, property, 8, data);
 }
 
 static Atom send_pixmap_selection(QClipboardData *d, Atom target, Window window, Atom property)
@@ -842,13 +870,13 @@ static Atom send_selection(QClipboardData *d, Atom target, Window window, Atom p
 
     if (data.isEmpty()) {
 	const char *fmt = qt_xdnd_atom_to_str(target);
-	DEBUG("QClipboard: send_selection(): converting to type '%s'", fmt);
+	QDEBUG("QClipboard: send_selection(): converting to type '%s'", fmt);
 	if (fmt && !d->source()->provides(fmt)) // Not a MIME type we can produce
 	    return None;
 	data = d->source()->encodedData(fmt);
     }
 
-    DEBUG("QClipboard: send_selection():\n"
+    QDEBUG("QClipboard: send_selection():\n"
 	  "    property type %lx\n"
 	  "    property name '%s'\n"
 	  "    format %d\n"
@@ -860,9 +888,8 @@ static Atom send_selection(QClipboardData *d, Atom target, Window window, Atom p
     static Atom motif_clip_temporary = *qt_xdnd_str_to_atom("CLIP_TEMPORARY");
     bool allow_incr = property != motif_clip_temporary;
 
-    const unsigned int increment =
-	(XMaxRequestSize(QPaintDevice::x11AppDisplay()) >= 65536 ?
-	 65536 : XMaxRequestSize(QPaintDevice::x11AppDisplay()));
+    // X_ChangeProperty protocol request is 24 bytes
+    const unsigned int increment = (XMaxRequestSize(QPaintDevice::x11AppDisplay()) * 4) - 24;
     if (data.size() > increment && allow_incr) {
 	long bytes = data.size();
 	XChangeProperty(QPaintDevice::x11AppDisplay(), window, property,
@@ -873,8 +900,8 @@ static Atom send_selection(QClipboardData *d, Atom target, Window window, Atom p
     }
 
     // make sure we can perform the XChangeProperty in a single request
-    if ((int)data.size() >= XMaxRequestSize(QPaintDevice::x11AppDisplay()) - 100)
-	return None; // ### perhaps use several XChangeProperty calls w/ PropModeAppend?
+    if (data.size() > increment)
+        return None; // ### perhaps use several XChangeProperty calls w/ PropModeAppend?
 
     // use a single request to transfer data
     XChangeProperty(QPaintDevice::x11AppDisplay(), window, property, target,
@@ -967,7 +994,7 @@ bool QClipboard::event( QEvent *e )
 	    if (d->timestamp != CurrentTime && xevent->xselectionclear.time < d->timestamp)
 		break;
 
-	    DEBUG("QClipboard: new selection owner 0x%lx at time %lx (ours %lx)",
+	    QDEBUG("QClipboard: new selection owner 0x%lx at time %lx (ours %lx)",
 		  XGetSelectionOwner(dpy, XA_PRIMARY),
 		  xevent->xselectionclear.time, d->timestamp);
 
@@ -986,7 +1013,7 @@ bool QClipboard::event( QEvent *e )
 	    if (d->timestamp != CurrentTime && xevent->xselectionclear.time < d->timestamp)
 		break;
 
-	    DEBUG("QClipboard: new clipboard owner 0x%lx at time %lx (%lx)",
+	    QDEBUG("QClipboard: new clipboard owner 0x%lx at time %lx (%lx)",
 		  XGetSelectionOwner(dpy, qt_xa_clipboard),
 		  xevent->xselectionclear.time, d->timestamp);
 
@@ -1032,7 +1059,7 @@ bool QClipboard::event( QEvent *e )
 	    event.xselection.property  = None;
 	    event.xselection.time      = req->time;
 
-	    DEBUG("QClipboard: SelectionRequest from %lx\n"
+	    QDEBUG("QClipboard: SelectionRequest from %lx\n"
 		  "    selection 0x%lx (%s) target 0x%lx (%s)",
 		  req->requestor,
 		  req->selection,
@@ -1041,13 +1068,10 @@ bool QClipboard::event( QEvent *e )
 		  qt_xdnd_atom_to_str(req->target));
 
 	    QClipboardData *d;
-	    Mode m;
 
 	    if ( req->selection == XA_PRIMARY ) {
-		m = Selection;
 		d = selectionData();
 	    } else if ( req->selection == qt_xa_clipboard ) {
-		m = Clipboard;
 		d = clipboardData();
 	    } else {
 #ifdef QT_CHECK_RANGE
@@ -1067,12 +1091,12 @@ bool QClipboard::event( QEvent *e )
 		break;
 	    }
 
-	    DEBUG("QClipboard: SelectionRequest at time %lx (ours %lx)",
+	    QDEBUG("QClipboard: SelectionRequest at time %lx (ours %lx)",
 		  req->time, d->timestamp);
 
 	    if (d->timestamp == CurrentTime // we don't own the selection anymore
 		|| (req->time != CurrentTime && req->time < d->timestamp)) {
-		DEBUG("QClipboard: SelectionRequest too old");
+		QDEBUG("QClipboard: SelectionRequest too old");
 		XSendEvent(dpy, req->requestor, False, NoEventMask, &event);
 		break;
 	    }
@@ -1174,7 +1198,7 @@ bool QClipboard::event( QEvent *e )
 	    // send selection notify to requestor
 	    XSendEvent(dpy, req->requestor, False, NoEventMask, &event);
 
-	    DEBUG("QClipboard: SelectionNotify to 0x%lx\n"
+	    QDEBUG("QClipboard: SelectionNotify to 0x%lx\n"
 		  "    property 0x%lx (%s)",
 		  req->requestor, event.xselection.property,
 		  qt_xdnd_atom_to_str(event.xselection.property));
@@ -1247,52 +1271,53 @@ const char* QClipboardWatcher::format( int n ) const
 
 	QClipboardWatcher *that = (QClipboardWatcher *) this;
 	QByteArray ba = getDataInFormat(xa_targets);
-	if (ba.size() == 0) // no clipboard data available
-	    return 0;
+	if (ba.size() > 0) {
+	    Atom *unsorted_target = (Atom *) ba.data();
+	    static Atom xa_text = *qt_xdnd_str_to_atom( "TEXT" );
+	    static Atom xa_compound_text = *qt_xdnd_str_to_atom( "COMPOUND_TEXT" );
+	    int i, size = ba.size() / sizeof(Atom);
 
-	Atom *unsorted_target = (Atom *) ba.data();
-	static Atom xa_text = *qt_xdnd_str_to_atom( "TEXT" );
-	static Atom xa_compound_text = *qt_xdnd_str_to_atom( "COMPOUND_TEXT" );
-	int i, size = ba.size() / sizeof(Atom);
+	    // sort TARGETS to prefer some types over others.  some apps
+	    // will report XA_STRING before COMPOUND_TEXT, and we want the
+	    // latter, not the former (if it is present).
+	    Atom* target = new Atom[size+4];
+	    memset( target, 0, (size+4) * sizeof(Atom) );
 
-	// sort TARGETS to prefer some types over others.  some apps
-	// will report XA_STRING before COMPOUND_TEXT, and we want the
-	// latter, not the former (if it is present).
-	Atom* target = new Atom[size+4];
-	memset( target, 0, (size+4) * sizeof(Atom) );
+	    for ( i = 0; i < size; ++i ) {
+		if ( unsorted_target[i] == qt_utf8_string )
+		    target[0] = unsorted_target[i];
+		else if ( unsorted_target[i] == xa_compound_text )
+		    target[1] = unsorted_target[i];
+		else if ( unsorted_target[i] == xa_text )
+		    target[2] = unsorted_target[i];
+		else if ( unsorted_target[i] == XA_STRING )
+		    target[3] = unsorted_target[i];
+		else
+		    target[i + 4] = unsorted_target[i];
+	    }
 
-	for ( i = 0; i < size; ++i ) {
-	    if ( unsorted_target[i] == qt_utf8_string )
-		target[0] = unsorted_target[i];
-	    else if ( unsorted_target[i] == xa_compound_text )
-		target[1] = unsorted_target[i];
-	    else if ( unsorted_target[i] == xa_text )
-		target[2] = unsorted_target[i];
-	    else if ( unsorted_target[i] == XA_STRING )
-		target[3] = unsorted_target[i];
-	    else
-		target[i + 4] = unsorted_target[i];
+	    for (i = 0; i < size + 4; ++i) {
+		if ( target[i] == 0 ) continue;
+
+		VQDEBUG("    format: %s", qt_xdnd_atom_to_str(target[i]));
+
+		if ( target[i] == XA_PIXMAP )
+		    that->formatList.append("image/ppm");
+		else if ( target[i] == XA_STRING )
+		    that->formatList.append( "text/plain;charset=ISO-8859-1" );
+		else if ( target[i] == qt_utf8_string )
+		    that->formatList.append( "text/plain;charset=UTF-8" );
+		else if ( target[i] == xa_text ||
+			  target[i] == xa_compound_text )
+		    that->formatList.append( "text/plain" );
+		else
+		    that->formatList.append(qt_xdnd_atom_to_str(target[i]));
+	    }
+	    delete []target;
+
+	    QDEBUG("QClipboardWatcher::format: %d formats available",
+		  int(that->formatList.count()));
 	}
-
-	for (i = 0; i < size + 4; ++i) {
-	    if ( target[i] == 0 ) continue;
-
-	    if ( target[i] == XA_PIXMAP )
-		that->formatList.append("image/ppm");
-	    else if ( target[i] == XA_STRING )
-		that->formatList.append( "text/plain;charset=ISO-8859-1" );
-	    else if ( target[i] == qt_utf8_string )
-		that->formatList.append( "text/plain;charset=UTF-8" );
-	    else if ( target[i] == xa_text ||
-		      target[i] == xa_compound_text )
-		that->formatList.append( "text/plain" );
-	    else
-		that->formatList.append(qt_xdnd_atom_to_str(target[i]));
-	}
-	delete []target;
-
-	DEBUG("QClipboardWatcher::format: %d formats available",
-	      that->formatList.count());
     }
 
     if (n >= 0 && n < (signed) formatList.count())
@@ -1307,7 +1332,7 @@ QByteArray QClipboardWatcher::encodedData( const char* fmt ) const
     if ( !fmt || empty() )
 	return QByteArray( 0 );
 
-    DEBUG("QClipboardWatcher::encodedData: fetching format '%s'", fmt);
+    QDEBUG("QClipboardWatcher::encodedData: fetching format '%s'", fmt);
 
     Atom fmtatom = 0;
 
@@ -1366,25 +1391,25 @@ QByteArray QClipboardWatcher::getDataInFormat(Atom fmtatom) const
     Display *dpy = QPaintDevice::x11AppDisplay();
     Window   win = requestor->winId();
 
-    DEBUG("QClipboardWatcher::getDataInFormat: selection '%s' format '%s'",
+    QDEBUG("QClipboardWatcher::getDataInFormat: selection '%s' format '%s'",
 	  qt_xdnd_atom_to_str(atom), qt_xdnd_atom_to_str(fmtatom));
 
     XSelectInput(dpy, win, NoEventMask); // don't listen for any events
 
     XDeleteProperty(dpy, win, qt_selection_property);
     XConvertSelection(dpy, atom, fmtatom, qt_selection_property, win, qt_x_time);
-    XFlush(dpy);
+    XSync(dpy, FALSE);
 
-    VDEBUG("QClipboardWatcher::getDataInFormat: waiting for SelectionNotify event");
+    VQDEBUG("QClipboardWatcher::getDataInFormat: waiting for SelectionNotify event");
 
     XEvent xevent;
     if ( !qt_xclb_wait_for_event(dpy,win,SelectionNotify,&xevent,clipboard_timeout) ||
 	 xevent.xselection.property == None ) {
-	DEBUG("QClipboardWatcher::getDataInFormat: format not available");
+	QDEBUG("QClipboardWatcher::getDataInFormat: format not available");
 	return buf;
     }
 
-    VDEBUG("QClipboardWatcher::getDataInFormat: fetching data...");
+    VQDEBUG("QClipboardWatcher::getDataInFormat: fetching data...");
 
     Atom   type;
     XSelectInput(dpy, win, PropertyChangeMask);
@@ -1401,7 +1426,7 @@ QByteArray QClipboardWatcher::getDataInFormat(Atom fmtatom) const
 
     XSelectInput(dpy, win, NoEventMask);
 
-    DEBUG("QClipboardWatcher::getDataInFormat: %d bytes received", buf.size());
+    QDEBUG("QClipboardWatcher::getDataInFormat: %d bytes received", buf.size());
 
     return buf;
 }
@@ -1487,7 +1512,8 @@ void QClipboard::setData( QMimeSource* src, Mode mode )
     }
 
     Window prevOwner = XGetSelectionOwner( dpy, atom );
-    XSetSelectionOwner(dpy, atom, newOwner, d->timestamp);
+    // use qt_x_time, since d->timestamp == CurrentTime when clearing
+    XSetSelectionOwner(dpy, atom, newOwner, qt_x_time);
 
     if ( mode == Selection )
 	emit selectionChanged();
