@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qevent.cpp   3.3.4   edited Jan 10 16:00 $
 **
 ** Implementation of event classes
 **
 ** Created : 931029
 **
-** Copyright (C) 1992-2001 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -90,11 +90,11 @@
     \value MidButton  the middle button.
     \value ShiftButton  a Shift key on the keyboard is also pressed.
     \value ControlButton  a Ctrl key on the keyboard is also pressed.
-    \value AltButton  an Alt (or Meta) key on the keyboard is also pressed.
+    \value AltButton  an Alt key on the keyboard is also pressed.
     \value MetaButton a Meta key on the keyboard is also pressed.
     \value Keypad  a keypad button is pressed.
-    \value KeyButtonMask a mask for ShiftButton, ControlButton and
-    AltButton.
+    \value KeyButtonMask a mask for ShiftButton, ControlButton,
+    AltButton and MetaButton.
     \value MouseButtonMask a mask for LeftButton, RightButton and MidButton.
 */
 
@@ -105,7 +105,7 @@
     types and the specialized classes for each type are these:
 
     \value None  Not an event.
-    \value Accessibility Accessibility information is requested.
+    \value Accessibility  Accessibility information is requested
     \value Timer  Regular timer events, \l{QTimerEvent}.
     \value MouseButtonPress  Mouse press, \l{QMouseEvent}.
     \value MouseButtonRelease  Mouse release, \l{QMouseEvent}.
@@ -113,9 +113,9 @@
     \value MouseMove  Mouse move, \l{QMouseEvent}.
     \value KeyPress  Key press (including Shift, for example), \l{QKeyEvent}.
     \value KeyRelease  Key release, \l{QKeyEvent}.
-    \value IMStart The start of input method composition.
-    \value IMCompose Input method composition is taking place.
-    \value IMEnd The end of input method composition.
+    \value IMStart  The start of input method composition, \l{QIMEvent}.
+    \value IMCompose  Input method composition is taking place, \l{QIMEvent}.
+    \value IMEnd  The end of input method composition, \l{QIMEvent}.
     \value FocusIn  Widget gains keyboard focus, \l{QFocusEvent}.
     \value FocusOut  Widget loses keyboard focus, \l{QFocusEvent}.
     \value Enter  Mouse enters widget's boundaries.
@@ -125,21 +125,21 @@
     \value Resize  Widget's size changed, \l{QResizeEvent}.
     \value Show  Widget was shown on screen, \l{QShowEvent}.
     \value Hide  Widget was hidden, \l{QHideEvent}.
-    \value ShowToParent A child widget has been shown.
-    \value HideToParent A child widget has been hidden.
+    \value ShowToParent  A child widget has been shown.
+    \value HideToParent  A child widget has been hidden.
     \value Close  Widget was closed (permanently), \l{QCloseEvent}.
-    \value ShowNormal Widget should be shown normally.
-    \value ShowMaximized Widget should be shown maximized.
-    \value ShowMinimized Widget should be shown minimized.
-    \value ShowFullScreen Widget should be shown full-screen.
-    \value ShowWindowRequest Widget's window should be shown. \obsolete
-    \value DeferredDelete The object will be deleted after it has
+    \value ShowNormal  Widget should be shown normally (obsolete).
+    \value ShowMaximized  Widget should be shown maximized (obsolete).
+    \value ShowMinimized  Widget should be shown minimized (obsolete).
+    \value ShowFullScreen  Widget should be shown full-screen (obsolete).
+    \value ShowWindowRequest  Widget's window should be shown (obsolete).
+    \value DeferredDelete  The object will be deleted after it has
     cleaned up.
     \value Accel  Key press in child for shortcut key handling, \l{QKeyEvent}.
     \value Wheel  Mouse wheel rolled, \l{QWheelEvent}.
-    \value ContextMenu context popup menu, \l{QContextMenuEvent}.
-    \value AccelAvailable  Internal event used by Qt on some platforms.
+    \value ContextMenu  Context popup menu, \l{QContextMenuEvent}
     \value AccelOverride  Key press in child, for overriding shortcut key handling, \l{QKeyEvent}.
+    \value AccelAvailable internal.
     \value WindowActivate  Window was activated.
     \value WindowDeactivate  Window was deactivated.
     \value CaptionChange  Widget's caption changed.
@@ -149,24 +149,38 @@
     \value PaletteChange  Palette of the widget changed.
     \value ParentPaletteChange  Palette of the parent widget changed.
     \value ApplicationPaletteChange  Default application palette changed.
-    \value Clipboard  Clipboard contents have changed, \l{QClipboard}.
-    \value SockAct  Socket activated; used to implement \l{QSocketNotifier}.
-    \value DragEnter  A drag and drop enters widget, \l{QDragEnterEvent}.
-    \value DragMove  A drag and drop is in progress, \l{QDragMoveEvent}.
-    \value DragLeave  A drag and drop leaves widget, \l{QDragLeaveEvent}.
-    \value Drop  A drag and drop is completed, \l{QDropEvent}.
+    \value Clipboard  Clipboard contents have changed.
+    \value SockAct  Socket activated, used to implement \l{QSocketNotifier}.
+    \value DragEnter  A drag-and-drop enters widget, \l{QDragEnterEvent}.
+    \value DragMove  A drag-and-drop is in progress, \l{QDragMoveEvent}.
+    \value DragLeave  A drag-and-drop leaves widget, \l{QDragLeaveEvent}.
+    \value Drop  A drag-and-drop is completed, \l{QDropEvent}.
     \value DragResponse  Internal event used by Qt on some platforms.
     \value ChildInserted  Object gets a child, \l{QChildEvent}.
     \value ChildRemoved  Object loses a child, \l{QChildEvent}.
     \value LayoutHint  Widget child has changed layout properties.
     \value ActivateControl  Internal event used by Qt on some platforms.
     \value DeactivateControl  Internal event used by Qt on some platforms.
+    \value LanguageChange  The application translation changed, \l{QTranslator}
+    \value LayoutDirectionChange  The direction of layouts changed
+    \value LocaleChange  The system locale changed
     \value Quit  Reserved.
     \value Create  Reserved.
     \value Destroy  Reserved.
     \value Reparent  Reserved.
-    \value Speech Reserved for speech input.
-    \value Tablet Wacom Tablet event.
+    \value Speech  Reserved for speech input.
+    \value TabletMove  A Wacom Tablet Move Event.
+    \value Style  Internal use only
+    \value TabletPress  A Wacom Tablet Press Event
+    \value TabletRelease  A Wacom Tablet Release Event
+    \value OkRequest  Internal event used by Qt on some platforms.
+    \value HelpRequest  Internal event used by Qt on some platforms.
+    \value IconDrag     Internal event used by Qt on some platforms when proxy icon is dragged.
+    \value WindowStateChange The window's state, i.e. minimized,
+    maximized or full-screen, has changed. See \l{QWidget::windowState()}.
+    \value WindowBlocked The window is modally blocked
+    \value WindowUnblocked The window leaves modal blocking
+
     \value User  User defined event.
     \value MaxUser  Last user event id.
 
@@ -289,7 +303,7 @@
 */
 
 QMouseEvent::QMouseEvent( Type type, const QPoint &pos, int button, int state )
-    : QEvent(type), p(pos), b(button),s((ushort)state){
+    : QEvent(type), p(pos), b(button),s((ushort)state), accpt(TRUE){
 	g = QCursor::pos();
 }
 
@@ -396,10 +410,11 @@ QMouseEvent::QMouseEvent( Type type, const QPoint &pos, int button, int state )
     keyboard modifiers), i.e. what buttons and keys were being pressed
     immediately before the event was generated.
 
-    Note that this means that for \c QEvent::MouseButtonPress and \c
-    QEvent::MouseButtonDblClick, the flag for the button() itself will
-    not be set in the state, whereas for \c QEvent::MouseButtonRelease
-    it will.
+    This means that if you have a \c QEvent::MouseButtonPress or a \c
+    QEvent::MouseButtonDblClick state() will \e not include the mouse
+    button that's pressed. But once the mouse button has been
+    released, the \c QEvent::MouseButtonRelease event will have the
+    button() that was pressed.
 
     This value is mainly interesting for \c QEvent::MouseMove; for the
     other cases, button() is more useful.
@@ -467,10 +482,10 @@ Qt::ButtonState QMouseEvent::stateAfter() const
 
     \ingroup events
 
-    Wheel events occur when a mouse wheel is turned while the widget
-    has focus. The rotation distance is provided by delta(). The
-    functions pos() and globalPos() return the mouse pointer location
-    at the time of the event.
+    Wheel events are sent to the widget under the mouse, and if that widget
+    does not handle the event they are sent to the focus widget. The rotation
+    distance is provided by delta(). The functions pos() and globalPos() return
+    the mouse pointer location at the time of the event.
 
     A wheel event contains a special accept flag that indicates
     whether the receiver wants the event. You should call
@@ -508,7 +523,7 @@ Qt::ButtonState QMouseEvent::stateAfter() const
 #ifndef QT_NO_WHEELEVENT
 QWheelEvent::QWheelEvent( const QPoint &pos, int delta, int state, Orientation orient )
     : QEvent(Wheel), p(pos), d(delta), s((ushort)state),
-      accpt(FALSE), o(orient)
+      accpt(TRUE), o(orient)
 {
     g = QCursor::pos();
 }
@@ -529,12 +544,12 @@ QWheelEvent::QWheelEvent( const QPoint &pos, int delta, int state, Orientation o
     \fn int QWheelEvent::delta() const
 
     Returns the distance that the wheel is rotated expressed in
-    multiples or divisions of \c WHEEL_DELTA, which is currently set
-    at 120. A positive value indicates that the wheel was rotated
-    forwards away from the user; a negative value indicates that the
-    wheel was rotated backwards toward the user.
+    multiples or divisions of the \e{wheel delta}, which is currently
+    defined to be 120. A positive value indicates that the wheel was
+    rotated forwards away from the user; a negative value indicates
+    that the wheel was rotated backwards toward the user.
 
-    The \c WHEEL_DELTA constant was set to 120 by the wheel mouse
+    The \e{wheel delta} constant was defined to be 120 by wheel mouse
     vendors to allow building finer-resolution wheels in the future,
     including perhaps a freely rotating wheel with no notches. The
     expectation is that such a device would send more messages per
@@ -653,9 +668,10 @@ QWheelEvent::QWheelEvent( const QPoint &pos, int delta, int state, Orientation o
     Qt.
 
     \value SHIFT the Shift keys provided on all standard keyboards.
+    \value META the Meta keys.
     \value CTRL the Ctrl keys.
     \value ALT the normal Alt keys, but not e.g. AltGr.
-    \value MODIFIER_MASK is a mask of Shift, Ctrl and Alt.
+    \value MODIFIER_MASK is a mask of Shift, Ctrl, Alt and Meta.
     \value UNICODE_ACCEL the accelerator is specified as a Unicode code
     point, not as a Qt Key.
 */
@@ -669,12 +685,13 @@ QWheelEvent::QWheelEvent( const QPoint &pos, int delta, int state, Orientation o
     Key events occur when a key is pressed or released when a widget
     has keyboard input focus.
 
-    A key event contains a special accept flag that indicates whether
-    the receiver wants the key event. You should call
-    QKeyEvent::ignore() if the key press or release event is not
-    handled by your widget. A key event is propagated up the parent
-    widget chain until a widget accepts it with QKeyEvent::accept() or
-    an event filter consumes it.
+  A key event contains a special accept flag that indicates whether the
+  receiver wants the key event.  You should call QKeyEvent::ignore() if the
+  key press or release event is not handled by your widget. A key event is
+  propagated up the parent widget chain until a widget accepts it with
+  QKeyEvent::accept() or an event filter consumes it.
+  Key events for multi media keys are ignored by default. You should call
+  QKeyEvent::accept() if your widget handles those events.
 
     The QWidget::setEnable() function can be used to enable or disable
     mouse and keyboard events for a widget.
@@ -711,13 +728,11 @@ QWheelEvent::QWheelEvent( const QPoint &pos, int delta, int state, Orientation o
     See \l Qt::Key for the list of keyboard codes. These codes are
     independent of the underlying window system.
 
-    Key code 0 means that the event is not a result of a known key
-    (e.g. it may be the result of a compose sequence or keyboard
-    macro).
+    A value of either 0 or Key_unknown means that the event is not
+    the result of a known key (e.g. it may be the result of a compose
+    sequence or a keyboard macro, or due to key event compression).
 
-    When key event compression is turned on, this function's return
-    value is undefined.  See QWidget::setKeyCompression() for details
-    on key event compression.
+    \sa QWidget::setKeyCompression()
 */
 
 /*!
@@ -732,7 +747,10 @@ QWheelEvent::QWheelEvent( const QPoint &pos, int delta, int state, Orientation o
 /*!
     \fn QString QKeyEvent::text() const
 
-    Returns the Unicode text that this key generated.
+    Returns the Unicode text that this key generated. The text returned
+    migth be empty, which is the case when pressing or
+    releasing modifying keys as Shift, Control, Alt and Meta. In these
+    cases key() will contain a valid value.
 
     \sa QWidget::setKeyCompression()
 */
@@ -743,8 +761,8 @@ QWheelEvent::QWheelEvent( const QPoint &pos, int delta, int state, Orientation o
     Returns the keyboard modifier flags that existed immediately
     before the event occurred.
 
-    The returned value is \c ShiftButton, \c ControlButton and \c
-    AltButton OR'ed together.
+    The returned value is \c ShiftButton, \c ControlButton, \c AltButton
+    and \c MetaButton OR'ed together.
 
     \sa stateAfter()
 */
@@ -768,6 +786,8 @@ Qt::ButtonState QKeyEvent::stateAfter() const
 	return Qt::ButtonState(state()^ControlButton);
     if ( key() == Key_Alt )
 	return Qt::ButtonState(state()^AltButton);
+    if ( key() == Key_Meta )
+	return Qt::ButtonState(state()^MetaButton);
     return state();
 }
 
@@ -809,10 +829,6 @@ Qt::ButtonState QKeyEvent::stateAfter() const
     Returns the number of single keys for this event. If text() is not
     empty, this is simply the length of the string.
 
-    However, Qt also compresses invisible keycodes such as BackSpace.
-    For those, count() returns the number of key presses/repeats this
-    event represents.
-
     \sa QWidget::setKeyCompression()
 */
 
@@ -861,6 +877,7 @@ Qt::ButtonState QKeyEvent::stateAfter() const
     \value Key_CapsLock
     \value Key_NumLock
     \value Key_ScrollLock
+    \value Key_Clear
     \value Key_F1
     \value Key_F2
     \value Key_F3
@@ -1070,6 +1087,55 @@ Qt::ButtonState QKeyEvent::stateAfter() const
     \value Key_thorn
     \value Key_ydiaeresis
 
+    Multimedia keys
+
+    \value Key_Back
+    \value Key_Forward
+    \value Key_Stop
+    \value Key_Refresh
+
+    \value Key_VolumeDown
+    \value Key_VolumeMute
+    \value Key_VolumeUp
+    \value Key_BassBoost
+    \value Key_BassUp
+    \value Key_BassDown
+    \value Key_TrebleUp
+    \value Key_TrebleDown
+
+    \value Key_MediaPlay
+    \value Key_MediaStop
+    \value Key_MediaPrev
+    \value Key_MediaNext
+    \value Key_MediaRecord
+
+    \value Key_HomePage
+    \value Key_Favorites
+    \value Key_Search
+    \value Key_Standby
+    \value Key_OpenUrl
+
+    \value Key_LaunchMail
+    \value Key_LaunchMedia
+    \value Key_Launch0
+    \value Key_Launch1
+    \value Key_Launch2
+    \value Key_Launch3
+    \value Key_Launch4
+    \value Key_Launch5
+    \value Key_Launch6
+    \value Key_Launch7
+    \value Key_Launch8
+    \value Key_Launch9
+    \value Key_LaunchA
+    \value Key_LaunchB
+    \value Key_LaunchC
+    \value Key_LaunchD
+    \value Key_LaunchE
+    \value Key_LaunchF
+
+    \value Key_MediaLast
+
     \value Key_unknown
 
     \value Key_Direction_L internal use only
@@ -1219,8 +1285,16 @@ void QFocusEvent::resetReason()
     \fn QPaintEvent::QPaintEvent( const QRect &paintRect, bool erased=TRUE )
 
     Constructs a paint event object with the rectangle that should be
-    updated. The region is given by \a paintRect. If \a erased is TRUE
-    the region will be cleared before repainting.
+    updated. The region is also given by \a paintRect. If \a erased is
+    TRUE the region will be cleared before repainting.
+*/
+
+/*!
+    \fn QPaintEvent::QPaintEvent( const QRegion &paintRegion, const QRect &paintRect, bool erased=TRUE )
+
+    Constructs a paint event object with the rectangle \a paintRect
+    that should be updated. The region is given by \a paintRegion. If
+    \a erased is TRUE the region will be cleared before repainting.
 */
 
 /*!
@@ -1417,6 +1491,61 @@ void QFocusEvent::resetReason()
     \sa accept()
 */
 
+/*!
+   \class QIconDragEvent qevent.h
+   \brief The QIconDragEvent class signals that a main icon drag has begun.
+
+    \ingroup events
+
+    Icon drag events are sent to widgets when the main icon of a window has been dragged away.
+    On Mac OS X this is fired when the proxy icon of a window is dragged off titlebar, in response to
+    this event is is normal to begin using drag and drop.
+*/
+
+/*!
+    \fn QIconDragEvent::QIconDragEvent()
+
+    Constructs an icon drag event object with the accept parameter
+    flag set to FALSE.
+
+    \sa accept()
+*/
+
+/*!
+    \fn bool QIconDragEvent::isAccepted() const
+
+    Returns TRUE if the receiver of the event has started a drag and
+    drop operation; otherwise returns FALSE.
+
+    \sa accept(), ignore()
+*/
+
+/*!
+    \fn void QIconDragEvent::accept()
+
+    Sets the accept flag of the icon drag event object.
+
+    Setting the accept flag indicates that the receiver of this event
+    has started a drag and drop oeration.
+
+    The accept flag is \e not set by default.
+
+    \sa ignore(), QWidget::hide()
+*/
+
+/*!
+    \fn void QIconDragEvent::ignore()
+
+    Clears the accept flag of the icon drag object.
+
+    Clearing the accept flag indicates that the receiver of this event
+    has not handled the icon drag as a result other events can be sent.
+
+    The icon drag event is constructed with the accept flag cleared.
+
+    \sa accept()
+*/
+
 /*!
     \class QContextMenuEvent qevent.h
     \brief The QContextMenuEvent class contains parameters that describe a context menu event.
@@ -1473,7 +1602,7 @@ void QFocusEvent::resetReason()
 */
 
 QContextMenuEvent::QContextMenuEvent( Reason reason, const QPoint &pos, int state )
-    : QEvent( ContextMenu ), p( pos ), accpt( FALSE ), consum(FALSE),
+    : QEvent( ContextMenu ), p( pos ), accpt(TRUE), consum(TRUE),
     reas( reason ), s((ushort)state)
 {
     gp = QCursor::pos();
@@ -1689,7 +1818,7 @@ QContextMenuEvent::QContextMenuEvent( Reason reason, const QPoint &pos, int stat
 
     After the user has selected the final string, an IMEndEvent will
     be sent to the widget. The event contains the final string the
-    user selected, and could be empty if they cancelled the
+    user selected, and could be empty if they canceled the
     composition. This string should be accepted as the final text the
     user entered, and the intermediate composition string should be
     cleared. These events are delivered to \l QWidget::imEndEvent().
@@ -1722,6 +1851,15 @@ QContextMenuEvent::QContextMenuEvent( Reason reason, const QPoint &pos, int stat
     \fn int QIMEvent::cursorPos() const
 
     Returns the current cursor position inside the composition string.
+    Will return -1 for IMStartEvent and IMEndEvent.
+*/
+
+/*!
+    \fn int QIMEvent::selectionLength() const
+
+    Returns the number of characters in the composition string (
+    starting at cursorPos() ) that should be marked as selected by the
+    input widget receiving the event.
     Will return 0 for IMStartEvent and IMEndEvent.
 */
 
@@ -1784,7 +1922,11 @@ QContextMenuEvent::QContextMenuEvent( Reason reason, const QPoint &pos, int stat
     The QWidget::setEnabled() function can be used to enable or
     disable mouse and keyboard events for a widget.
 
-    The event handler QWidget::tabletEvent() receives tablet events.
+  The event handler QWidget::tabletEvent() receives all three types of tablet
+  events.  Qt will first send a tabletEvent and then, if it is not accepted,
+  it will send a mouse event.  This allows applications that don't utilize
+  tablets to use a tablet like a mouse while also enabling those who want to
+  use both tablets and mouses differently.
 
 */
 
@@ -1803,23 +1945,40 @@ QContextMenuEvent::QContextMenuEvent( Reason reason, const QPoint &pos, int stat
 */
 
 /*!
-    \fn QTabletEvent::QTabletEvent( const QPoint &pos, int device, int pressure, int xTilt, int yTilt, const QPair<int,int> &uId )
+  \fn QTabletEvent::QTabletEvent( Type t, const QPoint &pos,
+                                  const QPoint &globalPos, int device,
+                                  int pressure, int xTilt, int yTilt,
+				  const QPair<int,int> &uId )
+  Construct a tablet event of type \a t.  The position of when the event occurred is given
+  int \a pos and \a globalPos.  \a device contains the \link TabletDevice device type\endlink,
+  \a pressure contains the pressure exerted on the \a device, \a xTilt and \a yTilt contain
+  \a device's degree of tilt from the X and Y axis respectively.  The \a uId contains an
+  event id.
 
-    Constructs a tablet event object.
+  On Irix, \a globalPos will contain the high-resolution coordinates received from the
+  tablet device driver, instead of from the windowing system.
 
-    The globalPos() is initialized to QCursor::pos(), i.e. \a pos,
-    which is usually (but not always) correct. Use the other
-    constructor if you need to specify the global position explicitly.
-    \a device contains the \link TabletDevice device type\endlink, \a
-    pressure contains the pressure exerted on the \a device, \a xTilt
-    and \a yTilt contain the \a device's degrees of tilt from the X
-    and Y axis respectively. The \a uId contains an event id.
-
-    \sa pos(), device(), pressure(), xTilt(), yTilt()
+  \sa pos(), globalPos(), device(), pressure(), xTilt(), yTilt()
 */
 
+QTabletEvent::QTabletEvent( Type t, const QPoint &pos, const QPoint &globalPos, int device,
+			    int pressure, int xTilt, int yTilt,
+			    const QPair<int, int> &uId )
+    : QEvent( t ),
+      mPos( pos ),
+      mGPos( globalPos ),
+      mDev( device ),
+      mPress( pressure ),
+      mXT( xTilt ),
+      mYT( yTilt ),
+      mType( uId.first ),
+      mPhy( uId.second ),
+      mbAcc(TRUE)
+{}
+
 /*!
-    \fn QTabletEvent::QTabletEvent( const QPoint &pos, const QPoint &globalPos, int device, int pressure, int xTilt, int yTilt, const QPair<int,int> &uId )
+  \obsolete
+  \fn QTabletEvent::QTabletEvent( const QPoint &pos, const QPoint &globalPos, int device, int pressure, int xTilt, int yTilt, const QPair<int,int> &uId )
 
     Constructs a tablet event object. The position when the event
     occurred is is given in \a pos and \a globalPos. \a device
@@ -2044,8 +2203,8 @@ QContextMenuEvent::QContextMenuEvent( Reason reason, const QPoint &pos, int stat
     QCustomEvent is a generic event class for user-defined events.
     User defined events can be sent to widgets or other QObject
     instances using QApplication::postEvent() or
-    QApplication::sendEvent(). Subclasses of QWidget can easily
-    receive custom events by implementing the QWidget::customEvent()
+    QApplication::sendEvent(). Subclasses of QObject can easily
+    receive custom events by implementing the QObject::customEvent()
     event handler function.
 
     QCustomEvent objects should be created with a type ID that
@@ -2070,8 +2229,8 @@ QContextMenuEvent::QContextMenuEvent( Reason reason, const QPoint &pos, int stat
     {
     public:
 	ColorChangeEvent( QColor color )
-	    : QCustomEvent( 346798 ), c( color ) {};
-	QColor color() const { return c; };
+	    : QCustomEvent( 65432 ), c( color ) {}
+	QColor color() const { return c; }
     private:
 	QColor c;
     };
@@ -2085,7 +2244,7 @@ QContextMenuEvent::QContextMenuEvent( Reason reason, const QPoint &pos, int stat
 
     void MyWidget::customEvent( QCustomEvent * e )
     {
-	if ( e->type() == 346798 ) {  // It must be a ColorChangeEvent
+	if ( e->type() == 65432 ) {  // It must be a ColorChangeEvent
 	    ColorChangeEvent* ce = (ColorChangeEvent*)e;
 	    newColor = ce->color();
 	}
@@ -2105,10 +2264,6 @@ QContextMenuEvent::QContextMenuEvent( Reason reason, const QPoint &pos, int stat
 QCustomEvent::QCustomEvent( int type )
     : QEvent( (QEvent::Type)type ), d( 0 )
 {
-#if defined(QT_CHECK_RANGE)
-    if ( type < (int)QEvent::User )
-	qWarning( "QCustomEvent: Illegal type id." );
-#endif
 }
 
 
@@ -2201,10 +2356,6 @@ QCustomEvent::QCustomEvent( int type )
     methods of existing QDragObject subclasses, such as
     QTextDrag::decode(), or your own subclasses.
 
-    \warning To accept or reject the drop, don't call this function,
-    call acceptAction() instead. This function indicates whether you
-    processed the event at all.
-
     \sa acceptAction()
 */
 
@@ -2404,15 +2555,10 @@ QCustomEvent::QCustomEvent( int type )
   Destroys the event. If it was \link
   QApplication::postEvent() posted \endlink,
   it will be removed from the list of events to be posted.
-
-  \internal
-  It used to print an error (useful for people who posted events
-  that were on the stack).
 */
 
 QEvent::~QEvent()
 {
-    if (posted)
+    if ( posted && qApp )
 	QApplication::removePostedEvent( this );
 }
-
