@@ -1,9 +1,9 @@
 /****************************************************************************
-** $Id: qt/qfontengine_x11.cpp   3.2.3   edited Nov 6 15:51 $
+** $Id: qt/qfontengine_x11.cpp   3.3.4   edited Nov 11 22:47 $
 **
 ** ???
 **
-** Copyright (C) 2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 2003-2004 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -146,7 +146,7 @@ void QFontEngineBox::draw( QPainter *p, int x, int y, const QTextEngine *, const
     glyph_metrics_t ci = boundingBox( glyphs, offsets, numGlyphs );
     p->drawRect( x + ci.x, y + ci.y, ci.width, ci.height );
     p->drawRect( x + ci.x, y + 50 + ci.y, ci.width, ci.height );
-     qDebug("bounding rect=%d %d (%d/%d)", ci.x, ci.y, ci.width, ci.height );
+    qDebug("bounding rect=%d %d (%d/%d)", ci.x, ci.y, ci.width, ci.height );
     p->restore();
     int xp = x;
     int yp = y;
@@ -164,19 +164,24 @@ void QFontEngineBox::draw( QPainter *p, int x, int y, const QTextEngine *, const
     } else {
 	if ( p->txop == QPainter::TxTranslate )
 	    p->map( x, y, &x, &y );
-	XRectangle _rects[32];
-	XRectangle *rects = _rects;
-	if ( si->num_glyphs > 32 )
-	    rects = new XRectangle[si->num_glyphs];
-	for (int k = 0; k < si->num_glyphs; k++) {
-	    rects[k].x = x + (k * _size);
-	    rects[k].y = y - _size + 2;
-	    rects[k].width = rects[k].height = _size - 3;
-	}
 
-	XDrawRectangles(dpy, hd, gc, rects, si->num_glyphs);
-	if ( rects != _rects )
-	    delete [] rects;
+	XRectangle rects[64];
+
+	int gl = 0;
+	while (gl < si->num_glyphs) {
+	    int toDraw = QMIN(64, si->num_glyphs-gl);
+	    int adv = toDraw*_size;
+	    if (x + adv < SHRT_MAX && x > SHRT_MIN) {
+		for (int k = 0; k < toDraw; k++) {
+		    rects[k].x = x + (k * _size);
+		    rects[k].y = y - _size + 2;
+		    rects[k].width = rects[k].height = _size - 3;
+		}
+		XDrawRectangles(dpy, hd, gc, rects, toDraw);
+	    }
+	    gl += toDraw;
+	    x += adv;
+	}
     }
 
     if ( textFlags != 0 )
@@ -438,7 +443,7 @@ void QFontEngineXLFD::draw( QPainter *p, int x, int y, const QTextEngine *engine
     int yorig = y;
 
     Qt::HANDLE font_id = _fs->fid;
-    if ( p->txop > QPainter::TxTranslate ) {
+    if ( p->txop > QPainter::TxTranslate || _scale < 0.9999 || _scale > 1.0001  ) {
 	bool degenerate = QABS( p->m11()*p->m22() - p->m12()*p->m21() ) < 0.01;
 	if ( !degenerate && xlfd_transformations != XlfdTrUnsupported ) {
 	    // need a transformed font from the server
@@ -528,53 +533,37 @@ void QFontEngineXLFD::draw( QPainter *p, int x, int y, const QTextEngine *engine
 	}
 	if ( degenerate || xlfd_transformations == XlfdTrUnsupported ) {
 	    // XServer or font don't support server side transformations, need to do it by hand
-            QRect bbox( 0, 0, si->width, si->ascent + si->descent + 1 );
-            int w=bbox.width(), h=bbox.height();
-            int aw = w, ah = h;
-            int tx=-bbox.x(), ty=-bbox.y();    // text position
+            int w = qRound(si->width/_scale), h = qRound((si->ascent + si->descent + 1)/_scale);
             QWMatrix mat1 = p->xmat;
-	    if ( aw == 0 || ah == 0 )
+            mat1.scale(_scale, _scale);
+	    if (w == 0 || h == 0)
 		return;
-	    double rx = (double)w / (double)aw;
-	    double ry = (double)h / (double)ah;
-            QWMatrix mat2 = QPixmap::trueMatrix( QWMatrix( rx, 0, 0, ry, 0, 0 )*mat1, aw, ah );
-	    QBitmap *wx_bm;
-#if 0
-            QString bm_key = gen_text_bitmap_key( mat2, dfont, str, pos, len );
-            wx_bm = get_text_bitmap( bm_key );
-            bool create_new_bm = wx_bm == 0;
-            if ( create_new_bm )
-#endif
-	    { 	        // no such cached bitmap
-                QBitmap bm( aw, ah, TRUE );     // create bitmap
-                QPainter paint;
-                paint.begin( &bm );             // draw text in bitmap
-		draw( &paint, 0, si->ascent, engine, si, textFlags );
-                paint.end();
-                wx_bm = new QBitmap( bm.xForm(mat2) ); // transform bitmap
-                if ( wx_bm->isNull() ) {
-                    delete wx_bm;               // nothing to draw
-                    return;
-                }
-            }
+            float tmp = _scale;
+            _scale = 1;
+            QWMatrix mat2 = QPixmap::trueMatrix(mat1, w, h);
+            QBitmap bm(w, h, TRUE);     // create bitmap
+            QPainter paint;
+            paint.begin( &bm );             // draw text in bitmap
+            draw( &paint, 0, qRound(si->ascent/tmp), engine, si, textFlags );
+            paint.end();
+            _scale = tmp;
+	    QBitmap wx_bm( bm.xForm(mat2) ); // transform bitmap
+            if ( wx_bm.isNull() )
+                return;
+
             double fx=x, fy=y - si->ascent, nfx, nfy;
-            mat1.map( fx,fy, &nfx,&nfy );
-            double tfx=tx, tfy=ty, dx, dy;
+            p->xmat.map( fx,fy, &nfx,&nfy );
+            double tfx=0, tfy=0, dx, dy;
             mat2.map( tfx, tfy, &dx, &dy );     // compute position of bitmap
             x = qRound(nfx-dx);
             y = qRound(nfy-dy);
+
             XSetFillStyle( dpy, gc, FillStippled );
-            XSetStipple( dpy, gc, wx_bm->handle() );
+            XSetStipple( dpy, gc, wx_bm.handle() );
             XSetTSOrigin( dpy, gc, x, y );
-            XFillRectangle( dpy, hd, gc, x, y, wx_bm->width(), wx_bm->height() );
+            XFillRectangle( dpy, hd, gc, x, y, wx_bm.width(), wx_bm.height() );
             XSetTSOrigin( dpy, gc, 0, 0 );
             XSetFillStyle( dpy, gc, FillSolid );
-#if 0
-	    if ( create_new_bm )
-                ins_text_bitmap( bm_key, wx_bm );
-#else
-	    delete wx_bm;
-#endif
             return;
         }
 	transform = TRUE;
@@ -590,7 +579,7 @@ void QFontEngineXLFD::draw( QPainter *p, int x, int y, const QTextEngine *engine
     glyph_metrics_t ci = boundingBox( glyphs, advances, offsets, si->num_glyphs );
     p->drawRect( x + ci.x, y + ci.y, ci.width, ci.height );
     p->drawRect( x + ci.x, y + 100 + ci.y, ci.width, ci.height );
-     qDebug("bounding rect=%d %d (%d/%d)", ci.x, ci.y, ci.width, ci.height );
+    qDebug("bounding rect=%d %d (%d/%d)", ci.x, ci.y, ci.width, ci.height );
     p->restore();
     int xp = x;
     int yp = y;
@@ -623,7 +612,8 @@ void QFontEngineXLFD::draw( QPainter *p, int x, int y, const QTextEngine *engine
 	    int yp = y+offsets[i].y-gi.yoff;
 	    if ( transform )
 		p->map( xp, yp, &xp, &yp );
-	    XDrawString16(dpy, hd, gc, xp, yp, chars+i, 1 );
+	    if (xp < SHRT_MAX && xp > SHRT_MIN)
+		XDrawString16(dpy, hd, gc, xp, yp, chars+i, 1 );
 	}
     } else {
 	if ( transform || si->hasPositioning ) {
@@ -633,7 +623,8 @@ void QFontEngineXLFD::draw( QPainter *p, int x, int y, const QTextEngine *engine
 		int yp = y+offsets[i].y;
 		if ( transform )
 		    p->map( xp, yp, &xp, &yp );
-		XDrawString16(dpy, hd, gc, xp, yp, chars+i, 1 );
+		if (xp < SHRT_MAX && xp > SHRT_MIN)
+		    XDrawString16(dpy, hd, gc, xp, yp, chars+i, 1 );
 		advance_t adv = advances[i];
 		// 	    qDebug("advance = %d/%d", adv.x, adv.y );
 		x += adv;
@@ -641,8 +632,17 @@ void QFontEngineXLFD::draw( QPainter *p, int x, int y, const QTextEngine *engine
 	    }
 	} else {
 	    // we can take a shortcut
-	    XDrawString16(dpy, hd, gc, x, y, chars, si->num_glyphs );
-	    x += si->width;
+	    int gl = 0;
+	    while (gl < si->num_glyphs) {
+		int toDraw = QMIN(64, si->num_glyphs-gl);
+		int adv = 0;
+		for (int i = gl; i < gl+toDraw; ++i)
+		    adv += advances[i];
+		if (x + adv < SHRT_MAX && x > SHRT_MIN)
+		    XDrawString16(dpy, hd, gc, x, y, chars+gl, toDraw);
+		gl += toDraw;
+		x += adv;
+	    }
 	}
     }
 
@@ -686,9 +686,9 @@ glyph_metrics_t QFontEngineXLFD::boundingBox( const glyph_t *glyphs, const advan
 	    overall.y = QMIN( overall.y, y );
 	    xmax = QMAX( xmax, overall.xoff + offsets[i].x + xcs->rbearing );
 	    ymax = QMAX( ymax, y + xcs->ascent + xcs->descent );
-	    overall.xoff += advances[i];
+	    overall.xoff += qRound(advances[i]/_scale);
 	} else {
-	    int size = ascent();
+	    int size = _fs->ascent;
 	    overall.x = QMIN(overall.x, overall.xoff );
 	    overall.y = QMIN(overall.y, overall.yoff - size );
 	    ymax = QMAX( ymax, overall.yoff );
@@ -718,7 +718,7 @@ glyph_metrics_t QFontEngineXLFD::boundingBox( glyph_t glyph )
     if (xcs) {
 	gm = glyph_metrics_t( xcs->lbearing, -xcs->ascent, xcs->rbearing- xcs->lbearing, xcs->ascent + xcs->descent, xcs->width, 0 );
     } else {
-	int size = ascent();
+	int size = _fs->ascent;
 	gm = glyph_metrics_t( 0, size, size, size, size, 0 );
     }
     if ( _scale != 1. ) {
@@ -1367,6 +1367,8 @@ static inline void unlockFTFace( XftFont *font )
 QFontEngineXft::QFontEngineXft( XftFont *font, XftPattern *pattern, int cmap )
     : _font( font ), _pattern( pattern ), _openType( 0 ), _cmap( cmap )
 {
+    _face = lockFTFace( _font );
+
 #ifndef QT_XFT2
     XftFontStruct *xftfs = getFontStruct( _font );
     if ( xftfs ) {
@@ -1376,9 +1378,21 @@ QFontEngineXft::QFontEngineXft( XftFont *font, XftPattern *pattern, int cmap )
 	// can convert from Unicode to glyph index
 	xftfs->charmap = -1;
     }
+#else
+    _cmap = -1;
+    // Xft maps Unicode and adobe roman for us.
+    for (int i = 0; i < _face->num_charmaps; ++i) {
+        FT_CharMap cm = _face->charmaps[i];
+//         qDebug("font has charmap %x", cm->encoding);
+        if (cm->encoding == ft_encoding_adobe_custom
+            || cm->encoding == ft_encoding_symbol) {
+//             qDebug("font has adobe custom or ms symbol charmap");
+            _cmap = i;
+            break;
+        }
+    }
 #endif // QT_XFT2
 
-    _face = lockFTFace( _font );
 
     cache_cost = _font->height * _font->max_advance_width *
 		 ( _face ? _face->num_glyphs : 1024 );
@@ -1416,6 +1430,17 @@ QFontEngineXft::~QFontEngineXft()
     }
 }
 
+#ifdef QT_XFT2
+static glyph_t getAdobeCharIndex(XftFont *font, int cmap, uint ucs4)
+{
+    FT_Face _face = XftLockFace( font );
+    FT_Set_Charmap(_face, _face->charmaps[cmap]);
+    glyph_t g = FT_Get_Char_Index(_face, ucs4);
+    XftUnlockFace(font);
+    return g;
+}
+#endif
+
 QFontEngine::Error QFontEngineXft::stringToCMap( const QChar *str, int len, glyph_t *glyphs, advance_t *advances, int *nglyphs, bool mirrored ) const
 {
     if ( *nglyphs < len ) {
@@ -1424,12 +1449,27 @@ QFontEngine::Error QFontEngineXft::stringToCMap( const QChar *str, int len, glyp
     }
 
 #ifdef QT_XFT2
-    if ( mirrored ) {
+    if (_cmap != -1) {
+	for ( int i = 0; i < len; ++i ) {
+	    unsigned short uc = str[i].unicode();
+	    glyphs[i] = uc < cmapCacheSize ? cmapCache[uc] : 0;
+	    if ( !glyphs[i] ) {
+		glyph_t glyph = XftCharIndex(0, _font, uc);
+                if (!glyph)
+                    glyph = getAdobeCharIndex(_font, _cmap, uc);
+		glyphs[i] = glyph;
+		if ( uc < cmapCacheSize )
+		    ((QFontEngineXft *)this)->cmapCache[uc] = glyph;
+	    }
+	}
+    } else if ( mirrored ) {
 	for ( int i = 0; i < len; ++i ) {
 	    unsigned short uc = ::mirroredChar(str[i]).unicode();
 	    glyphs[i] = uc < cmapCacheSize ? cmapCache[uc] : 0;
 	    if ( !glyphs[i] ) {
-		glyph_t glyph = XftCharIndex( QPaintDevice::x11AppDisplay(), _font, uc );
+		if (uc == 0xa0)
+		    uc = 0x20;
+		glyph_t glyph = XftCharIndex(0, _font, uc);
 		glyphs[i] = glyph;
 		if ( uc < cmapCacheSize )
 		    ((QFontEngineXft *)this)->cmapCache[uc] = glyph;
@@ -1440,7 +1480,9 @@ QFontEngine::Error QFontEngineXft::stringToCMap( const QChar *str, int len, glyp
 	    unsigned short uc = str[i].unicode();
 	    glyphs[i] = uc < cmapCacheSize ? cmapCache[uc] : 0;
 	    if ( !glyphs[i] ) {
-		glyph_t glyph = XftCharIndex( QPaintDevice::x11AppDisplay(), _font, uc );
+		if (uc == 0xa0)
+		    uc = 0x20;
+		glyph_t glyph = XftCharIndex(0, _font, uc);
 		glyphs[i] = glyph;
 		if ( uc < cmapCacheSize )
 		    ((QFontEngineXft *)this)->cmapCache[uc] = glyph;
@@ -1456,7 +1498,7 @@ QFontEngine::Error QFontEngineXft::stringToCMap( const QChar *str, int len, glyp
 		XGlyphInfo gi;
 		XftGlyphExtents( QPaintDevice::x11AppDisplay(), _font, &glyph, 1, &gi );
 		advances[i] = gi.xOff;
-		if ( glyph < widthCacheSize && gi.xOff < 0x100 )
+		if ( glyph < widthCacheSize && gi.xOff > 0 && gi.xOff < 0x100 )
 		    ((QFontEngineXft *)this)->widthCache[glyph] = gi.xOff;
 	    }
 	}
@@ -1522,7 +1564,7 @@ QFontEngine::Error QFontEngineXft::stringToCMap( const QChar *str, int len, glyp
 		XGlyphInfo gi;
 		XftTextExtents16(QPaintDevice::x11AppDisplay(), _font, &glyph, 1, &gi);
 		advances[i] = gi.xOff;
-		if ( glyph < widthCacheSize && gi.xOff < 0x100 )
+		if ( glyph < widthCacheSize && gi.xOff > 0 && gi.xOff < 0x100 )
 		    ((QFontEngineXft *)this)->widthCache[glyph] = gi.xOff;
 	    }
 	}
@@ -1549,7 +1591,7 @@ void QFontEngineXft::recalcAdvances( int len, glyph_t *glyphs, advance_t *advanc
 	    XGlyphInfo gi;
 	    XftGlyphExtents( QPaintDevice::x11AppDisplay(), _font, &glyph, 1, &gi );
 	    advances[i] = gi.xOff;
-	    if ( glyph < widthCacheSize && gi.xOff < 0x100 )
+	    if ( glyph < widthCacheSize && gi.xOff > 0 && gi.xOff < 0x100 )
 		((QFontEngineXft *)this)->widthCache[glyph] = gi.xOff;
 	}
 	if ( _scale != 1. ) {
@@ -1565,7 +1607,7 @@ void QFontEngineXft::recalcAdvances( int len, glyph_t *glyphs, advance_t *advanc
 	    XGlyphInfo gi;
 	    XftTextExtents16(QPaintDevice::x11AppDisplay(), _font, &glyph, 1, &gi);
 	    advances[i] = gi.xOff;
-	    if ( glyph < widthCacheSize && gi.xOff < 0x100 )
+	    if ( glyph < widthCacheSize && gi.xOff > 0 && gi.xOff < 0x100 )
 		((QFontEngineXft *)this)->widthCache[glyph] = gi.xOff;
 	}
     }
@@ -1590,8 +1632,8 @@ void QFontEngineXft::draw( QPainter *p, int x, int y, const QTextEngine *engine,
 
     XftFont *fnt = _font;
     bool transform = FALSE;
-    if ( p->txop >= QPainter::TxScale ) {
-	if (! (_face->face_flags & FT_FACE_FLAG_SCALABLE)) {
+    if ( p->txop >= QPainter::TxScale || p->rop != Qt::CopyROP) {
+	if (!(_face->face_flags & FT_FACE_FLAG_SCALABLE) || p->rop != Qt::CopyROP) {
 	    Qt::HANDLE hd = p->device()->handle();
 	    GC gc = p->gc;
 
@@ -1635,9 +1677,8 @@ void QFontEngineXft::draw( QPainter *p, int x, int y, const QTextEngine *engine,
 	    return;
 	}
 
-	XftPattern *pattern = XftPatternDuplicate( _pattern );
 	XftMatrix *mat = 0;
-	XftPatternGetMatrix( pattern, XFT_MATRIX, 0, &mat );
+	XftPatternGetMatrix( _pattern, XFT_MATRIX, 0, &mat );
 	XftMatrix m2;
 	m2.xx = p->m11()*_scale;
 	m2.xy = -p->m21()*_scale;
@@ -1677,6 +1718,7 @@ void QFontEngineXft::draw( QPainter *p, int x, int y, const QTextEngine *engine,
 	    if ( mat )
 		XftMatrixMultiply( &m2, &m2, mat );
 
+            XftPattern *pattern = XftPatternDuplicate( _pattern );
 	    XftPatternDel( pattern, XFT_MATRIX );
 	    XftPatternAddMatrix( pattern, XFT_MATRIX, &m2 );
 
@@ -1748,9 +1790,11 @@ void QFontEngineXft::draw( QPainter *p, int x, int y, const QTextEngine *engine,
 		    p->map( xp, yp, &xp, &yp );
 #ifdef QT_XFT2
 		FT_UInt glyph = *(glyphs + i);
-		XftDrawGlyphs( draw, &col, fnt, xp, yp, &glyph, 1 );
+		if (xp < SHRT_MAX && xp > SHRT_MIN)
+		    XftDrawGlyphs( draw, &col, fnt, xp, yp, &glyph, 1 );
 #else
-		XftDrawString16( draw, &col, fnt, xp, yp, (XftChar16 *) (glyphs+i), 1);
+		if (xp < SHRT_MAX && xp > SHRT_MIN)
+		    XftDrawString16( draw, &col, fnt, xp, yp, (XftChar16 *) (glyphs+i), 1);
 #endif // QT_XFT2
 #ifdef FONTENGINE_DEBUG
 		glyph_metrics_t gi = boundingBox( glyphs[i] );
@@ -1771,9 +1815,11 @@ void QFontEngineXft::draw( QPainter *p, int x, int y, const QTextEngine *engine,
 		    p->map( xp, yp, &xp, &yp );
 #ifdef QT_XFT2
 		FT_UInt glyph = *(glyphs + i);
-		XftDrawGlyphs( draw, &col, fnt, xp, yp, &glyph, 1 );
+		if (xp < SHRT_MAX && xp > SHRT_MIN)
+		    XftDrawGlyphs( draw, &col, fnt, xp, yp, &glyph, 1 );
 #else
-		XftDrawString16( draw, &col, fnt, xp, yp, (XftChar16 *) (glyphs+i), 1 );
+		if (xp < SHRT_MAX && xp > SHRT_MIN)
+		    XftDrawString16( draw, &col, fnt, xp, yp, (XftChar16 *) (glyphs+i), 1 );
 #endif // QT_XFT2
 		// 	    qDebug("advance = %d/%d", adv.x, adv.y );
 		x += advances[i];
@@ -1785,32 +1831,49 @@ void QFontEngineXft::draw( QPainter *p, int x, int y, const QTextEngine *engine,
 	// Drawing them as one string increases performance significantly.
 #ifdef QT_XFT2
 	// #### we should use a different method anyways on Xft2
-	FT_UInt g[128];
-	FT_UInt *gl = g;
-	if ( si->num_glyphs > 128 )
-	    gl = new FT_UInt[si->num_glyphs];
-	if ( si->analysis.bidiLevel % 2 )
-	    for ( int i = 0; i < si->num_glyphs; i++ )
-		gl[i] = glyphs[si->num_glyphs-1-i];
-	else
-	    for ( int i = 0; i < si->num_glyphs; i++ )
-		gl[i] = glyphs[i];
-	XftDrawGlyphs( draw, &col, fnt, x, y, gl, si->num_glyphs );
-	if ( gl != g )
-	    delete [] gl;
+	FT_UInt g[64];
+	int gl = 0;
+	while (gl < si->num_glyphs) {
+	    int toDraw = QMIN(64, si->num_glyphs-gl);
+	    int adv = 0;
+	    if ( si->analysis.bidiLevel % 2 ) {
+		for ( int i = 0; i < toDraw; i++ ) {
+		    g[i] = glyphs[si->num_glyphs-1-(gl+i)];
+		    adv += advances[si->num_glyphs-1-(gl+i)];
+		}
+	    } else {
+		for ( int i = 0; i < toDraw; i++ ) {
+		    g[i] = glyphs[gl+i];
+		    adv += advances[gl+i];
+		}
+	    }
+	    if (x + adv < SHRT_MAX && x > SHRT_MIN)
+		XftDrawGlyphs( draw, &col, fnt, x, y, g, toDraw );
+	    gl += toDraw;
+	    x += adv;
+	}
 #else
-	XftChar16 g[128];
-	XftChar16 *gl = (XftChar16 *)glyphs;
-	if ( si->analysis.bidiLevel % 2 ) {
-	    gl = g;
-	    if ( si->num_glyphs > 128 )
-		gl = new XftChar16[si->num_glyphs];
-	    for ( int i = 0; i < si->num_glyphs; i++ )
-		gl[i] = glyphs[si->num_glyphs-1-i];
+	XftChar16 g[64];
+	int gl = 0;
+	while (gl < si->num_glyphs) {
+	    int toDraw = QMIN(64, si->num_glyphs-gl);
+	    int adv = 0;
+	    if ( si->analysis.bidiLevel % 2 ) {
+		for ( int i = 0; i < toDraw; i++ ) {
+		    g[i] = glyphs[si->num_glyphs-1-(gl+i)];
+		    adv += advances[si->num_glyphs-1-(gl+i)];
+		}
+	    } else {
+		for ( int i = 0; i < toDraw; i++ ) {
+		    g[i] = glyphs[gl+i];
+		    adv += advances[gl+i];
+		}
+	    }
+	    if (x + adv < SHRT_MAX && x > SHRT_MIN)
+		XftDrawString16( draw, &col, fnt, x, y, g, toDraw );
+	    gl += toDraw;
+	    x += adv;
 	}
-	XftDrawString16( draw, &col, fnt, x, y, gl, si->num_glyphs );
-	if ( gl != (XftChar16 *)glyphs && gl != g )
-	    delete [] gl;
 #endif // QT_XFT2
     }
 
@@ -1839,27 +1902,37 @@ glyph_metrics_t QFontEngineXft::boundingBox( const glyph_t *glyphs, const advanc
     glyph_metrics_t overall;
     int ymax = 0;
     int xmax = 0;
-    for (int i = 0; i < numGlyphs; i++) {
-	getGlyphInfo( &xgi, _font, glyphs[i] );
-	int x = overall.xoff + offsets[i].x - xgi.x;
-	int y = overall.yoff + offsets[i].y - xgi.y;
-	overall.x = QMIN( overall.x, x );
-	overall.y = QMIN( overall.y, y );
-	xmax = QMAX( xmax, x + xgi.width );
-	ymax = QMAX( ymax, y + xgi.height );
-	overall.xoff += advances[i];
+    if (_scale != 1) {
+        for (int i = 0; i < numGlyphs; i++) {
+            getGlyphInfo( &xgi, _font, glyphs[i] );
+            int x = overall.xoff + offsets[i].x - xgi.x;
+            int y = overall.yoff + offsets[i].y - xgi.y;
+            overall.x = QMIN( overall.x, x );
+            overall.y = QMIN( overall.y, y );
+            xmax = QMAX( xmax, x + xgi.width );
+            ymax = QMAX( ymax, y + xgi.height );
+            overall.xoff += qRound(advances[i]/_scale);
+        }
+	overall.x = qRound(overall.x * _scale);
+	overall.y = qRound(overall.y * _scale);
+	overall.xoff = qRound(overall.xoff * _scale);
+	overall.yoff = qRound(overall.yoff * _scale);
+	xmax = qRound(xmax * _scale);
+	ymax = qRound(ymax * _scale);
+    } else {
+        for (int i = 0; i < numGlyphs; i++) {
+            getGlyphInfo( &xgi, _font, glyphs[i] );
+            int x = overall.xoff + offsets[i].x - xgi.x;
+            int y = overall.yoff + offsets[i].y - xgi.y;
+            overall.x = QMIN( overall.x, x );
+            overall.y = QMIN( overall.y, y );
+            xmax = QMAX( xmax, x + xgi.width );
+            ymax = QMAX( ymax, y + xgi.height );
+            overall.xoff += advances[i];
+        }
     }
     overall.height = ymax - overall.y;
     overall.width = xmax - overall.x;
-
-    if ( _scale != 1. ) {
-	overall.x = qRound(overall.x * _scale);
-	overall.y = qRound(overall.y * _scale);
-	overall.height = qRound(overall.height * _scale);
-	overall.width = qRound(overall.width * _scale);
-	overall.xoff = qRound(overall.xoff * _scale);
-	overall.yoff = qRound(overall.yoff * _scale);
-    }
     return overall;
 }
 
@@ -1925,16 +1998,59 @@ int QFontEngineXft::maxCharWidth() const
     return qRound(_font->max_advance_width*_scale);
 }
 
+static const ushort char_table[] = {
+	40,
+	67,
+	70,
+	75,
+	86,
+	88,
+	89,
+	91,
+	102,
+	114,
+	124,
+	127,
+	205,
+	645,
+	884,
+	922,
+	1070,
+	12386
+};
+
+static const int char_table_entries = sizeof(char_table)/sizeof(ushort);
+
+
 int QFontEngineXft::minLeftBearing() const
 {
-    // ### fix for Xft2
-    return 0;
+    if ( lbearing == SHRT_MIN )
+	minRightBearing(); // calculates both
+
+    return lbearing;
 }
 
 int QFontEngineXft::minRightBearing() const
 {
-    // ### fix for Xft2
-    return 0;
+    if ( rbearing == SHRT_MIN ) {
+	QFontEngineXft *that = (QFontEngineXft *)this;
+	that->lbearing = that->rbearing = 0;
+	QChar *ch = (QChar *)char_table;
+	glyph_t glyphs[char_table_entries];
+	int ng = char_table_entries;
+	stringToCMap(ch, char_table_entries, glyphs, 0, &ng, FALSE);
+	while (--ng) {
+	    if (glyphs[ng]) {
+		glyph_metrics_t gi = that->boundingBox( glyphs[ng] );
+		if (gi.xoff) {
+		    that->lbearing = QMIN(lbearing, gi.x);
+		    that->rbearing = QMIN(rbearing, gi.xoff - gi.x - gi.width);
+		}
+	    }
+	}
+    }
+
+    return rbearing;
 }
 
 int QFontEngineXft::cmap() const
@@ -1957,20 +2073,29 @@ bool QFontEngineXft::canRender( const QChar *string, int len )
     bool allExist = TRUE;
 
 #ifdef QT_XFT2
-    for ( int i = 0; i < len; i++ ) {
-	if ( ! XftCharExists( QPaintDevice::x11AppDisplay(), _font,
-			      string[i].unicode() ) ) {
-	    allExist = FALSE;
-	    break;
-	}
+    if (_cmap != -1) {
+        for ( int i = 0; i < len; i++ ) {
+            if (!XftCharExists(0, _font, string[i].unicode())
+                && getAdobeCharIndex(_font, _cmap, string[i].unicode()) == 0) {
+                allExist = FALSE;
+                break;
+            }
+        }
+    } else {
+        for ( int i = 0; i < len; i++ ) {
+            if (!XftCharExists(0, _font, string[i].unicode())) {
+                allExist = FALSE;
+                break;
+            }
+        }
     }
 #else
     glyph_t glyphs[256];
     int nglyphs = 255;
     glyph_t *g = glyphs;
-    if ( stringToCMap( string, len, g, 0, &nglyphs, false ) == OutOfMemory ) {
+    if ( stringToCMap( string, len, g, 0, &nglyphs, FALSE ) == OutOfMemory ) {
 	g = (glyph_t *)malloc( nglyphs*sizeof(glyph_t) );
-	stringToCMap( string, len, g, 0, &nglyphs, false );
+	stringToCMap( string, len, g, 0, &nglyphs, FALSE );
     }
 
     for ( int i = 0; i < nglyphs; i++ ) {
@@ -1993,11 +2118,11 @@ QOpenType *QFontEngineXft::openType() const
     if ( _openType )
 	return _openType;
 
-    if ( ! FT_IS_SFNT( _face ) )
+    if ( !_face || ! FT_IS_SFNT( _face ) )
 	return 0;
 
     QFontEngineXft *that = (QFontEngineXft *)this;
-    that->_openType = new QOpenType( that->_face );
+    that->_openType = new QOpenType(that);
     return _openType;
 }
 
@@ -2150,10 +2275,10 @@ static const unsigned int supported_scripts [] = {
 //     FT_MAKE_TAG( 'j', 'a', 'm', 'o' ),
 };
 
-
-QOpenType::QOpenType( FT_Face _face )
-    : face( _face ), gdef( 0 ), gsub( 0 ), gpos( 0 ), current_script( 0 )
+QOpenType::QOpenType(QFontEngineXft *fe)
+    : xft(fe), gdef( 0 ), gsub( 0 ), gpos( 0 ), current_script( 0 )
 {
+    face = xft->face();
     hasGDef = hasGSub = hasGPos = TRUE;
     str = tmp = 0;
     positions = 0;
@@ -2193,7 +2318,7 @@ bool QOpenType::supportsScript( unsigned int script )
     FT_Error error;
     if ( !gdef ) {
 	if ( (error = TT_Load_GDEF_Table( face, &gdef )) ) {
-// 	    qDebug("error loading gdef table: %d", error );
+//  	    qDebug("error loading gdef table: %d", error );
 	    hasGDef = FALSE;
 	}
     }
@@ -2201,10 +2326,10 @@ bool QOpenType::supportsScript( unsigned int script )
     if ( !gsub ) {
 	if ( (error = TT_Load_GSUB_Table( face, &gsub, gdef )) ) {
 	    if ( error != FT_Err_Table_Missing ) {
-// 		qDebug("error loading gsub table: %d", error );
+//  		qDebug("error loading gsub table: %d", error );
 		return FALSE;
 	    } else {
-// 		qDebug("face doesn't have a gsub table" );
+//  		qDebug("face doesn't have a gsub table" );
 		hasGSub = FALSE;
 	    }
 	}
@@ -2212,13 +2337,8 @@ bool QOpenType::supportsScript( unsigned int script )
 
     if ( !gpos ) {
 	if ( (error = TT_Load_GPOS_Table( face, &gpos, gdef )) ) {
-	    if ( error != FT_Err_Table_Missing ) {
-// 		qDebug("error loading gpos table: %d", error );
-		return FALSE;
-	    } else {
-// 		qDebug("face doesn't have a gpos table" );
-		hasGPos = FALSE;
-	    }
+//  		qDebug("error loading gpos table: %d", error );
+	    hasGPos = FALSE;
 	}
     }
 
@@ -2254,9 +2374,8 @@ bool QOpenType::loadTables( FT_ULong script)
     qDebug("script %s has script index %d", tag_to_string(script), script_index );
 #endif
 
-    TTO_FeatureList featurelist = gsub->FeatureList;
-
 #ifdef OT_DEBUG
+    TTO_FeatureList featurelist = gsub->FeatureList;
     int numfeatures = featurelist.FeatureCount;
     qDebug("gsub table has %d features", numfeatures );
     for( int i = 0; i < numfeatures; i++ ) {
@@ -2299,7 +2418,7 @@ bool QOpenType::loadTables( FT_ULong script)
 }
 
 void QOpenType::init(glyph_t *glyphs, GlyphAttributes *glyphAttributes, int num_glyphs,
-		     unsigned short *logClusters, int len, int char_offset)
+		     unsigned short *logClusters, int len, int /*char_offset*/)
 {
     if ( !str )
 	TT_GSUB_String_New(&str);
@@ -2313,23 +2432,20 @@ void QOpenType::init(glyph_t *glyphs, GlyphAttributes *glyphAttributes, int num_
 
     length = len;
 
+    Q_ASSERT(len == num_glyphs);
     memcpy(str->string, glyphs, num_glyphs*sizeof(unsigned short));
-    // map from log clusters.
-    int i = length - 1;
-    int j = num_glyphs;
-    while (j--) {
-	if (logClusters[i] > j)
-	    --i;
-	str->character_index[j] = i + char_offset;
-    }
+
+    for (int i = 0; i < num_glyphs; ++i)
+	str->character_index[i] = i;
+
 #ifdef OT_DEBUG
     qDebug("-----------------------------------------");
-    qDebug("log clusters before shaping: char_offset = %d",  char_offset);
+    qDebug("log clusters before shaping:");
     for (int j = 0; j < length; j++)
 	qDebug("    log[%d] = %d", j, logClusters[j] );
     qDebug("original glyphs:");
-    for (i = 0; i < num_glyphs; ++i)
-	qDebug("   glyph=%4x char_index=%d", str->string[i], str->character_index[i]);
+    for (int i = 0; i < num_glyphs; ++i)
+	qDebug("   glyph=%4x char_index=%d mark: %d cmb: %d", str->string[i], str->character_index[i], glyphAttributes[i].mark, glyphAttributes[i].combiningClass);
 #endif
     str->length = num_glyphs;
     orig_nglyphs = num_glyphs;
@@ -2362,16 +2478,12 @@ void QOpenType::applyGSUBFeature(unsigned int featureTag, bool *where)
 
     memset(where_to_apply, 1, str->length);
     if (where) {
-	int j = str->length-1;
-	for (int i = orig_nglyphs-1; i >= 0; --i) {
-	    if (str->character_index[j] > i)
-		--j;
-	    if (!where[i])
-		where_to_apply[j] = 0;
+	for (uint i = 0; i < str->length; ++i) {
+	    where_to_apply[i] = where[str->character_index[i]];
 	}
 #ifdef OT_DEBUG
-	for (int i = 0; i < (int)str->length; ++i)
-	    qDebug("   apply=%s", where_to_apply[i] ? "true" : "false");
+// 	for (int i = 0; i < (int)str->length; ++i)
+// 	    qDebug("   apply=%s", where_to_apply[i] ? "true" : "false");
 #endif
     }
 
@@ -2413,8 +2525,15 @@ void QOpenType::applyGPOSFeatures()
 	    if (error != FT_Err_Ok)
 		continue;
 
-	    // ### is FT_LOAD_DEFAULT the right thing to do?
-	    TT_GPOS_Apply_Feature( face, gpos, feature_index, FT_LOAD_DEFAULT, str, &positions, FALSE, false );
+#ifdef OT_DEBUG
+	    qDebug("applying POS feature %s with index %d", tag_to_string( r->FeatureTag ), feature_index);
+#endif
+
+	    str->pos = 0;
+	    // ensure the face has the correct size and matrix set
+	    face = lockFTFace(xft->_font);
+	    TT_GPOS_Apply_Feature( face, gpos, feature_index, FT_LOAD_DEFAULT, str, &positions, FALSE, FALSE );
+	    unlockFTFace(xft->_font);
 	}
     }
     positioned = TRUE;
@@ -2447,9 +2566,9 @@ void QOpenType::appendTo(QTextEngine *engine, QScriptItem *si, bool doLogCluster
 	int oldCi = 0;
 	for ( int i = 0; i < (int)str->length; i++ ) {
 	    int ci = str->character_index[i];
-	    glyphAttributes[i] = tmpAttributes[tmpLogClusters[ci]];
-	    // 	qDebug("   ci[%d] = %d mark=%d, cs=%d tmplc=%d",
-	    // 	       i, ci, glyphAttributes[i].mark, glyphAttributes[i].clusterStart,  tmpLogClusters[ci]);
+	    glyphAttributes[i] = tmpAttributes[ci];
+	    // 	qDebug("   ci[%d] = %d mark=%d, cmb=%d, cs=%d tmplc=%d",
+	    // 	       i, ci, glyphAttributes[i].mark, glyphAttributes[i].combiningClass, glyphAttributes[i].clusterStart,  tmpLogClusters[ci]);
 	    if ( !glyphAttributes[i].mark && glyphAttributes[i].clusterStart && ci != oldCi ) {
 		for ( int j = oldCi; j < ci; j++ )
 		    logClusters[j] = clusterStart;
@@ -2467,13 +2586,14 @@ void QOpenType::appendTo(QTextEngine *engine, QScriptItem *si, bool doLogCluster
     for ( int i = 0; i < (int)str->length; i++ ) {
 	if ( glyphAttributes[i].mark )
 	    advances[i] = 0;
-// 	    qDebug("   adv=%d", advances[i]);
+// 	    qDebug("   adv=%d (mark=%d)", advances[i], glyphAttributes[i].mark);
     }
+    si->num_glyphs += str->length;
 
     // positioning code:
     if ( hasGPos && positioned) {
 	float scale = si->fontEngine->scale();
-// 	qDebug("positioned glyphs:" );
+//   	qDebug("positioned glyphs:" );
 	for ( int i = 0; i < (int)str->length; i++) {
 // 	    qDebug("    %d:\t orig advance: %d\tadv=(%d/%d)\tpos=(%d/%d)\tback=%d\tnew_advance=%d", i,
 // 		   advances[i], (int)(positions[i].x_advance >> 6), (int)(positions[i].y_advance >> 6 ),
@@ -2481,10 +2601,14 @@ void QOpenType::appendTo(QTextEngine *engine, QScriptItem *si, bool doLogCluster
 // 		   positions[i].back, positions[i].new_advance );
 	    // ###### fix the case where we have y advances. How do we handle this in Uniscribe?????
 	    if ( positions[i].new_advance ) {
-		advances[i] = qRound((positions[i].x_advance >> 6)*scale);
+		advances[i] = si->analysis.bidiLevel % 2 ?
+			      -qRound((positions[i].x_advance >> 6)*scale) :
+			      qRound((positions[i].x_advance >> 6)*scale);
 		//advances[i].y = -positions[i].y_advance >> 6;
 	    } else {
-		advances[i] += qRound((positions[i].x_advance >> 6)*scale);
+		advances[i] += si->analysis.bidiLevel % 2 ?
+			      -qRound((positions[i].x_advance >> 6)*scale) :
+			      qRound((positions[i].x_advance >> 6)*scale);
 		//advances[i].y -= positions[i].y_advance >> 6;
 	    }
 	    offsets[i].x = qRound((positions[i].x_pos >> 6)*scale);
@@ -2513,13 +2637,11 @@ void QOpenType::appendTo(QTextEngine *engine, QScriptItem *si, bool doLogCluster
     }
     qDebug("final glyphs:");
     for (int i = 0; i < (int)str->length; ++i)
-	qDebug("   glyph=%4x char_index=%d mark: %d, clusterStart: %d width=%d",
-	       glyphs[i], str->character_index[i], glyphAttributes[i].mark, glyphAttributes[i].clusterStart,
+	qDebug("   glyph=%4x char_index=%d mark: %d cmp: %d, clusterStart: %d width=%d",
+	       glyphs[i], str->character_index[i], glyphAttributes[i].mark, glyphAttributes[i].combiningClass, glyphAttributes[i].clusterStart,
 	       advances[i]);
     qDebug("-----------------------------------------");
 #endif
-
-    si->num_glyphs += str->length;
 }
 
 #endif
