@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qlayout.cpp   3.2.3   edited Jun 11 18:55 $
+** $Id: qt/qlayout.cpp   3.3.4   edited Jan 3 19:37 $
 **
 ** Implementation of layout classes
 **
@@ -555,12 +555,10 @@ static void distributeMultiBox( QMemArray<QLayoutStruct> &chain, int spacing,
     int w = 0;
     int wh = 0;
     int max = 0;
-    bool exp = FALSE;
     for ( i = start; i <= end; i++ ) {
 	w += chain[i].minimumSize;
 	wh += chain[i].sizeHint;
 	max += chain[i].maximumSize;
-	exp = exp || chain[i].expansive;
 	chain[i].empty = FALSE;
 	if ( stretchArray[i] == 0 )
 	    chain[i].stretch = QMAX(chain[i].stretch,stretch);
@@ -856,17 +854,17 @@ public:
     QLayoutItem *takeCurrent() {
 	QLayoutItem *item = 0;
 	if ( multi ) {
+	     if ( !data->multi || idx >= (int)data->multi->count() )
+		return 0;
 	    QGridMultiBox *b = data->multi->take( idx );
-	    if ( b ) {
-		item = b->takeItem();
-		delete b;
-	    }
+	    item = b->takeItem();
+	    delete b;
 	} else {
+	    if ( idx >= (int)data->things.count() )
+		return 0;
 	    QGridBox *b = data->things.take( idx );
-	    if ( b ) {
-		item = b->takeItem();
-		delete b;
-	    }
+	    item = b->takeItem();
+	    delete b;
 	}
 	return item;
     }
@@ -1101,7 +1099,7 @@ int QGridLayout::minimumHeightForWidth( int w ) const
 
 /*!
     Searches for widget \a w in this layout (not including child
-    layouts). If \a w is found, it sets \c *\a row and \c *\a col to
+    layouts). If \a w is found, it sets \c \a row and \c \a col to
     the row and column and returns TRUE; otherwise returns FALSE.
 
     Note: if a widget spans multiple rows/columns, the top-left cell
@@ -1302,6 +1300,10 @@ void QGridLayout::addMultiCellWidget( QWidget *w, int fromRow, int toRow,
 
     \a layout becomes a child of the grid layout.
 
+    When a layout is constructed with another layout as its parent,
+    you don't need to call addLayout(); the child layout is
+    automatically added to the parent layout as it is constructed.
+
     \sa addMultiCellLayout()
 */
 void QGridLayout::addLayout( QLayout *layout, int row, int col )
@@ -2151,6 +2153,10 @@ void QBoxLayout::addWidget( QWidget *widget, int stretch,
     Adds \a layout to the end of the box, with serial stretch factor
     \a stretch.
 
+    When a layout is constructed with another layout as its parent,
+    you don't need to call addLayout(); the child layout is
+    automatically added to the parent layout as it is constructed.
+
     \sa insertLayout(), setAutoAdd(), addWidget(), addSpacing()
 */
 void QBoxLayout::addLayout( QLayout *layout, int stretch )
