@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qmime.cpp   3.3.4   edited Apr 1 2004 $
 **
 ** Implementation of MIME support
 **
@@ -106,20 +106,11 @@ void QMimeSource::clearCache()
 */
 QMimeSource::~QMimeSource()
 {
+#ifndef QT_NO_CLIPBOARD
+    extern void qt_clipboard_cleanup_mime_source(QMimeSource *);
+    qt_clipboard_cleanup_mime_source(this);
+#endif
     clearCache();
-
-#if defined(Q_WS_X11)
-    if (QApplication::closingDown()) return;
-
-#ifndef QT_NO_MIMECLIPBOARD
-    if (QApplication::clipboard()->data() == this) {
-#ifdef QT_CHECK_RANGE
-	qWarning("QMimeSource::~QMimeSource: clipboard data deleted!");
-#endif
-	QApplication::clipboard()->clobber();
-    }
-#endif
-#endif // QT_NO_CLIPBOARD
 }
 
 /*!
@@ -233,12 +224,20 @@ public:
     QMimeSourceFactory::defaultFactory()->setImage( "myimage", QImage(myimage_data) );
     \endcode
 
-    Now you can create a rich text QLabel with:
+    Now you can create a rich text QLabel with
+
     \code
     QLabel* label = new QLabel(
 	"Rich text with embedded image:<img source=\"myimage\">"
 	"Isn't that <em>cute</em>?" );
     \endcode
+
+    When no longer needed, you can clear the data from the factory:
+
+    \code
+    delete label;
+    QMimeSourceFactory::defaultFactory()->setData( "myimage", 0 );
+    \endcode
 */
 
 
@@ -283,12 +282,13 @@ QMimeSource* QMimeSourceFactory::dataInternal(const QString& abs_name, const QMa
 	    mimetype = QCString("image/")+QCString(imgfmt).lower();
 
 	QFile f(abs_name);
-	if ( f.open(IO_ReadOnly) ) {
+	if ( f.open(IO_ReadOnly) && f.size() ) {
 	    QByteArray ba(f.size());
 	    f.readBlock(ba.data(), ba.size());
 	    QStoredDrag* sr = new QStoredDrag( mimetype );
 	    sr->setEncodedData( ba );
-	    r = sr;
+	    delete d->last;
+	    d->last = r = sr;
 	}
     }
 
@@ -325,16 +325,19 @@ QMimeSource* QMimeSourceFactory::dataInternal(const QString& abs_name, const QMa
     setExtensionType("txt", "text/plain");
     setExtensionType("xml", "text/xml;charset=UTF-8");
     \endcode
-    The effect of these is that file names ending in "html" or "htm"
-    will be treated as text encoded in the iso8859-1 encoding, those
-    ending in "txt" will be treated as text encoded in the local
-    encoding; those ending in "xml" will be treated as text encoded in
-    Unicode UTF-8 encoding. The text subtype ("html", "plain", or
-    "xml") does not affect the factory, but users of the factory may
-    behave differently. We recommend creating "xml" files where
-    practical. These files can be viewed regardless of the runtime
-    encoding and can encode any Unicode characters without resorting
-    to encoding definitions inside the file.
+    The effect of these is that file names ending in "txt" will be
+    treated as text encoded in the local encoding; those ending in
+    "xml" will be treated as text encoded in Unicode UTF-8 encoding.
+    The text/html type is treated specially, since the encoding can be
+    specified in the html file itself. "html" or "htm" will be treated
+    as text encoded in the encoding specified by the html meta tag, if
+    none could be found, the charset of the mime type will be used.
+    The text subtype ("html", "plain", or "xml") does not affect the
+    factory, but users of the factory may behave differently. We
+    recommend creating "xml" files where practical. These files can be
+    viewed regardless of the runtime encoding and can encode any
+    Unicode characters without resorting to encoding definitions
+    inside the file.
 
     Any file data that is not recognized will be retrieved as a
     QMimeSource providing the "application/octet-stream" mime type,
@@ -357,7 +360,7 @@ const QMimeSource* QMimeSourceFactory::data(const QString& abs_name) const
     QStringList::Iterator it;
     if ( abs_name[0] == '/'
 #ifdef Q_WS_WIN
-	    || abs_name[0] && abs_name[1] == ':'
+	    || ( abs_name[0] && abs_name[1] == ':' ) || abs_name.startsWith("\\\\")
 #endif
     )
     {
@@ -404,9 +407,6 @@ const QMimeSource* QMimeSourceFactory::data(const QString& abs_name) const
 	r = (QMimeSource*)defaultFactory()->data( abs_name );
     }
 
-
-    delete d->last;
-    d->last = r;
     return r;
 }
 
@@ -426,7 +426,7 @@ void QMimeSourceFactory::setFilePath( const QStringList& path )
 */
 QStringList QMimeSourceFactory::filePath() const
 {
-  return d->path;
+    return d->path;
 }
 
 /*!
@@ -436,7 +436,7 @@ QStringList QMimeSourceFactory::filePath() const
 */
 void QMimeSourceFactory::addFilePath( const QString& p )
 {
-  d->path += p;
+    d->path += p;
 }
 
 /*!
@@ -467,8 +467,14 @@ QString QMimeSourceFactory::makeAbsolute(const QString& abs_or_rel_name, const Q
     if ( abs_or_rel_name.isEmpty() )
 	return context;
     QFileInfo c( context );
-    QFileInfo r( c.dir(TRUE), abs_or_rel_name );
-    return r.absFilePath();
+    if (!c.isDir()) {
+	QFileInfo r( c.dir(TRUE), abs_or_rel_name );
+	return r.absFilePath();
+    } else {
+	QDir d(context);
+	QFileInfo r(d, abs_or_rel_name);
+	return r.absFilePath();
+    }
 }
 
 /*!
@@ -518,21 +524,21 @@ void QMimeSourceFactory::setPixmap( const QString& abs_name, const QPixmap& pixm
 }
 
 /*!
-    Sets \a data to be the data item associated with the absolute name
-    \a abs_name.
+  Sets \a data to be the data item associated with
+  the absolute name \a abs_name. Note that the ownership of \a data is
+  transferred to the factory: do not delete or access the pointer after
+  passing it to this function.
 
-    The ownership of \a data is transferred to the factory: do not
-    delete or access the pointer after passing it to this function.
+  Passing 0 for data removes previously stored data.
 */
 void QMimeSourceFactory::setData( const QString& abs_name, QMimeSource* data )
-{ 
-    if ( d->stored.contains(abs_name) ) 
+{
+    if ( d->stored.contains(abs_name) )
 	delete d->stored[abs_name];
-    d->stored.replace(abs_name,data); 
+    d->stored.replace(abs_name,data);
 }
 
 
-
 /*!
     Returns the application-wide default mime source factory. This
     factory is used by rich text rendering classes such as
@@ -543,14 +549,14 @@ void QMimeSourceFactory::setData( const QString& abs_name, QMimeSource* data )
 
     \sa setDefaultFactory()
 */
-QMimeSourceFactory* QMimeSourceFactory::defaultFactory() 
-{ 
-    if (!defaultfactory) 
-    { 
+QMimeSourceFactory* QMimeSourceFactory::defaultFactory()
+{
+    if (!defaultfactory)
+    {
 	defaultfactory = new QMimeSourceFactory();
-	qmime_cleanup_factory.set( &defaultfactory ); 
-    } 
-    return defaultfactory; 
+	qmime_cleanup_factory.set( &defaultfactory );
+    }
+    return defaultfactory;
 }
 
 /*!
@@ -562,7 +568,9 @@ QMimeSourceFactory* QMimeSourceFactory::defaultFactory()
 */
 void QMimeSourceFactory::setDefaultFactory( QMimeSourceFactory* factory)
 {
-    if ( defaultfactory != factory )
+    if ( !defaultfactory )
+	qmime_cleanup_factory.set( &defaultfactory );
+    else if ( defaultfactory != factory )
 	delete defaultfactory;
     defaultfactory = factory;
 }
