@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qobject.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/qobject.cpp   3.3.4   edited Jan 5 17:37 $
 **
 ** Implementation of QObject class
 **
@@ -581,7 +581,7 @@ bool QObject::inherits( const char *clname ) const
     return metaObject()->inherits( clname );
 }
 
-/*! 
+/*!
     \internal
 
     Returns TRUE if \a object inherits \a superClass within
@@ -591,7 +591,7 @@ bool QObject::inherits( const char *clname ) const
 */
 void *qt_inheritedBy( QMetaObject *superClass, const QObject *object )
 {
-    if (!object) 
+    if (!object)
 	return 0;
     register QMetaObject *mo = object->metaObject();
     while (mo) {
@@ -981,17 +981,16 @@ void QObject::blockSignals( bool block )
     }
     \endcode
 
-    There is practically no upper limit for the interval value (more
-    than one year is possible). Note that QTimer's accuracy depends on
-    the underlying operating system and hardware. Most platforms
-    support an accuracy of 20ms; some provide more. If Qt is unable to
-    deliver the requested number of timer clicks, it will silently
-    discard some.
+    Note that QTimer's accuracy depends on the underlying operating
+    system and hardware. Most platforms support an accuracy of 20 ms;
+    some provide more. If Qt is unable to deliver the requested
+    number of timer clicks, it will silently discard some.
 
     The QTimer class provides a high-level programming interface with
     one-shot timers and timer signals instead of events.
 
-    \sa timerEvent(), killTimer(), killTimers()
+    \sa timerEvent(), killTimer(), killTimers(), QEventLoop::awake(),
+        QEventLoop::aboutToBlock()
 */
 
 int QObject::startTimer( int interval )
@@ -1729,22 +1728,23 @@ bool QObject::connect( const QObject *sender,	const char *signal,
 	return FALSE;
     }
 #endif
-    QCString signal_name = qt_rmWS( signal );	// white space stripped
-    QCString member_name = qt_rmWS( member );
-    signal = signal_name;
-    member = member_name;
-
     QMetaObject *smeta = sender->metaObject();
 
 #if defined(QT_CHECK_RANGE)
     if ( !check_signal_macro( sender, signal, "connect", "bind" ) )
 	return FALSE;
 #endif
-    signal++;					// skip member type code
+    QCString nw_signal(signal);			// Assume already normalized
+    ++signal;					// skip member type code
 
     int signal_index = smeta->findSignal( signal, TRUE );
+    if ( signal_index < 0 ) {			// normalize and retry
+	nw_signal = qt_rmWS( signal-1 );	// remove whitespace
+	signal = nw_signal.data()+1;		// skip member type code
+	signal_index = smeta->findSignal( signal, TRUE );
+    }
 
-    if ( signal_index < 0  ) {	// no such signal
+    if ( signal_index < 0  ) {			// no such signal
 #if defined(QT_CHECK_RANGE)
 	err_member_notfound( QSIGNAL_CODE, sender, signal, "connect" );
 	err_info_about_candidates( QSIGNAL_CODE, smeta, signal, "connect" );
@@ -1766,14 +1766,25 @@ bool QObject::connect( const QObject *sender,	const char *signal,
 #endif
     member++;					// skip code
 
+    QCString nw_member ;
     QMetaObject *rmeta = r->metaObject();
     int member_index = -1;
     switch ( membcode ) {			// get receiver member
 	case QSLOT_CODE:
 	    member_index = rmeta->findSlot( member, TRUE );
+	    if ( member_index < 0 ) {		// normalize and retry
+		nw_member = qt_rmWS(member);	// remove whitespace
+		member = nw_member;
+		member_index = rmeta->findSlot( member, TRUE );
+	    }
 	    break;
 	case QSIGNAL_CODE:
 	    member_index = rmeta->findSignal( member, TRUE );
+	    if ( member_index < 0 ) {		// normalize and retry
+		nw_member = qt_rmWS(member);	// remove whitespace
+		member = nw_member;
+		member_index = rmeta->findSignal( member, TRUE );
+	    }
 	    break;
     }
     if ( member_index < 0  ) {
@@ -1821,7 +1832,7 @@ bool QObject::connect( const QObject *sender,	const char *signal,
     }
 #endif
     connectInternal( sender, signal_index, receiver, membcode, member_index );
-    s->connectNotify( signal_name );
+    s->connectNotify( nw_signal );
     return TRUE;
 }
 
@@ -1956,32 +1967,39 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 #endif
     if ( !sender->connections )			// no connected signals
 	return FALSE;
-    QCString signal_name;
-    QCString member_name;
     QObject *s = (QObject *)sender;
     QObject *r = (QObject *)receiver;
     int member_index = -1;
     int membcode = -1;
+    QCString nw_member;
     if ( member ) {
-	member_name = qt_rmWS( member );
-	member = member_name.data();
 	membcode = member[0] - '0';
 #if defined(QT_CHECK_RANGE)
 	if ( !check_member_code( membcode, r, member, "disconnect" ) )
 	    return FALSE;
 #endif
-	member++;
+	++member;
 	QMetaObject *rmeta = r->metaObject();
 
 	switch ( membcode ) {			// get receiver member
 	case QSLOT_CODE:
 	    member_index = rmeta->findSlot( member, TRUE );
+	    if ( member_index < 0 ) {		// normalize and retry
+		nw_member = qt_rmWS(member);	// remove whitespace
+		member = nw_member;
+		member_index = rmeta->findSlot( member, TRUE );
+	    }
 	    break;
 	case QSIGNAL_CODE:
 	    member_index = rmeta->findSignal( member, TRUE );
+	    if ( member_index < 0 ) {		// normalize and retry
+		nw_member = qt_rmWS(member);	// remove whitespace
+		member = nw_member;
+		member_index = rmeta->findSignal( member, TRUE );
+	    }
 	    break;
 	}
-	if ( member_index < 0 ) {				// no such member
+	if ( member_index < 0 ) {		// no such member
 #if defined(QT_CHECK_RANGE)
 	    err_member_notfound( membcode, r, member, "disconnect" );
 	    err_info_about_candidates( membcode, rmeta, member, "connect" );
@@ -1997,18 +2015,22 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 	else
 	    return FALSE;
     } else {					// specific signal
-	signal_name = qt_rmWS( signal );
-	signal = signal_name.data();
 #if defined(QT_CHECK_RANGE)
 	if ( !check_signal_macro( s, signal, "disconnect", "unbind" ) )
 	    return FALSE;
 #endif
-	signal++;
+	QCString nw_signal(signal);		// Assume already normalized
+	++signal;				// skip member type code
 
 	QMetaObject *smeta = s->metaObject();
-	if ( !smeta )			// no meta object
+	if ( !smeta )				// no meta object
 	    return FALSE;
 	int signal_index = smeta->findSignal( signal, TRUE );
+	if ( signal_index < 0 ) {		// normalize and retry
+	    nw_signal = qt_rmWS( signal-1 );	// remove whitespace
+	    signal = nw_signal.data()+1;	// skip member type code
+	    signal_index = smeta->findSignal( signal, TRUE );
+	}
 	if ( signal_index < 0 ) {
 #if defined(QT_CHECK_RANGE)
 		qWarning( "QObject::disconnect: No such signal %s::%s",
@@ -2031,7 +2053,7 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 	    res = disconnectInternal( s, signal_index, r, membcode,  member_index );
 	}
 	if ( res )
-	    s->disconnectNotify( signal_name );
+	    s->disconnectNotify( nw_signal );
 	return res;
     }
     return TRUE;
@@ -2065,7 +2087,7 @@ bool QObject::disconnectInternal( const QObject *sender, int signal_index,
 		} else if ( r == c->object() &&
 			    ( member_index == -1 ||
 			      member_index == c->member() && c->memberType() == membcode ) ) {
-		    removeObjFromList( c->object()->senderObjects, s );
+		    removeObjFromList( c->object()->senderObjects, s, TRUE );
 		    success = TRUE;
 		    clist->remove();
 		    c = clist->current();
@@ -2659,4 +2681,3 @@ QObjectUserData* QObject::userData( uint id ) const
 }
 
 #endif // QT_NO_USERDATA
-
