@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qpainter_x11.cpp   3.2.3   edited Sep 16 11:20 $
+** $Id: qt/qpainter_x11.cpp   3.3.4   edited Jan 12 17:28 $
 **
 ** Implementation of QPainter class for X11
 **
@@ -87,8 +87,8 @@ void qt_erase_region( QWidget* w, const QRegion& region)
 {
     QRegion reg = region;
 
-    if ( !w->isTopLevel() && w->backgroundPixmap()
-         && w->backgroundOrigin() != QWidget::WidgetOrigin ) {
+    if ( QPainter::redirect(w) || (!w->isTopLevel() && w->backgroundPixmap()
+                                   && w->backgroundOrigin() != QWidget::WidgetOrigin) ) {
 	QPoint offset = w->backgroundOffset();
 	int ox = offset.x();
 	int oy = offset.y();
@@ -98,9 +98,11 @@ void qt_erase_region( QWidget* w, const QRegion& region)
             ((QWFlagWidget*)w)->clearWFlags( Qt::WPaintUnclipped );
         QPainter p( w );
         p.setClipRegion( region ); // automatically includes paintEventDevice if required
-        p.drawTiledPixmap( 0, 0, w->width(), w->height(),
-                           *w->backgroundPixmap(),
-                           ox, oy );
+        if ( w->backgroundPixmap() )
+            p.drawTiledPixmap( 0, 0, w->width(), w->height(),
+                               *w->backgroundPixmap(), ox, oy );
+        else
+            p.fillRect( w->rect(), w->eraseColor() );
         if ( unclipped )
             ((QWFlagWidget*)w)->setWFlags( Qt::WPaintUnclipped );
         return;
@@ -119,16 +121,16 @@ void qt_erase_region( QWidget* w, const QRegion& region)
 
 void qt_erase_rect( QWidget* w, const QRect& r)
 {
-    if ( w == paintEventDevice || w->backgroundOrigin() != QWidget::WidgetOrigin )
+    if ( QPainter::redirect(w) || w == paintEventDevice
+         || w->backgroundOrigin() != QWidget::WidgetOrigin )
         qt_erase_region( w, r );
     else
         XClearArea( w->x11Display(), w->winId(), r.x(), r.y(), r.width(), r.height(), False );
 
 }
 
-#ifdef QT_NO_XRENDER
-typedef unsigned long Picture;
-static const Picture rendhd = 0;
+#ifdef QT_NO_XFTFREETYPE
+static const Qt::HANDLE rendhd = 0;
 #endif
 
 // hack, so we don't have to make QRegion::clipRectangles() public or include
@@ -138,31 +140,46 @@ inline void *qt_getClipRects( const QRegion &r, int &num )
     return r.clipRectangles( num );
 }
 
-static inline void x11SetClipRegion( Display *dpy, GC gc, Picture rh, const QRegion &r )
+static inline void x11SetClipRegion(Display *dpy, GC gc, GC gc2, Qt::HANDLE draw, const QRegion &r)
 {
     int num;
     XRectangle *rects = (XRectangle *)qt_getClipRects( r, num );
-#ifndef QT_NO_XRENDER
-    if (rh)
-	XRenderSetPictureClipRectangles(dpy, rh, 0, 0, rects, num );
+
+    if (gc)
+        XSetClipRectangles( dpy, gc, 0, 0, rects, num, YXBanded );
+    if (gc2)
+        XSetClipRectangles( dpy, gc2, 0, 0, rects, num, YXBanded );
+
+#ifndef QT_NO_XFTFREETYPE
+    if (draw)
+        XftDrawSetClipRectangles((XftDraw *) draw, 0, 0, rects, num);
 #else
-    Q_UNUSED( rh );
-#endif // QT_NO_XRENDER
-    XSetClipRectangles( dpy, gc, 0, 0, rects, num, Unsorted );
+    Q_UNUSED(draw);
+#endif // QT_NO_XFTFREETYPE
 }
 
-static inline void x11SetClipRegion( Display *dpy, GC gc, Picture rh, const QRegion &r, GC gc2 )
+static inline void x11ClearClipRegion(Display *dpy, GC gc, GC gc2, Qt::HANDLE draw)
 {
-    int num;
-    XRectangle *rects = (XRectangle *)qt_getClipRects( r, num );
-#ifndef QT_NO_XRENDER
-    if (rh)
-	XRenderSetPictureClipRectangles(dpy, rh, 0, 0, rects, num );
+    if (gc)
+        XSetClipMask(dpy, gc, None);
+    if (gc2)
+        XSetClipMask(dpy, gc2, None);
+
+#ifndef QT_NO_XFTFREETYPE
+    if (draw) {
+#  ifdef QT_XFT2
+	XftDrawSetClip((XftDraw *) draw, None);
+#  else
+	// stupid Xft1
+	Picture pict = XftDrawPicture((XftDraw *) draw);
+	XRenderPictureAttributes pattr;
+	pattr.clip_mask = None;
+	XRenderChangePicture(dpy, pict, CPClipMask, &pattr);
+#  endif // QT_XFT2
+    }
 #else
-    Q_UNUSED( rh );
-#endif // QT_NO_XRENDER
-    XSetClipRectangles( dpy, gc, 0, 0, rects, num, Unsorted );
-    XSetClipRectangles( dpy, gc2, 0, 0, rects, num, Unsorted );
+    Q_UNUSED(draw);
+#endif // QT_NO_XFTFREETYPE
 }
 
 
@@ -359,6 +376,9 @@ static void free_gc( Display *dpy, GC gc, bool privateGC = FALSE )
             p++;
         }
     }
+
+    // not found in gc_array
+    XFreeGC(dpy, gc);
 }
 
 
@@ -631,8 +651,10 @@ void QPainter::updateFont()
 {
     clearf(DirtyFont);
     if ( testf(ExtDev) ) {
-	if ( pfont ) delete pfont;
-	pfont = new QFont( cfont.d, pdev );
+        if (pdev->devType() == QInternal::Printer) {
+            if ( pfont ) delete pfont;
+            pfont = new QFont( cfont.d, pdev );
+        }
         QPDevCmdParam param[1];
         param[0].font = &cfont;
         if ( !pdev->cmd( QPaintDevice::PdcSetFont, this, param ) || !hd )
@@ -688,20 +710,13 @@ void QPainter::updatePen()
     if ( !internclipok ) {
         if ( pdev == paintEventDevice && paintEventClipRegion ) {
             if ( penRef &&((QGCC*)penRef)->clip_serial < gc_cache_clip_serial ) {
-		x11SetClipRegion( dpy, gc, rendhd, *paintEventClipRegion );
+		x11SetClipRegion( dpy, gc, 0, rendhd, *paintEventClipRegion );
                 ((QGCC*)penRef)->clip_serial = gc_cache_clip_serial;
             } else if ( !penRef ) {
-		x11SetClipRegion( dpy, gc, rendhd, *paintEventClipRegion );
+		x11SetClipRegion( dpy, gc, 0, rendhd, *paintEventClipRegion );
             }
         } else if (penRef && ((QGCC*)penRef)->clip_serial ) {
-#ifndef QT_NO_XRENDER
-            if (rendhd) {
-                XRenderPictureAttributes pattr;
-                pattr.clip_mask = None;
-                XRenderChangePicture(dpy, rendhd, CPClipMask, &pattr);
-            }
-#endif // QT_NO_XRENDER
-            XSetClipMask( dpy, gc, None );
+            x11ClearClipRegion(dpy, gc, 0, rendhd);
             ((QGCC*)penRef)->clip_serial = 0;
         }
     }
@@ -892,20 +907,13 @@ void QPainter::updateBrush()
     if ( !internclipok ) {
         if ( pdev == paintEventDevice && paintEventClipRegion ) {
             if ( brushRef &&((QGCC*)brushRef)->clip_serial < gc_cache_clip_serial ) {
-		x11SetClipRegion( dpy, gc_brush, rendhd, *paintEventClipRegion );
+		x11SetClipRegion( dpy, gc_brush, 0, rendhd, *paintEventClipRegion );
                 ((QGCC*)brushRef)->clip_serial = gc_cache_clip_serial;
             } else if ( !brushRef ){
-		x11SetClipRegion( dpy, gc_brush, rendhd, *paintEventClipRegion );
+		x11SetClipRegion( dpy, gc_brush, 0, rendhd, *paintEventClipRegion );
             }
         } else if (brushRef && ((QGCC*)brushRef)->clip_serial ) {
-#ifndef QT_NO_XRENDER
-            if (rendhd) {
-                XRenderPictureAttributes pattr;
-                pattr.clip_mask = None;
-                XRenderChangePicture(dpy, rendhd, CPClipMask, &pattr);
-            }
-#endif // QT_NO_XRENDER
-            XSetClipMask( dpy, gc_brush, None );
+            x11ClearClipRegion(dpy, gc_brush, 0, rendhd);
             ((QGCC*)brushRef)->clip_serial = 0;
         }
     }
@@ -1038,7 +1046,7 @@ bool QPainter::begin( const QPaintDevice *pd, bool unclipped )
     dpy = pdev->x11Display();                   // get display variable
     scrn = pdev->x11Screen();			// get screen variable
     hd  = pdev->handle();                       // get handle to drawable
-    rendhd = pdev->x11RenderHandle();
+    rendhd = pdev->rendhd;
 
     if ( testf(ExtDev) ) {                      // external device
         if ( !pdev->cmd( QPaintDevice::PdcBegin, this, 0 ) ) {
@@ -1100,12 +1108,9 @@ bool QPainter::begin( const QPaintDevice *pd, bool unclipped )
             updateBrush();
             XSetSubwindowMode( dpy, gc, IncludeInferiors );
             XSetSubwindowMode( dpy, gc_brush, IncludeInferiors );
-#ifndef QT_NO_XRENDER
-	    if (rendhd) {
-		XRenderPictureAttributes pattr;
-		pattr.subwindow_mode = IncludeInferiors;
-		XRenderChangePicture(dpy, rendhd, CPSubwindowMode, &pattr);
-	    }
+#ifndef QT_NO_XFTFREETYPE
+	    if (rendhd)
+                XftDrawSetSubwindowMode((XftDraw *) rendhd, IncludeInferiors);
 #endif
         }
     } else if ( dt == QInternal::Pixmap ) {             // device is a pixmap
@@ -1210,15 +1215,13 @@ bool QPainter::end()                            // end painting
     if ( testf(ExtDev) )
         pdev->cmd( QPaintDevice::PdcEnd, this, 0 );
 
-#ifndef QT_NO_XRENDER
+#ifndef QT_NO_XFTFREETYPE
     if (rendhd) {
         // reset clipping/subwindow mode on our render picture
-        XRenderPictureAttributes pattr;
-        pattr.subwindow_mode = ClipByChildren;
-        pattr.clip_mask = None;
-        XRenderChangePicture(dpy, rendhd, CPClipMask | CPSubwindowMode, &pattr);
+        XftDrawSetClip((XftDraw *) rendhd, None);
+        XftDrawSetSubwindowMode((XftDraw *) rendhd, ClipByChildren);
     }
-#endif // QT_NO_XRENDER
+#endif // QT_NO_XFTFREETYPE
 
     if ( pfont ) {
 	delete pfont;
@@ -1354,7 +1357,8 @@ static const short ropCodes[] = {                     // ROP translation table
 
 
 /*!
-    Sets the raster operation to \a r. The default is \c CopyROP.
+    Sets the \link Qt::RasterOp raster operation \endlink to \a r.
+    The default is \c CopyROP.
 
     \sa rasterOp() Qt::RasterOp
 */
@@ -1458,22 +1462,12 @@ void QPainter::setClipping( bool enable )
             updatePen();
         if ( brushRef )
             updateBrush();
-	x11SetClipRegion( dpy, gc, rendhd, rgn, gc_brush );
+	x11SetClipRegion( dpy, gc, gc_brush, rendhd, rgn );
     } else {
         if ( pdev == paintEventDevice && paintEventClipRegion ) {
-	    x11SetClipRegion( dpy, gc, rendhd, *paintEventClipRegion, gc_brush );
+	    x11SetClipRegion( dpy, gc, gc_brush , rendhd, *paintEventClipRegion );
         } else {
-            XSetClipMask( dpy, gc, None );
-            XSetClipMask( dpy, gc_brush, None );
-
-#ifndef QT_NO_XRENDER
-            if (rendhd) {
-                XRenderPictureAttributes pattr;
-                pattr.clip_mask = None;
-                XRenderChangePicture(dpy, rendhd, CPClipMask, &pattr);
-            }
-#endif // QT_NO_XRENDER
-
+            x11ClearClipRegion(dpy, gc, gc_brush, rendhd);
         }
     }
 }
@@ -1597,6 +1591,10 @@ void QPainter::drawPoint( int x, int y )
     index are drawn. If \a npoints is negative (the default) the rest
     of the points from \a index are drawn. If \a npoints is zero or
     greater, \a npoints points are drawn.
+
+    \warning On X11, coordinates that do not fit into 16-bit signed
+    values are truncated. This limitation is expected to go away in
+    Qt 4.
 */
 
 void QPainter::drawPoints( const QPointArray& a, int index, int npoints )
@@ -2271,6 +2269,10 @@ void QPainter::drawChord( int x, int y, int w, int h, int a, int alen )
     Draws the 1st line from \a a[index] to \a a[index+1]. Draws the
     2nd line from \a a[index+2] to \a a[index+3] etc.
 
+    \warning On X11, coordinates that do not fit into 16-bit signed
+    values are truncated. This limitation is expected to go away in
+    Qt 4.
+
     \sa drawPolyline(), drawPolygon(), QPen
 */
 
@@ -2318,6 +2320,10 @@ void QPainter::drawLineSegments( const QPointArray &a, int index, int nlines )
     If \a npoints is -1 (the default) all points until the end of the
     array are used (i.e. a.size()-index-1 line segments are drawn).
 
+    \warning On X11, coordinates that do not fit into 16-bit signed
+    values are truncated. This limitation is expected to go away in
+    Qt 4.
+
     \sa drawLineSegments(), drawPolygon(), QPen
 */
 
@@ -2363,7 +2369,7 @@ void QPainter::drawPolyline( const QPointArray &a, int index, int npoints )
     }
 }
 
-int global_polygon_shape = Complex;
+static int global_polygon_shape = Complex;
 
 /*!
     Draws the polygon defined by the \a npoints points in \a a
@@ -2380,6 +2386,10 @@ int global_polygon_shape = Complex;
     \a winding is FALSE, the polygon is filled using the even-odd
     (alternative) fill algorithm.
 
+    \warning On X11, coordinates that do not fit into 16-bit signed
+    values are truncated. This limitation is expected to go away in
+    Qt 4.
+
     \sa drawLineSegments(), drawPolyline(), QPen
 */
 
@@ -2446,6 +2456,10 @@ void QPainter::drawPolygon( const QPointArray &a, bool winding,
 
     On some platforms (e.g. X Window), this is faster than
     drawPolygon().
+
+    \warning On X11, coordinates that do not fit into 16-bit signed
+    values are truncated. This limitation is expected to go away in
+    Qt 4.
 */
 void QPainter::drawConvexPolygon( const QPointArray &pa,
                                   int index, int npoints )
@@ -2463,6 +2477,10 @@ void QPainter::drawConvexPolygon( const QPointArray &pa,
 
     Control points after \a a[index + 3] are ignored. Nothing happens
     if there aren't enough control points.
+
+    \warning On X11, coordinates that do not fit into 16-bit signed
+    values are truncated. This limitation is expected to go away in
+    Qt 4.
 */
 
 void QPainter::drawCubicBezier( const QPointArray &a, int index )
@@ -2625,9 +2643,9 @@ void QPainter::drawPixmap( int x, int y, const QPixmap &pixmap,
             XSetFillStyle( dpy, gc, FillSolid );
             if ( !selfmask ) {
                 if ( pdev == paintEventDevice && paintEventClipRegion ) {
-                    x11SetClipRegion( dpy, gc, rendhd, *paintEventClipRegion );
+                    x11SetClipRegion( dpy, gc, 0, rendhd, *paintEventClipRegion );
 		} else {
-                    XSetClipMask( dpy, gc, None );
+                    x11ClearClipRegion(dpy, gc, 0, rendhd);
 		}
             }
         } else {
@@ -2658,7 +2676,7 @@ void QPainter::drawPixmap( int x, int y, const QPixmap &pixmap,
         XSetForeground( dpy, cgc, 1 );
 	int num;
 	XRectangle *rects = (XRectangle *)qt_getClipRects( rgn, num );
-        XSetClipRectangles( dpy, cgc, -x, -y, rects, num, Unsorted );
+        XSetClipRectangles( dpy, cgc, -x, -y, rects, num, YXBanded );
         XSetFillStyle( dpy, cgc, FillOpaqueStippled );
         XSetStipple( dpy, cgc, mask->handle() );
         XSetTSOrigin( dpy, cgc, -sx, -sy );
@@ -2681,19 +2699,20 @@ void QPainter::drawPixmap( int x, int y, const QPixmap &pixmap,
         XSetTSOrigin( dpy, gc, 0, 0 );
         XSetFillStyle( dpy, gc, FillSolid );
     } else {
-#ifndef QT_NO_XRENDER
+#if !defined(QT_NO_XFTFREETYPE) && !defined(QT_NO_XRENDER)
+        Picture pict = rendhd ? XftDrawPicture((XftDraw *) rendhd) : None;
 	QPixmap *alpha = pixmap.data->alphapm;
 
-	if ( rendhd && pixmap.x11RenderHandle() &&
+	if ( pict && pixmap.x11RenderHandle() &&
 	     alpha && alpha->x11RenderHandle()) {
 	    XRenderComposite(dpy, PictOpOver, pixmap.x11RenderHandle(),
-			     alpha->x11RenderHandle(), rendhd,
+			     alpha->x11RenderHandle(), pict,
 			     sx, sy, sx, sy, x, y, sw, sh);
 	} else
-#endif // QT_NO_XRENDER
-	{
-	    XCopyArea( dpy, pixmap.handle(), hd, gc, sx, sy, sw, sh, x, y );
-	}
+#endif // !QT_NO_XFTFREETYPE && !QT_NO_XRENDER
+        {
+            XCopyArea( dpy, pixmap.handle(), hd, gc, sx, sy, sw, sh, x, y );
+        }
     }
 
     if ( mask ) {                               // restore clipping
@@ -2794,10 +2813,11 @@ void QPainter::drawTiledPixmap( int x, int y, int w, int h,
         if ( txop == TxTranslate )
             map( x, y, &x, &y );
 
-#ifndef QT_NO_XRENDER
+#if !defined(QT_NO_XFTFREETYPE) && !defined(QT_NO_XRENDER)
+        Picture pict = rendhd ? XftDrawPicture((XftDraw *) rendhd) : None;
 	QPixmap *alpha = pixmap.data->alphapm;
 
-        if (rendhd && pixmap.x11RenderHandle() && alpha && alpha->x11RenderHandle()) {
+        if (pict && pixmap.x11RenderHandle() && alpha && alpha->x11RenderHandle()) {
 	    // this is essentially drawTile() from above, inlined for
 	    // the XRenderComposite call
 	    int yPos, xPos, drawH, drawW, yOff, xOff;
@@ -2814,7 +2834,7 @@ void QPainter::drawTiledPixmap( int x, int y, int w, int h,
 		    if ( xPos + drawW > x + w )    // Cropping last column
 			drawW = x + w - xPos;
 		    XRenderComposite(dpy, PictOpOver, pixmap.x11RenderHandle(),
-				     alpha->x11RenderHandle(), rendhd,
+				     alpha->x11RenderHandle(), pict,
 				     xOff, yOff, xOff, yOff, xPos, yPos, drawW, drawH);
 		    xPos += drawW;
 		    xOff = 0;
@@ -2824,7 +2844,7 @@ void QPainter::drawTiledPixmap( int x, int y, int w, int h,
 	    }
 	    return;
 	}
-#endif // QT_NO_XRENDER
+#endif // !QT_NO_XFTFREETYPE && !QT_NO_XRENDER
 
 	XSetTile( dpy, gc, pixmap.handle() );
 	XSetFillStyle( dpy, gc, FillTiled );
@@ -2993,7 +3013,7 @@ void QPainter::drawText( int x, int y, const QString &str, int pos, int len, QPa
 	    return;
     }
 
-    bool simple = str.simpleText();
+    bool simple = (dir == QPainter::Auto) && str.simpleText();
     // we can't take the complete string here as we would otherwise
     // get quadratic behaviour when drawing long strings in parts.
     // we do however need some chars around the part we paint to get arabic shaping correct.
@@ -3019,17 +3039,17 @@ void QPainter::drawText( int x, int y, const QString &str, int pos, int len, QPa
     engine.currentItem = 0;
     engine.firstItemInLine = -1;
 
-    if ( !simple ) {
-	layout.setBoundary( pos );
-	layout.setBoundary( pos + len );
-    }
-
     if ( dir != Auto ) {
 	int level = dir == RTL ? 1 : 0;
 	for ( int i = engine.items.size(); i >= 0; i-- )
 	    engine.items[i].analysis.bidiLevel = level;
     }
 
+    if ( !simple ) {
+	layout.setBoundary( pos );
+	layout.setBoundary( pos + len );
+    }
+
     // small hack to force skipping of unneeded items
     start = 0;
     while ( engine.items[start].position < pos )
@@ -3098,7 +3118,7 @@ void QPainter::drawTextItem( int x,  int y, const QTextItem &ti, int textFlags )
 
     engine->shape( ti.item );
     QFontEngine *fe = si->fontEngine;
-    assert( fe );
+    assert( fe != 0 );
 
     x += si->x;
     y += si->y;
