@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qpixmap.cpp   3.2.3   edited Aug 12 14:02 $
+** $Id: qt/qpixmap.cpp   3.3.4   edited Jan 12 17:30 $
 **
 ** Implementation of QPixmap class
 **
@@ -102,8 +102,11 @@
 
     Note regarding Windows 95 and 98: on Windows 9x the system crashes
     if you create more than about 1000 pixmaps, independent of the
-    size of the pixmaps or installed RAM. Windows NT and 2000 do not
-    have this limitation.
+    size of the pixmaps or installed RAM. Windows NT-systems (including
+    2000, XP and following versions) do not have the same limitation,
+    but depending on the graphics equipment the system will fail to
+    allocate pixmap objects at some point (due to system running out of
+    GDI resources).
 
     Qt tries to work around the resource limitation. If you set the
     pixmap optimization to \c QPixmap::MemoryOptim and the width of
@@ -125,6 +128,9 @@
 	QPixmap::setDefaultOptimization( QPixmap::NormalOptim );
     \endcode
 
+    In general it is recommended to make as much use of QPixmap's
+    implicit sharing and the QPixmapCache as possible.
+
     \sa QBitmap, QImage, QImageIO, \link shclass.html Shared Classes\endlink
 */
 
@@ -158,7 +164,8 @@
     \value NoOptim  No optimization (currently the same as \c
 	MemoryOptim).
 
-    \value MemoryOptim  Optimize for minimal memory use.
+    \value MemoryOptim  Optimize for minimal memory use on Windows
+	9x and X11 systems.
 
     \value NormalOptim  Optimize for typical usage. Often uses more
 	memory than \c MemoryOptim, and is often faster.
@@ -719,13 +726,21 @@ void QPixmap::setMask( const QBitmap &newmask )
 	data->selfmask = TRUE;			// mask == pixmap
 	return;
     }
-    detach();
-    data->selfmask = FALSE;
+
     if ( newmask.isNull() ) {			// reset the mask
-	delete data->mask;
-	data->mask = 0;
+	if (data->mask) {
+	    detach();
+	    data->selfmask = FALSE;
+
+	    delete data->mask;
+	    data->mask = 0;
+	}
 	return;
     }
+
+    detach();
+    data->selfmask = FALSE;
+
     if ( newmask.width() != width() || newmask.height() != height() ) {
 #if defined(QT_CHECK_RANGE)
 	qWarning( "QPixmap::setMask: The pixmap and the mask must have "
@@ -1103,8 +1118,6 @@ static QPixmap grabChildWidgets( QWidget * w )
     QApplication::sendEvent( w, &e );
     QSharedDoubleBuffer::setDisabled( dblbfr );
     QPainter::redirect( w, oldRedirect );
-    if ( w->testWFlags( Qt::WRepaintNoErase ) )
-	w->repaint( FALSE );
 
     const QObjectList * children = w->children();
     if ( children ) {
@@ -1363,15 +1376,15 @@ bool qt_xForm_helper( const QWMatrix &trueMat, int xoffset,
 	uchar *sptr, int sbpl, int sWidth, int sHeight
 	)
 {
-    int m11 = (int)(trueMat.m11()*65536.0);
-    int m12 = (int)(trueMat.m12()*65536.0);
-    int m21 = (int)(trueMat.m21()*65536.0);
-    int m22 = (int)(trueMat.m22()*65536.0);
-    int dx  = (int)(trueMat.dx() *65536.0);
-    int dy  = (int)(trueMat.dy() *65536.0);
+    int m11 = int(trueMat.m11()*65536.0 + 1.);
+    int m12 = int(trueMat.m12()*65536.0 + 1.);
+    int m21 = int(trueMat.m21()*65536.0 + 1.);
+    int m22 = int(trueMat.m22()*65536.0 + 1.);
+    int dx  = qRound(trueMat.dx() *65536.0);
+    int dy  = qRound(trueMat.dy() *65536.0);
 
-    int m21ydx = dx + (xoffset<<16) + QABS(m11)/2;
-    int m22ydy = dy + QABS(m22)/2;
+    int m21ydx = dx + (xoffset<<16);
+    int m22ydy = dy;
     uint trigx;
     uint trigy;
     uint maxws = sWidth<<16;
