@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qpointarray.cpp   3.2.3   edited Jun 30 10:46 $
+** $Id: qt/qpointarray.cpp   3.3.4   edited May 4 2004 $
 **
 ** Implementation of QPointArray class
 **
@@ -480,7 +480,7 @@ static inline int fix_angle( int a )
     whereas negative values mean the clockwise direction. Zero degrees
     is at the 3 o'clock position.
 
-    See the \link qcanvas.html#anglediagram angle diagram\endlink.
+    See the \link qcanvasellipse.html#anglediagram angle diagram\endlink.
 */
 
 void QPointArray::makeArc( int x, int y, int w, int h, int a1, int a2 )
@@ -583,7 +583,7 @@ qtr_elips(QPointArray& a, int off, double dxP, double dyP, double dxQ, double dy
     whereas negative values mean the clockwise direction. Zero degrees
     is at the 3 o'clock position.
 
-    See the \link qcanvas.html#anglediagram angle diagram\endlink.
+    See the \link qcanvasellipse.html#anglediagram angle diagram\endlink.
 */
 void QPointArray::makeArc( int x, int y, int w, int h,
 			       int a1, int a2,
