@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qprocess.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/qprocess.cpp   3.3.4   edited Feb 9 2004 $
 **
 ** Implementation of QProcess class
 **
@@ -159,6 +159,20 @@
     readyReadStdout() and readyReadStderr() signals and read the data
     as soon as it becomes available.
 
+    Please note that QProcess does not emulate a shell. This means that
+    QProcess does not do any expansion of arguments: a '*' is passed as a '*'
+    to the program and is \e not replaced by all the files, a '$HOME' is also
+    passed literally and is \e not replaced by the environment variable HOME
+    and the special characters for IO redirection ('>', '|', etc.) are also
+    passed literally and do \e not have the special meaning as they have in a
+    shell.
+
+    Also note that QProcess does not emulate a terminal. This means that
+    certain programs which need direct terminal control, do not work as
+    expected with QProcess. Such programs include console email programs (like
+    pine and mutt) but also programs which require the user to enter a password
+    (like su and ssh).
+
     \section1 Notes for Windows users
 
     Some Windows commands, for example, \c dir, are not provided by
@@ -170,10 +184,7 @@
 
     Under Windows there are certain problems starting 16-bit applications
     and capturing their output. Microsoft recommends using an intermediate
-    application to start 16-bit applications. See Knowledge Base article \link
-    http://support.microsoft.com/default.aspx?scid=KB;en-us;q150956 Q150956
-    \endlink for details on this and example code for an intermediate
-    application.
+    application to start 16-bit applications.
 
     \sa QSocket
 */
@@ -479,28 +490,35 @@ QByteArray QProcess::readStderr()
     newline or carriage return characters, and returns it. Returns
     QString::null if canReadLineStdout() returns FALSE.
 
+    By default, the text is interpreted to be in Latin-1 encoding. If you need
+    other codecs, you can set a different codec with
+    QTextCodec::setCodecForCStrings().
+
     \sa canReadLineStdout() readyReadStdout() readStdout() readLineStderr()
 */
 QString QProcess::readLineStdout()
 {
     QByteArray a( 256 );
     QMembuf *buf = membufStdout();
-    if ( buf->scanNewline( &a ) ) {
-	uint size = a.size();
-	buf->consumeBytes( size, 0 );
+    if ( !buf->scanNewline( &a ) ) {
+      if ( !canReadLineStdout() )
+	return QString::null;
 
-	// get rid of terminating \n or \r\n
-	if ( size>0 && a.at( size - 1 ) == '\n' ) {
-	    if ( size>1 && a.at( size - 2 ) == '\r' )
-		a.at( size - 2 ) = '\0';
-	    else
-		a.at( size - 1 ) = '\0';
-	}
-	return QString( a );
-    } else if ( canReadLineStdout() ) {
+      if ( !buf->scanNewline( &a ) )
 	return QString( buf->readAll() );
     }
-    return QString::null;
+
+    uint size = a.size();
+    buf->consumeBytes( size, 0 );
+
+    // get rid of terminating \n or \r\n
+    if ( size>0 && a.at( size - 1 ) == '\n' ) {
+      if ( size>1 && a.at( size - 2 ) == '\r' )
+	a.at( size - 2 ) = '\0';
+      else
+	a.at( size - 1 ) = '\0';
+    }
+    return QString( a );
 }
 
 /*!
@@ -508,28 +526,35 @@ QString QProcess::readLineStdout()
     newline or carriage return characters and returns it. Returns
     QString::null if canReadLineStderr() returns FALSE.
 
+    By default, the text is interpreted to be in Latin-1 encoding. If you need
+    other codecs, you can set a different codec with
+    QTextCodec::setCodecForCStrings().
+
     \sa canReadLineStderr() readyReadStderr() readStderr() readLineStdout()
 */
 QString QProcess::readLineStderr()
 {
     QByteArray a( 256 );
     QMembuf *buf = membufStderr();
-    if ( buf->scanNewline( &a ) ) {
-	uint size = a.size();
-	buf->consumeBytes( size, 0 );
+    if ( !buf->scanNewline( &a ) ) {
+      if ( !canReadLineStderr() )
+	return QString::null;
 
-	// get rid of terminating \n or \r\n
-	if ( size>0 && a.at( size - 1 ) == '\n' ) {
-	    if ( size>1 && a.at( size - 2 ) == '\r' )
-		a.at( size - 2 ) = '\0';
-	    else
-		a.at( size - 1 ) = '\0';
-	}
-	return QString( a );
-    } else if ( canReadLineStderr() ) {
+      if ( !buf->scanNewline( &a ) )
 	return QString( buf->readAll() );
     }
-    return QString::null;
+
+    uint size = a.size();
+    buf->consumeBytes( size, 0 );
+
+    // get rid of terminating \n or \r\n
+    if ( size>0 && a.at( size - 1 ) == '\n' ) {
+      if ( size>1 && a.at( size - 2 ) == '\r' )
+	a.at( size - 2 ) = '\0';
+      else
+	a.at( size - 1 ) = '\0';
+    }
+    return QString( a );
 }
 
 /*!
@@ -541,7 +566,7 @@ QString QProcess::readLineStderr()
     this signal is emitted immediately.
 
     This signal is especially useful if you want to know when you can
-    safely delete the QProcess object when you are not intrested in
+    safely delete the QProcess object when you are not interested in
     reading from standard output or standard error.
 
     \sa launch() QObject::deleteLater()
