@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qregion_x11.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/qregion_x11.cpp   3.3.4   edited Sep 24 11:29 $
 **
 ** Implementation of QRegion class for X11
 **
@@ -2135,9 +2135,9 @@ static int PtsToRegion(register int numFullPtBlocks, register int iCurPtBlock,
 	if (!numFullPtBlocks)
 	    i = iCurPtBlock >> 1;
 	for (pts = CurPtBlock->pts; i--; pts += 2) {
-	    if ( pts->x() > pts[1].x() )
+	    if ( pts->x() == pts[1].x() )
 		continue;
-	    if (numRects && pts->x() == rects->left() && pts->y() == rects->bottom() &&
+	    if (numRects && pts->x() == rects->left() && pts->y() == rects->bottom() + 1 &&
 		pts[1].x() == rects->right() &&
 		(numRects == 1 || rects[-1].top() != rects->top()) &&
 		(i && pts[2].y() > pts[1].y() )) {
@@ -2146,7 +2146,7 @@ static int PtsToRegion(register int numFullPtBlocks, register int iCurPtBlock,
 	    }
 	    numRects++;
 	    rects++;
-	    qt_setCoords( rects, pts->x(), pts->y(), pts[1].x(), pts[1].y() );
+	    qt_setCoords( rects, pts->x(), pts->y(), pts[1].x() - 1, pts[1].y() );
 	    if (rects->left() < extents->left())
 		extents->setLeft( rects->left() );
 	    if (rects->right() > extents->right())
@@ -2503,7 +2503,7 @@ QRegion::QRegion( const QRect &r, RegionType t )
 
 QRegion::QRegion( const QPointArray &a, bool winding )
 {
-    if (a.size() > 0) {
+    if (a.size() > 2) {
 	data = new QRegionData;
 	Q_CHECK_PTR( data );
 	data->is_null = FALSE;
