@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qsize.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qsize.cpp   3.3.4   edited Oct 23 2003 $
 **
 ** Implementation of QSize class
 **
@@ -40,7 +40,7 @@
 
 
 /*!
-  \class QSize qsize.h
+  \class QSize
   \brief The QSize class defines the size of a two-dimensional object.
 
   \ingroup images
@@ -48,9 +48,9 @@
 
   A size is specified by a width and a height.
 
-  The coordinate type is QCOORD (defined in qwindowdefs.h as \c int).
+  The coordinate type is QCOORD (defined in \c <qwindowdefs.h> as \c int).
   The minimum value of QCOORD is QCOORD_MIN (-2147483648) and the maximum
-  value is  QCOORD_MAX (2147483647).
+  value is QCOORD_MAX (2147483647).
 
   The size can be set in the constructor and changed with setWidth()
   and setHeight(), or using operator+=(), operator-=(), operator*=()
@@ -86,8 +86,8 @@
 
 /*!
   \fn bool QSize::isEmpty() const
-  Returns TRUE if the width is <= 0 or the height is <= 0,
-  otherwise FALSE.
+  Returns TRUE if the width is less than or equal to 0, or the height is
+  less than or equal to 0; otherwise returns FALSE.
 */
 
 /*!
@@ -131,6 +131,85 @@ void QSize::transpose()
     ht = tmp;
 }
 
+/*! \enum QSize::ScaleMode
+
+    This enum type defines the different ways of scaling a size.
+
+    \img scaling.png
+
+    \value ScaleFree  The size is scaled freely. The ratio is not preserved.
+    \value ScaleMin  The size is scaled to a rectangle as large as possible
+                     inside a given rectangle, preserving the aspect ratio.
+    \value ScaleMax  The size is scaled to a rectangle as small as possible
+                     outside a given rectangle, preserving the aspect ratio.
+
+    \sa QSize::scale(), QImage::scale(), QImage::smoothScale()
+*/
+
+/*!
+    Scales the size to a rectangle of width \a w and height \a h according
+    to the ScaleMode \a mode.
+
+    \list
+    \i If \a mode is \c ScaleFree, the size is set to (\a w, \a h).
+    \i If \a mode is \c ScaleMin, the current size is scaled to a rectangle
+       as large as possible inside (\a w, \a h), preserving the aspect ratio.
+    \i If \a mode is \c ScaleMax, the current size is scaled to a rectangle
+       as small as possible outside (\a w, \a h), preserving the aspect ratio.
+    \endlist
+
+    Example:
+    \code
+    QSize t1( 10, 12 );
+    t1.scale( 60, 60, QSize::ScaleFree );
+    // t1 is (60, 60)
+
+    QSize t2( 10, 12 );
+    t2.scale( 60, 60, QSize::ScaleMin );
+    // t2 is (50, 60)
+
+    QSize t3( 10, 12 );
+    t3.scale( 60, 60, QSize::ScaleMax );
+    // t3 is (60, 72)
+    \endcode
+*/
+void QSize::scale( int w, int h, ScaleMode mode )
+{
+    if ( mode == ScaleFree ) {
+	wd = (QCOORD)w;
+	ht = (QCOORD)h;
+    } else {
+	bool useHeight = TRUE;
+	int w0 = width();
+	int h0 = height();
+	int rw = h * w0 / h0;
+
+	if ( mode == ScaleMin ) {
+	    useHeight = ( rw <= w );
+	} else { // mode == ScaleMax
+	    useHeight = ( rw >= w );
+	}
+
+	if ( useHeight ) {
+	    wd = (QCOORD)rw;
+	    ht = (QCOORD)h;
+	} else {
+	    wd = (QCOORD)w;
+	    ht = (QCOORD)( w * h0 / w0 );
+	}
+    }
+}
+
+/*!
+    \overload
+
+    Equivalent to scale(\a{s}.width(), \a{s}.height(), \a mode).
+*/
+void QSize::scale( const QSize &s, ScaleMode mode )
+{
+    scale( s.width(), s.height(), mode );
+}
+
 /*!
   \fn QCOORD &QSize::rwidth()
   Returns a reference to the width.
