@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qsound.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/qsound.cpp   3.3.4   edited May 21 2004 $
 **
 ** Implementation of QSound class and QAuServer internal class
 **
@@ -181,16 +181,11 @@ QSound::~QSound()
 /*!
     Returns TRUE if the sound has finished playing; otherwise returns FALSE.
 
-    Note: On Windows this will always return TRUE as there is no way to determine
-    when the sound has finished.
+    \warning On Windows this function always returns TRUE for unlooped sounds.
 */
 bool QSound::isFinished() const
 {
-#ifdef Q_OS_WIN
-    return TRUE;
-#else
     return d->looprem == 0;
-#endif
 }
 
 /*!
@@ -220,9 +215,6 @@ int QSound::loops() const
 /*!
     Returns the number of times the sound will loop. This value
     decreases each time the sound loops.
-
-    This function is not supported on Windows and will always return
-    the total number of loops.
 */
 int QSound::loopsRemaining() const
 {
@@ -233,9 +225,6 @@ int QSound::loopsRemaining() const
     Sets the sound to repeat \a l times when it is played. Passing the
     value -1 will cause the sound to loop indefinitely.
 
-    On Windows, sounds with a loop count different from 1 will always 
-    loop indefinitely.
-
     \sa loops()
 */
 void QSound::setLoops(int l)
@@ -254,6 +243,9 @@ QString QSound::fileName() const
 /*!
     Stops the sound playing.
 
+    On Windows the current loop will finish if a sound is played
+    in a loop.
+
     \sa play()
 */
 void QSound::stop()
