@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qstylesheet.cpp   3.2.3   edited Nov 3 16:38 $
+** $Id: qt/qstylesheet.cpp   3.3.4   edited May 14 2004 $
 **
 ** Implementation of the QStyleSheet class
 **
@@ -898,6 +898,8 @@ int QStyleSheetItem::lineSpacing() const
 	 \i A sublevel heading.
     \row \i \c{<h3>...</h3>}
 	 \i A sub-sublevel heading.
+    \row \i \c{<h4>...</h4>} \c{<h5>...</h5>}
+	 \i Headings of lesser importance.
     \row \i \c{<p>...</p>}
 	 \i A left-aligned paragraph. Adjust the alignment with the \c
 	    align attribute. Possible values are \c left, \c right and
@@ -917,10 +919,22 @@ int QStyleSheetItem::lineSpacing() const
     \row \i \c{<li>...</li>}
 	 \i A list item. This tag can be used only within the context
 	    of \c{<ol>} or \c{<ul>}.
+    \row \i \c{<dl>...</dl>}
+	 \i A list of definitions, consisting of terms and descriptions.
+    \row \i \c{<dt>...</dt>}
+	 \i A term in a list of definitions. This tag can be used only
+	    in the context of \c{<dl>...</dl>}.
+    \row \i \c{<dd>...</dd>}
+	 \i A description in a list of definitions. This tag can be
+	    used only in the context of \c{<dl>...</dl>}.
     \row \i \c{<pre>...</pre>}
 	 \i For larger chunks of code. Whitespaces in the contents are
 	    preserved. For small bits of code use the inline-style \c
 	    code.
+    \row \i \c{<div>...</div>} and \c{<span>...</span>}
+	 \i Block grouping elements. These are used to structure the
+	    document, and are often used to provide hints about the
+	    intended presentation of the document.
     \endtable
 
     Anchors and links are done with a single tag:
