@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qgl.cpp   3.2.3   edited Aug 12 16:27 $
+** $Id: qt/qgl.cpp   3.3.4   edited Aug 10 18:40 $
 **
 ** Implementation of OpenGL classes for Qt
 **
@@ -45,6 +45,16 @@
 static QGLFormat* qgl_default_format = 0;
 static QGLFormat* qgl_default_overlay_format = 0;
 
+#if defined(Q_WS_X11)
+#include "private/qt_x11_p.h"
+#define INT32 dummy_INT32
+#define INT8 dummy_INT8
+#include <GL/glx.h>
+#undef INT32
+#undef INT8
+#include "qgl_x11_p.h"
+static void *gl_pixmap_visual = 0;
+#endif
 
 static QCleanupHandler<QGLFormat> qgl_cleanup_format;
 
@@ -874,10 +884,11 @@ void QGLContext::init( QPaintDevice *dev )
 {
     d = new Private;
     d->valid = FALSE;
-    setDevice( dev );
 #if defined(Q_WS_X11)
+    qt_resolve_gl_symbols();
     gpm = 0;
 #endif
+    setDevice( dev );
 #if defined(Q_WS_WIN)
     dc = 0;
     win = 0;
@@ -1274,17 +1285,8 @@ static QGLWidgetPrivate * qgl_d( const QGLWidget * w )
 
 void qgl_delete_d( const QGLWidget * w )
 {
-    if ( qgl_d_ptr ) {
-	QGLWidgetPrivate * d = qgl_d_ptr->find( (void *) w );
-	((QGLWidget*) w)->makeCurrent();
-	if ( d ) {
-	    QMapIterator<QString, int> it;
-	    for ( it = d->displayListCache.begin(); it != d->displayListCache.end(); ++it ) {
-		glDeleteLists( it.data(), 256 );
-	    }
-	}
+    if ( qgl_d_ptr )
 	qgl_d_ptr->remove( (void *) w );
-    }
 }
 
 /*!
@@ -1816,12 +1818,83 @@ void QGLWidget::paintEvent( QPaintEvent * )
 
 QPixmap QGLWidget::renderPixmap( int w, int h, bool useContext )
 {
-    QPixmap nullPm;
     QSize sz = size();
     if ( (w > 0) && (h > 0) )
 	sz = QSize( w, h );
-    QPixmap pm( sz );
+    QPixmap pm;
+    pm.resize( sz );
+
+#if defined(Q_WS_X11)
+    // If we are using OpenGL widgets we HAVE to make sure that
+    // the default visual is GL enabled, otherwise it will wreck
+    // havock when e.g trying to render to GLXPixmaps via
+    // QPixmap. This is because QPixmap is always created with a
+    // QPaintDevice that uses x_appvisual per default. Preferably,
+    // use a visual that has depth and stencil buffers.
+
+    if (!gl_pixmap_visual) {
+	int nvis;
+	Visual *vis = (Visual *) QPaintDevice::x11AppVisual();
+	int screen = QPaintDevice::x11AppScreen();
+	Display *appDpy = QPaintDevice::x11AppDisplay();
+	XVisualInfo * vi;
+	XVisualInfo visInfo;
+	memset( &visInfo, 0, sizeof(XVisualInfo) );
+	visInfo.visualid = XVisualIDFromVisual( vis );
+	visInfo.screen = screen;
+	vi = XGetVisualInfo( appDpy, VisualIDMask | VisualScreenMask, &visInfo, &nvis );
+	if ( vi ) {
+	    int useGL;
+	    int ret = glXGetConfig( appDpy, vi, GLX_USE_GL, &useGL );
+	    if ( ret != 0 || !useGL ) {
+		// We have to find another visual that is GL capable
+		int i;
+		XVisualInfo * visuals;
+		memset( &visInfo, 0, sizeof(XVisualInfo) );
+		visInfo.screen = screen;
+		visInfo.c_class = vi->c_class;
+		visInfo.depth = vi->depth;
+		visuals = XGetVisualInfo( appDpy, VisualClassMask |
+					  VisualDepthMask |
+					  VisualScreenMask, &visInfo,
+					  &nvis );
+		if ( visuals ) {
+		    for ( i = 0; i < nvis; i++ ) {
+			int ret = glXGetConfig( appDpy, &visuals[i], GLX_USE_GL, &useGL );
+			if ( ret == 0 && useGL ) {
+			    vis = visuals[i].visual;
+			    break;
+			}
+		    }
+		    XFree( visuals );
+		}
+	    }
+	    XFree( vi );
+	}
+	gl_pixmap_visual = vis;
+    }
+
+    if (gl_pixmap_visual != QPaintDevice::x11AppVisual()) {
+	int nvis = 0;
+	XVisualInfo visInfo;
+	memset( &visInfo, 0, sizeof(XVisualInfo) );
+	visInfo.visualid = XVisualIDFromVisual( (Visual *) gl_pixmap_visual );
+	visInfo.screen = QPaintDevice::x11AppScreen();
+	XVisualInfo *vi = XGetVisualInfo( QPaintDevice::x11AppDisplay(), VisualIDMask | VisualScreenMask,
+					  &visInfo, &nvis );
+	if (vi) {
+	    QPaintDeviceX11Data* xd = pm.getX11Data( TRUE );
+	    xd->x_depth = vi->depth;
+	    xd->x_visual = (Visual *) gl_pixmap_visual;
+	    pm.setX11Data( xd );
+	    XFree(vi);
+	}
+    }
+
+#endif
+
     glcx->doneCurrent();
+
     bool success = TRUE;
 
     if ( useContext && isValid() && renderCxPm( &pm ) )
@@ -1848,10 +1921,18 @@ QPixmap QGLWidget::renderPixmap( int w, int h, bool useContext )
     if ( wasCurrent )
 	ocx->makeCurrent();
 
-    if ( success )
+    if ( success ) {
+#if defined(Q_WS_X11)
+	if (gl_pixmap_visual != QPaintDevice::x11AppVisual()) {
+	    QImage image = pm.convertToImage();
+	    QPixmap p;
+	    p = image;
+	    return p;
+	}
+#endif
 	return pm;
-    else
-	return nullPm;
+    } else
+	return QPixmap();
 }
 
 
@@ -1947,7 +2028,7 @@ void QGLWidget::glDraw()
 	return;
     makeCurrent();
     if ( glcx->deviceIsPixmap() )
-	glDrawBuffer( GL_FRONT_LEFT );
+	glDrawBuffer( GL_FRONT );
     if ( !glcx->initialized() ) {
 	glInit();
 	QPaintDeviceMetrics dm( glcx->device() );
@@ -2047,7 +2128,7 @@ void QGLWidget::qglClearColor( const QColor& c ) const
     the glTexImage2D() documentation. The width must be 2^m + 2*border
     and the height 2^n + 2*border where m and n are integers and
     border is either 0 or 1.
-    
+
     Another function in the same example uses \e tex1 with OpenGL.
 */
 
@@ -2112,12 +2193,27 @@ int QGLWidget::displayListBase( const QFont & fnt, int listBase )
     // always regenerate font disp. lists for pixmaps - hw accelerated
     // contexts can't handle this otherwise
     bool regenerate = glcx->deviceIsPixmap();
-    QString key = fnt.key() + QString::number( (int)regenerate );
+
+#if 0 // QT_NO_XFTFREETYPE
+    // font color needs to be part of the font cache key when using
+    // antialiased fonts since one set of glyphs needs to be generated
+    // for each font color
+    QString color_key;
+    if (fnt.styleStrategy() != QFont::NoAntialias) {
+	GLfloat color[4];
+	glGetFloatv(GL_CURRENT_COLOR, color);
+	color_key.sprintf("%f_%f_%f",color[0], color[1], color[2]);
+    }
+    QString key = fnt.key() + color_key + QString::number((int) regenerate);
+#else
+    QString key = fnt.key() + QString::number((int) regenerate);
+#endif
+
     if ( !regenerate && (d->displayListCache.find( key ) != d->displayListCache.end()) ) {
 	base = d->displayListCache[ key ];
     } else {
 	int maxBase = listBase - 256;
-	QMapIterator<QString,int> it;
+	QMapConstIterator<QString,int> it;
 	for ( it = d->displayListCache.begin(); it != d->displayListCache.end(); ++it ) {
 	    if ( maxBase < it.data() ) {
 		maxBase = it.data();
@@ -2157,9 +2253,7 @@ int QGLWidget::displayListBase( const QFont & fnt, int listBase )
 void QGLWidget::renderText( int x, int y, const QString & str, const QFont & fnt, int listBase )
 {
     makeCurrent();
-
-    // save GL state
-    glPushAttrib( GL_TRANSFORM_BIT | GL_VIEWPORT_BIT );
+    glPushAttrib( GL_TRANSFORM_BIT | GL_VIEWPORT_BIT | GL_LIST_BIT | GL_CURRENT_BIT );
     glMatrixMode( GL_PROJECTION );
     glPushMatrix();
     glLoadIdentity();
@@ -2168,7 +2262,8 @@ void QGLWidget::renderText( int x, int y, const QString & str, const QFont & fnt
     glPushMatrix();
     glLoadIdentity();
 
-    glRasterPos2i( x, y );
+    glRasterPos2i( 0, 0 );
+    glBitmap(0, 0, 0, 0, x, -y, NULL);
     glListBase( displayListBase( fnt, listBase ) );
     glCallLists( str.length(), GL_UNSIGNED_BYTE, str.local8Bit().data() );
 
@@ -2191,8 +2286,10 @@ void QGLWidget::renderText( double x, double y, double z, const QString & str, c
 {
     makeCurrent();
     glRasterPos3d( x, y, z );
+    glPushAttrib( GL_LIST_BIT );
     glListBase( displayListBase( fnt, listBase ) );
     glCallLists( str.length(), GL_UNSIGNED_BYTE, str.local8Bit().data() );
+    glPopAttrib();
 }
 
 /*****************************************************************************
