@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qmotifstyle.cpp   3.2.3   edited May 27 08:45 $
+** $Id: qt/qmotifstyle.cpp   3.3.4   edited Jan 21 2004 $
 **
 ** Implementation of Motif-like style class
 **
@@ -1361,7 +1361,7 @@ void QMotifStyle::drawComplexControl( ComplexControl control,
 		int y = r.y();
 		int c;
 		QPointArray dotlines;
-		if ( subActive == SC_All && sub == SC_ListViewExpand ) {
+		if ( subActive == (uint)SC_All && sub == SC_ListViewExpand ) {
 		    c = 2;
 		    dotlines.resize(2);
 		    dotlines[0] = QPoint( r.right(), r.top() );
@@ -1383,53 +1383,55 @@ void QMotifStyle::drawComplexControl( ComplexControl control,
 		    // paint stuff in the magical area
 		    QListView* v = item->listView();
 		    while ( child && y < r.height() ) {
-			int lh;
-			if ( !item->multiLinesEnabled() )
-			    lh = child ? child->height() : 0;
-			else
-			    lh = p->fontMetrics().height() + 2 * v->itemMargin();
-			lh = QMAX( lh, QApplication::globalStrut().height() );
-			if ( lh % 2 > 0 )
-			    lh++;
-			linebot = y + lh/2;
-			if ( (child->isExpandable() || child->childCount()) &&
-			     (child->height() > 0) ) {
-			    // needs a box
-			    p->setPen( cg.text() );
-			    p->drawRect( bx-4, linebot-4, 9, 9 );
-			    QPointArray a;
-			    if ( child->isOpen() )
-				a.setPoints( 3, bx-2, linebot-2,
-					     bx, linebot+2,
-					     bx+2, linebot-2 ); //RightArrow
+			if (child->isVisible()) {
+			    int lh;
+			    if ( !item->multiLinesEnabled() )
+				lh = child->height();
 			    else
-				a.setPoints( 3, bx-2, linebot-2,
-					     bx+2, linebot,
-					     bx-2, linebot+2 ); //DownArrow
-			    p->setBrush( cg.text() );
-			    p->drawPolygon( a );
-			    p->setBrush( NoBrush );
-			    // dotlinery
-			    dotlines[c++] = QPoint( bx, linetop );
-			    dotlines[c++] = QPoint( bx, linebot - 5 );
-			    dotlines[c++] = QPoint( bx + 5, linebot );
-			    dotlines[c++] = QPoint( r.width(), linebot );
-			    linetop = linebot + 5;
-			} else {
-			    // just dotlinery
-			    dotlines[c++] = QPoint( bx+1, linebot );
-			    dotlines[c++] = QPoint( r.width(), linebot );
+				lh = p->fontMetrics().height() + 2 * v->itemMargin();
+			    lh = QMAX( lh, QApplication::globalStrut().height() );
+			    if ( lh % 2 > 0 )
+				lh++;
+			    linebot = y + lh/2;
+			    if ( (child->isExpandable() || child->childCount()) &&
+				 (child->height() > 0) ) {
+				// needs a box
+				p->setPen( cg.text() );
+				p->drawRect( bx-4, linebot-4, 9, 9 );
+				QPointArray a;
+				if ( child->isOpen() )
+				    a.setPoints( 3, bx-2, linebot-2,
+						 bx, linebot+2,
+						 bx+2, linebot-2 ); //RightArrow
+				else
+				    a.setPoints( 3, bx-2, linebot-2,
+						 bx+2, linebot,
+						 bx-2, linebot+2 ); //DownArrow
+				p->setBrush( cg.text() );
+				p->drawPolygon( a );
+				p->setBrush( NoBrush );
+				// dotlinery
+				dotlines[c++] = QPoint( bx, linetop );
+				dotlines[c++] = QPoint( bx, linebot - 5 );
+				dotlines[c++] = QPoint( bx + 5, linebot );
+				dotlines[c++] = QPoint( r.width(), linebot );
+				linetop = linebot + 5;
+			    } else {
+				// just dotlinery
+				dotlines[c++] = QPoint( bx+1, linebot );
+				dotlines[c++] = QPoint( r.width(), linebot );
+			    }
+			    y += child->totalHeight();
 			}
-
-			y += child->totalHeight();
 			child = child->nextSibling();
 		    }
 
-		    // Expand line height to edge of rectangle if there's a
-		    // child, and it's visible
-		    if ( child && (child->height() > 0) ) {
+		    // Expand line height to edge of rectangle if there's any
+		    // visible child below
+		    while ( child && child->height() <= 0)
+			child = child->nextSibling();
+		    if ( child )
 			linebot = r.height();
-		    }
 
 		    if ( linetop < linebot ) {
 			dotlines[c++] = QPoint( bx, linetop );
