@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qfileinfo.cpp   3.2.3   edited Aug 21 13:43 $
+** $Id: qt/qfileinfo.cpp   3.3.4   edited Jan 6 14:16 $
 **
 ** Implementation of QFileInfo class
 **
@@ -149,6 +149,9 @@ QFileInfo::QFileInfo()
     Constructs a new QFileInfo that gives information about the given
     file. The \a file can also include an absolute or relative path.
 
+    \warning Some functions might behave in a counter-intuitive way
+    if \a file has a trailing directory separator.
+
     \sa setFile(), isRelative(), QDir::setCurrent(), QDir::isRelativePath()
 */
 
@@ -484,7 +487,7 @@ QDir QFileInfo::dir( bool absPath ) const
 
 
 /*!
-    Returns TRUE if the user can write to the file; otherwise returns FALSE.
+    Returns TRUE if the file is readable; otherwise returns FALSE.
 
     \sa isWritable(), isExecutable(), permission()
 */
