@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qbutton.cpp   3.2.3   edited Sep 30 15:59 $
+** $Id: qt/qbutton.cpp   3.3.4   edited Sep 13 17:32 $
 **
 ** Implementation of QButton widget class
 **
@@ -170,9 +170,10 @@ QTimer *QButton::timer()
 
     QButton provides five signals:
     \list 1
-    \i pressed() is emitted when the left mouse button is pressed while
-       the mouse cursor is inside the button.
-    \i released() is emitted when the left mouse button is released.
+    \i pressed() is emitted when the button is pressed. E.g. with the mouse
+       or when animateClick() is called.
+    \i released() is emitted when the button is released. E.g. when the mouse
+       is released or the cursor is moved outside the widget.
     \i clicked() is emitted when the button is first pressed and then
        released when the accelerator key is typed, or when
        animateClick() is called.
@@ -326,7 +327,9 @@ QTimer *QButton::timer()
     This property will return a QString::null if the button has no
     text. If the text has an ampersand (\&) in it, then an
     accelerator is automatically created for it using the character
-    that follows the '\&' as the accelerator key.
+    that follows the '\&' as the accelerator key. Any previous
+    accelerator will be overwritten, or cleared if no accelerator is
+    defined by the text.
 
     There is no default text.
 */
@@ -434,6 +437,9 @@ QButton::~QButton()
     The QButtonGroup::clicked() signal does the same job, if you want
     to connect several buttons to the same slot.
 
+    \warning Don't launch a model dialog in response to this signal
+    for a button that has \c autoRepeat turned on.
+
     \sa pressed(), released(), toggled() autoRepeat down
 */
 
@@ -718,20 +724,28 @@ void QButton::keyPressEvent( QKeyEvent *e )
     case Key_Up:
     case Key_Left:
 #ifndef QT_NO_BUTTONGROUP
-	if ( group() )
+	if ( group() ) {
 	    group()->moveFocus( e->key() );
-	else
+	} else
 #endif
+	{
+	    QFocusEvent::setReason(QFocusEvent::Backtab);
 	    focusNextPrevChild( FALSE );
+	    QFocusEvent::resetReason();
+	}
 	break;
     case Key_Right:
     case Key_Down:
 #ifndef QT_NO_BUTTONGROUP
-	if ( group() )
+	if ( group() ) {
 	    group()->moveFocus( e->key() );
-	else
+	} else
 #endif
+	{
+	    QFocusEvent::setReason(QFocusEvent::Tab);
 	    focusNextPrevChild( TRUE );
+	    QFocusEvent::resetReason();
+	}
 	break;
     case Key_Escape:
 	if ( buttonDown ) {
@@ -782,7 +796,7 @@ void QButton::mousePressEvent( QMouseEvent *e )
 #endif
 	QGuardedPtr<QTimer> t = timer();
 	emit pressed();
-	if ( repeat && t )
+	if ( t && repeat )
 	    t->start( AUTO_REPEAT_DELAY, TRUE );
     }
 }
@@ -794,9 +808,9 @@ void QButton::mouseReleaseEvent( QMouseEvent *e)
 
 	// clean up apperance if left button has been pressed
 	if (mlbDown || buttonDown) {
-	    mlbDown = FALSE;				
+	    mlbDown = FALSE;
 	    buttonDown = FALSE;
-	    
+
 	    if ( autoMask() )
 		updateMask();
 	    repaint( FALSE );
@@ -809,6 +823,8 @@ void QButton::mouseReleaseEvent( QMouseEvent *e)
 	return;
     if ( d )
 	timer()->stop();
+
+    const bool oldButtonDown = buttonDown;
     mlbDown = FALSE;				// left mouse button up
     buttonDown = FALSE;
     if ( hitButton( e->pos() ) ) {		// mouse release on button
@@ -823,7 +839,8 @@ void QButton::mouseReleaseEvent( QMouseEvent *e)
 #if defined(QT_ACCESSIBILITY_SUPPORT)
 	QAccessible::updateAccessibility( this, 0, QAccessible::StateChanged );
 #endif
-	emit released();
+        if (oldButtonDown)
+            emit released();
     }
 }
 
