@@ -1,11 +1,11 @@
 /**********************************************************************
-** $Id: qt/qcombobox.cpp   3.2.3   edited Aug 28 12:16 $
+** $Id: qt/qcombobox.cpp   3.3.4   edited Nov 25 14:57 $
 **
 ** Implementation of QComboBox widget class
 **
 ** Created : 940426
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2004 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -1257,7 +1257,7 @@ void QComboBox::paintEvent( QPaintEvent * )
 	    p.drawRect( xPos - 5, 4, width() - xPos + 1 , height() - 8 );
     } else if(!d->usingListBox()) {
 	style().drawComplexControl( QStyle::CC_ComboBox, &p, this, rect(), g,
-				    flags, QStyle::SC_All,
+				    flags, (uint)QStyle::SC_All,
 				    (d->arrowDown ?
 				     QStyle::SC_ComboBoxArrow :
 				     QStyle::SC_None ));
@@ -1287,7 +1287,7 @@ void QComboBox::paintEvent( QPaintEvent * )
 	}
     } else {
 	style().drawComplexControl( QStyle::CC_ComboBox, &p, this, rect(), g,
-				    flags, QStyle::SC_All,
+				    flags, (uint)QStyle::SC_All,
 				    (d->arrowDown ?
 				     QStyle::SC_ComboBoxArrow :
 				     QStyle::SC_None ));
@@ -1344,6 +1344,7 @@ void QComboBox::mousePressEvent( QMouseEvent *e )
 	if ( d->usingListBox() ) {
 	    listBox()->blockSignals( TRUE );
 	    qApp->sendEvent( listBox(), e ); // trigger the listbox's autoscroll
+	    listBox()->setCurrentItem(d->current);
 	    listBox()->blockSignals( FALSE );
 	    popup();
 	    if ( arrowRect.contains( e->pos() ) ) {
@@ -1514,7 +1515,7 @@ static int listHeight( QListBox *l, int sl )
 
 void QComboBox::popup()
 {
-    if ( !count() )
+    if ( !count() || d->poppedUp )
 	return;
 
     if( !d->usingListBox() || style().styleHint(QStyle::SH_ComboBox_Popup, this) ) {
@@ -1712,7 +1713,6 @@ bool QComboBox::eventFilter( QObject *object, QEvent *event )
 	    else
 		( (QKeyEvent*)event )->ignore();
 	} else if ( event->type() == QEvent::KeyRelease ) {
-	    d->completeNow = FALSE;
 	    keyReleaseEvent( (QKeyEvent *)event );
 	    return ((QKeyEvent *)event)->isAccepted();
 	} else if ( event->type() == QEvent::FocusIn ) {
@@ -1720,10 +1720,10 @@ bool QComboBox::eventFilter( QObject *object, QEvent *event )
 	} else if ( event->type() == QEvent::FocusOut ) {
 	    focusOutEvent( (QFocusEvent *)event );
 	} else if ( d->useCompletion && d->completeNow ) {
+	    d->completeNow = FALSE;
 	    if ( !d->ed->text().isNull() &&
 		 d->ed->cursorPosition() > d->completeAt &&
 		 d->ed->cursorPosition() == (int)d->ed->text().length() ) {
-		d->completeNow = FALSE;
 		QString ct( d->ed->text() );
 		int i = completionIndex( ct, currentItem() );
 		if ( i > -1 ) {
@@ -1738,8 +1738,8 @@ bool QComboBox::eventFilter( QObject *object, QEvent *event )
 		}
 	    }
 	}
-    } else if ( d->usingListBox() && ( object == d->listBox() ||
-				       object == d->listBox()->viewport() )) {
+    } else if ( d->usingListBox() && ( object == d->listBox() || 
+                                       object == d->listBox()->viewport() )) {
 	QMouseEvent *e = (QMouseEvent*)event;
 	switch( event->type() ) {
 	case QEvent::MouseMove:
@@ -1853,6 +1853,8 @@ bool QComboBox::eventFilter( QObject *object, QEvent *event )
 	case QEvent::MouseButtonDblClick:
 	case QEvent::MouseButtonPress:
 	    if ( !d->popup()->rect().contains( e->pos() ) ) {
+                d->poppedUp = FALSE;
+                d->arrowDown = FALSE;
 		// remove filter, event will take down popup:
 		d->popup()->removeEventFilter( this );
 		// ### uglehack!
@@ -1974,7 +1976,9 @@ void QComboBox::returnPressed()
 
 	switch ( insertionPolicy() ) {
 	case AtCurrent:
-	    if ( s != text( currentItem() ) )
+	    if (count() == 0)
+		insertItem(s);
+	    else if ( s != text( currentItem() ) )
 		changeItem( s, currentItem() );
 	    emit activated( currentItem() );
 	    emit activated( s );
@@ -1992,7 +1996,7 @@ void QComboBox::returnPressed()
 	    c = currentItem();
 	    break;
 	case AfterCurrent:
-	    c = currentItem() + 1;
+	    c = count() == 0 ? 0 : currentItem() + 1;
 	    break;
 	}
 	insertItem( s, c );
@@ -2077,10 +2081,12 @@ void QComboBox::setListBox( QListBox * newListBox )
 {
     clear();
 
-    if ( d->usingListBox() )
+    if ( d->usingListBox() ) {
 	delete d->listBox();
-    else
+    } else {
 	delete d->popup();
+        d->setPopupMenu(0, FALSE);
+    }
 
     newListBox->reparent( this, WType_Popup, QPoint(0,0), FALSE );
     d->setListBox( newListBox );
@@ -2255,8 +2261,8 @@ void QComboBox::setLineEdit( QLineEdit *edit )
     if ( edit->parent() != this )
 	edit->reparent( this, QPoint(0,0), FALSE );
 
-    connect (edit, SIGNAL( textChanged( const QString& ) ),
-	     this, SIGNAL( textChanged( const QString& ) ) );
+    connect (edit, SIGNAL( textChanged(const QString&) ),
+	     this, SIGNAL( textChanged(const QString&) ) );
     connect( edit, SIGNAL(returnPressed()), SLOT(returnPressed()) );
 
     edit->setFrame( FALSE );
@@ -2276,4 +2282,17 @@ void QComboBox::setLineEdit( QLineEdit *edit )
     update();
 }
 
+/*!
+    \reimp
+*/
+void QComboBox::hide()
+{
+    QWidget::hide();
+
+    if (listBox())
+	listBox()->hide();
+    else if (d->popup())
+	d->popup()->hide();
+}
+
 #endif // QT_NO_COMBOBOX
