@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qdockwindow.cpp   3.2.3   edited Oct 30 19:20 $
+** $Id: qt/qdockwindow.cpp   3.3.4   edited Jan 4 19:27 $
 **
 ** Implementation of the QDockWindow class
 **
@@ -731,10 +731,10 @@ void QDockWindowTitleBar::mouseDoubleClickEvent( QMouseEvent * )
     Single clicking the minimized handle will restore the dock window
     to the last dock area that it was docked in.
     \endomit
-    If the user clicks the close button (which appears on floating
-    dock windows by default) the dock window will disappear. You can
-    control whether or not a dock window has a close button with
-    setCloseMode().
+    If the user clicks the close button (which does not appear on
+    dock windows by default - see \l closeMode) the dock window will
+    disappear. You can control whether or not a dock window has a
+    close button with setCloseMode().
 
     QMainWindow provides four dock areas (top, left, right and bottom)
     which can be used by dock windows. For many applications using the
@@ -742,7 +742,8 @@ void QDockWindowTitleBar::mouseDoubleClickEvent( QMouseEvent * )
     QDockArea documentation if you want to create your own dock
     areas.) In QMainWindow a right-click popup menu (the dock window
     menu) is available which lists dock windows and can be used to
-    show or hide them.
+    show or hide them. (The popup menu only lists dock windows that
+    have a \link setCaption() caption\endlink.)
 
     When you construct a dock window you \e must pass it a QDockArea
     or a QMainWindow as its parent if you want it docked. Pass 0 for
@@ -923,8 +924,8 @@ QDockWindow::QDockWindow( QWidget* parent, const char* name, WFlags f )
     parent is a QMainWindow the dock window will be docked in the main
     window's \c Top dock area.
 
-    If \a p is \c OutsideDock, the parent \e must be 0 and the dock
-    window is created as a floating window.
+    If \a p is \c OutsideDock, the dock window is created as a floating
+    window.
 
     We recommend creating the dock area \c InDock with a QMainWindow
     as parent then calling QMainWindow::moveDockWindow() to move the
@@ -1061,8 +1062,8 @@ void QDockWindow::init()
     connect( titleBar, SIGNAL( doubleClicked() ), this, SLOT( dock() ) );
     connect( verHandle, SIGNAL( doubleClicked() ), this, SLOT( undock() ) );
     connect( horHandle, SIGNAL( doubleClicked() ), this, SLOT( undock() ) );
-    connect( this, SIGNAL( orientationChanged( Orientation ) ),
-	     this, SLOT( setOrientation( Orientation ) ) );
+    connect( this, SIGNAL( orientationChanged(Orientation) ),
+	     this, SLOT( setOrientation(Orientation) ) );
 }
 
 /*!
@@ -1328,12 +1329,24 @@ void QDockWindow::updateGui()
 		    hHandleTop->hide();
 		}
 	    }
-	}
+	} else if ( area() ) { // hide resize handles if resizing is disabled
+            if ( orientation() == Horizontal ) {
+                hHandleTop->hide();
+                hHandleBottom->hide();
+            } else {
+                vHandleLeft->hide();
+                vHandleRight->hide();
+            }
+        }
+#ifndef Q_OS_TEMP
 	if ( moveEnabled )
 	    setLineWidth( 1 );
 	else
 	    setLineWidth( 0 );
 	hbox->setMargin( lineWidth() );
+#else
+	hbox->setMargin( 2 );
+#endif
 	widgetResizeHandler->setActive( FALSE );
     }
 }
@@ -1417,6 +1430,7 @@ void QDockWindow::updatePosition( const QPoint &globalPos )
 	QApplication::sendPostedEvents( this, QEvent::LayoutHint );
 	if ( ::qt_cast<QToolBar*>(this) )
 	    adjustSize();
+	clearWState(WState_Resized); // Ensures size is recalculated (non-opaque).
 	show();
 	if ( parentWidget() && isTopLevel() )
 	    parentWidget()->setActiveWindow();
@@ -1619,6 +1633,9 @@ int QDockWindow::closeMode() const
     setHorizontallyStretchable(TRUE) or setResizeEnabled(TRUE).
 
     \sa setResizeEnabled()
+
+    \bug Strecthability is broken. You must call setResizeEnabled(TRUE) to get
+    proper behavior and even then QDockWindow does not limit stretchablilty.
 */
 
 void QDockWindow::setHorizontallyStretchable( bool b )
@@ -1634,6 +1651,9 @@ void QDockWindow::setHorizontallyStretchable( bool b )
     setVerticallyStretchable(TRUE) or setResizeEnabled(TRUE).
 
     \sa setResizeEnabled()
+
+    \bug Strecthability is broken. You must call setResizeEnabled(TRUE) to get
+    proper behavior and even then QDockWindow does not limit stretchablilty.
 */
 
 void QDockWindow::setVerticallyStretchable( bool b )
@@ -1660,6 +1680,9 @@ bool QDockWindow::isVerticallyStretchable() const
     setVerticallyStretchable(), or with setResizeEnabled().
 
     \sa setResizeEnabled()
+
+    \bug Strecthability is broken. You must call setResizeEnabled(TRUE) to get
+    proper behavior and even then QDockWindow does not limit stretchablilty.
 */
 
 bool QDockWindow::isStretchable() const
@@ -1862,8 +1885,10 @@ void QDockWindow::undock( QWidget *w )
     if ( ::qt_cast<QToolBar*>(this) )
 	adjustSize();
     if ( !w ) {
-	if ( !parentWidget() || parentWidget()->isVisible() )
+	if ( !parentWidget() || parentWidget()->isVisible() ) {
+	    clearWState(WState_Resized); // Ensures size is recalculated (opaque).
 	    show();
+	}
     } else {
 	reparent( w, 0, QPoint( 0, 0 ), FALSE );
 	move( -width() - 5, -height() - 5 );
@@ -1980,6 +2005,8 @@ bool QDockWindow::opaqueMoving() const
 void QDockWindow::setCaption( const QString &s )
 {
     titleBar->setCaption( s );
+    verHandle->update();
+    horHandle->update();
 #ifndef QT_NO_WIDGET_TOPEXTRA
     QFrame::setCaption( s );
 #endif
