@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qt/qgroupbox.cpp   3.2.3   edited Jul 4 13:55 $
+** $Id: qt/qgroupbox.cpp   3.3.4   edited Oct 22 2003 $
 **
 ** Implementation of QGroupBox widget class
 **
@@ -360,6 +360,8 @@ void QGroupBox::paintEvent( QPaintEvent *event )
 	if(va & AlignTop)
 	    r.moveBy(0, fm.descent());
 	QColor pen( (QRgb) style().styleHint(QStyle::SH_GroupBox_TextLabelColor, this )  );
+	if (!style().styleHint(QStyle::SH_UnderlineAccelerator, this))
+	    va |= NoAccel;
 	style().drawItem( &paint, r, ShowPrefix | AlignHCenter | va, colorGroup(),
 			  isEnabled(), 0, str, -1, ownPalette() ? 0 : &pen );
 	paint.setClipRegion( event->region().subtract( r ) ); // clip everything but title
@@ -671,7 +673,7 @@ void QGroupBox::fixFocus()
 	     && w->isVisibleTo(this) ) {
 	    if ( w->hasFocus()
 #ifndef QT_NO_RADIOBUTTON
-		 || ( !best && ::qt_cast<QRadioButton*>(w) 
+		 || ( !best && ::qt_cast<QRadioButton*>(w)
 		 && ((QRadioButton*)w)->isChecked() )
 #endif
 		    )
@@ -837,10 +839,10 @@ void QGroupBox::setCheckable( bool b )
 	    d->checkbox = new QCheckBox( title(), this, "qt_groupbox_checkbox" );
 	    setChecked( TRUE );
 	    setChildrenEnabled( TRUE );
-	    connect( d->checkbox, SIGNAL( toggled( bool ) ),
-		     this, SLOT( setChildrenEnabled( bool ) ) );
-	    connect( d->checkbox, SIGNAL( toggled( bool ) ),
-		     this, SIGNAL( toggled( bool ) ) );
+	    connect( d->checkbox, SIGNAL( toggled(bool) ),
+		     this, SLOT( setChildrenEnabled(bool) ) );
+	    connect( d->checkbox, SIGNAL( toggled(bool) ),
+		     this, SIGNAL( toggled(bool) ) );
 	    updateCheckBoxGeometry();
 	}
 	d->checkbox->show();
@@ -912,9 +914,9 @@ void QGroupBox::setChildrenEnabled( bool b )
     QObject *o;
     while( (o = it.current()) ) {
 	++it;
-	if ( o->isWidgetType() 
+	if ( o->isWidgetType()
 #ifndef QT_NO_CHECKBOX
-	     && o != d->checkbox 
+	     && o != d->checkbox
 #endif
 	     ) {
 	    QWidget *w = (QWidget*)o;
