@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qheader.cpp   3.2.3   edited May 21 15:28 $
+** $Id: qt/qheader.cpp   3.3.4   edited Dec 11 01:26 $
 **
 ** Implementation of QHeader widget class (table header)
 **
@@ -528,6 +528,30 @@ int QHeader::findLine( int c )
 	return i;
 }
 
+/*!
+    Returns the handle at position \a p, or -1 if there is no handle at \a p.
+*/
+int QHeader::handleAt(int p)
+{
+    int section = d->sectionAt( p );
+    if ( section >= 0 ) {
+	int GripMargin = (bool)d->resize[ section ] ?
+	    style().pixelMetric( QStyle::PM_HeaderGripMargin ) : 0;
+	int index = d->s2i[section];
+	if ( (index > 0 && p < d->positions[index] + GripMargin) ||
+	    (p > d->positions[index] + d->sizes[section] - GripMargin) ) {
+	    if ( index > 0 && p < d->positions[index]  + GripMargin )
+		section = d->i2s[--index];
+	    // dont show icon if streaching is enabled it is at the end of the last section
+	    if ( d->resize.testBit(section) && (d->fullSize == -2 || index != count() - 1)) {
+		return section;
+	    }
+	}
+    }
+
+    return -1;
+}
+
 /*!
   \obsolete
 
@@ -748,9 +772,6 @@ void QHeader::mouseReleaseEvent( QMouseEvent *e )
 */
 void QHeader::mouseMoveEvent( QMouseEvent *e )
 {
-    int section;
-    bool hit;
-
     int c = orient == Horizontal ? e->pos().x() : e->pos().y();
     c += offset();
 
@@ -760,29 +781,13 @@ void QHeader::mouseMoveEvent( QMouseEvent *e )
 
     switch( state ) {
     case Idle:
-	hit = FALSE;
-	if ( (section = d->sectionAt( c )) >= 0 ) {
-	    int GripMargin = (bool)d->resize[ section ] ?
-		style().pixelMetric( QStyle::PM_HeaderGripMargin ) : 0;
-	    int index = d->s2i[section];
-	    if ( (index > 0 && c < d->positions[index] + GripMargin) ||
-		 (c > d->positions[index] + d->sizes[section] - GripMargin) ) {
-		if ( index > 0 && c < d->positions[index]  + GripMargin )
-		    section = d->i2s[--index];
-		if ( d->resize.testBit(section) ) {
-		    hit = TRUE;
 #ifndef QT_NO_CURSOR
-		    if ( orient == Horizontal )
-			setCursor( splitHCursor );
-		    else
-			setCursor( splitVCursor );
-#endif
-		}
-	    }
-	}
-#ifndef QT_NO_CURSOR
-	if ( !hit )
+	if ( handleAt(c) < 0 )
 	    unsetCursor();
+	else if ( orient == Horizontal )
+	    setCursor( splitHCursor );
+	else
+	    setCursor( splitVCursor );
 #endif
 	break;
     case Blocked:
@@ -827,17 +832,14 @@ void QHeader::mouseMoveEvent( QMouseEvent *e )
 
 void QHeader::mouseDoubleClickEvent( QMouseEvent *e )
 {
-    int c = orient == Horizontal ? e->pos().x() : e->pos().y();
-    c += offset();
+    int p = orient == Horizontal ? e->pos().x() : e->pos().y();
+    p += offset();
     if( reverse() )
-	c = d->lastPos - c;
+	p = d->lastPos - p;
 
-    int sec = -1;
-    if ( handleIdx != -1 )
-	sec = d->i2s[handleIdx];
-    else
-	sec = d->sectionAt( c );
-    emit sectionHandleDoubleClicked( sec );
+    int header = handleAt(p);
+    if (header >= 0)
+	emit sectionHandleDoubleClicked( header );
 }
 
 /*
@@ -972,7 +974,10 @@ void QHeader::setLabel( int section, const QString &s, int size )
 {
     if ( section < 0 || section >= count() )
 	return;
-    d->labels.insert( section, new QString( s ) );
+    if ( s.isNull() )
+	d->labels.remove( section );
+    else
+	d->labels.insert( section, new QString( s ) );
 
     setSectionSizeAndHeight( section, size );
 
@@ -1097,13 +1102,18 @@ QSize QHeader::sectionSizeHint( int section, const QFontMetrics& fm ) const
     QString *label = d->labels[section];
     if ( label ) {
 	int lines = label->contains( '\n' ) + 1;
-	bound.setHeight( fm.height() +  fm.lineSpacing() * (lines - 1) );
 	int w = 0;
-	for ( int i = 0; i < lines; ++i ) {
-	    QString s = label->section( '\n', i, i );
-	    int tmpw = fm.width( s );
-	    w = QMAX( w, tmpw );
-	}
+        if (lines > 1) {
+            bound.setHeight(fm.height() + fm.lineSpacing() * (lines - 1));
+            QStringList list = QStringList::split('\n', *label);
+            for (int i=0; i <(int)list.count(); ++i) {
+                int tmpw = fm.width(*(list.at(i)));
+                w = QMAX(w, tmpw);
+            }
+        } else {
+            bound.setHeight(fm.height());
+            w = fm.width(*label);
+        }
 	bound.setWidth( w );
     }
     int arrowWidth = 0;
@@ -1234,15 +1244,13 @@ QSize QHeader::sizeHint() const
 	width = 0;
 	height = QMAX( height, d->height );
 	for ( int i = 0; i < count(); i++ )
-	    width += d->fullSize < -1 ? d->sizes[i] :
-		     sectionSizeHint( mapToSection( i ), fm ).width();
+	    width += d->sizes[i];
     } else {
 	width = fm.width( ' ' );
 	height = 0;
 	width = QMAX( width, d->height );
 	for ( int i = 0; i < count(); i++ )
-	    height += d->fullSize < -1 ? d->sizes[i] :
-		      sectionSizeHint( mapToSection( i ), fm ).height();
+	    height += d->sizes[i];
     }
     return (style().sizeFromContents(QStyle::CT_Header, this,
 				     QSize(width, height)).expandedTo(QApplication::globalStrut()));
@@ -1456,7 +1464,7 @@ void QHeader::paintSection( QPainter *p, int index, const QRect& fr )
     if ( sectionSize( section ) <= 0 )
 	return;
 
-    QStyle::SFlags flags = ( orient == Horizontal ? QStyle::Style_Horizontal : 0 );
+    QStyle::SFlags flags = (orient == Horizontal ? QStyle::Style_Horizontal : QStyle::Style_Default);
     //pass in some hint about the sort indicator if it is used
     if(d->sortSection != section)
 	flags |= QStyle::Style_Off;
@@ -1465,10 +1473,11 @@ void QHeader::paintSection( QPainter *p, int index, const QRect& fr )
     if(isEnabled())
 	flags |= QStyle::Style_Enabled;
     if(isClickEnabled(section)) {
-	if(index == oldHandleIdx)
+	if(section == d->sortSection)
 	    flags |= QStyle::Style_Sunken; //currently selected
 	if((state == Pressed || state == Moving) && index == handleIdx)
 	    flags |= QStyle::Style_Down; //currently pressed
+
     }
     if(!(flags & QStyle::Style_Down))
 	flags |= QStyle::Style_Raised;
@@ -1545,8 +1554,8 @@ void QHeader::paintSectionLabel( QPainter *p, int index, const QRect& fr )
     int dx = 0, dy = 0;
     QStyle::SFlags flags = QStyle::Style_Default;
     if ( index == handleIdx && ( state == Pressed || state == Moving ) ) {
-	dx = style().pixelMetric( QStyle::PM_ButtonShiftVertical, this );
-	dy = style().pixelMetric( QStyle::PM_ButtonShiftHorizontal, this );
+	dx = style().pixelMetric( QStyle::PM_ButtonShiftHorizontal, this );
+	dy = style().pixelMetric( QStyle::PM_ButtonShiftVertical, this );
 	flags |= QStyle::Style_Sunken;
     }
     if ( isEnabled() )
@@ -1579,9 +1588,11 @@ void QHeader::paintSectionLabel( QPainter *p, int index, const QRect& fr )
 	    flags |= QStyle::Style_Down;
 	else
 	    flags |= QStyle::Style_Up;
+        QRect ar(fr.x() + tw - arrowWidth - 6 + ew, 4, arrowWidth, arrowHeight);
+        if (label(section).isRightToLeft())
+            ar.moveBy( 2*(fr.right() - ar.right()) + ar.width() - fr.width(), 0 );
 	style().drawPrimitive( QStyle::PE_HeaderArrow, p,
-			       QRect(fr.x() + tw - arrowWidth - 6 + ew, 4, arrowWidth, arrowHeight),
-			       colorGroup(), flags, QStyleOption( this ) );
+			       ar, colorGroup(), flags, QStyleOption( this ) );
     }
 }
 
