@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qt/qlabel.cpp   3.2.3   edited Sep 19 16:59 $
+** $Id: qt/qlabel.cpp   3.3.4   edited Mar 15 2004 $
 **
 ** Implementation of QLabel widget class
 **
@@ -584,6 +584,7 @@ QSize QLabel::sizeForWidth( int w ) const
 	    w = xw * 80;
 	else if ( w < 0 )
 	    w = 2000;
+	w -= hextra;
 	br = fm.boundingRect( 0, 0, w ,2000, alignment(), text() );
 	if ( tryWidth && br.height() < 4*fm.lineSpacing() && br.width() > w/2 )
 		br = fm.boundingRect( 0, 0, w/2, 2000, alignment(), text() );
@@ -780,7 +781,7 @@ void QLabel::drawContents( QPainter *p )
 	    cg.setColor( QColorGroup::Text, cg.light() );
 	    doc->draw(p, cr.x()+1, cr.y()+yo+1, cr, cg, 0);
 	}
-	
+
 	// QSimpleRichText always draws with QColorGroup::Text as with
 	// background mode PaletteBase. QLabel typically has
 	// background mode PaletteBackground, so we create a temporary
@@ -833,8 +834,11 @@ void QLabel::drawContents( QPainter *p )
 	    pix = d->pix;
 	}
 #endif
+	int alignment = align;
+	if ((align & ShowPrefix) && !style().styleHint(QStyle::SH_UnderlineAccelerator, this))
+	    alignment |= NoAccel;
 	// ordinary text or pixmap label
-	style().drawItem( p, cr, align, colorGroup(), isEnabled(),
+	style().drawItem( p, cr, alignment, colorGroup(), isEnabled(),
 			  pix, ltext );
     }
 }
@@ -1084,6 +1088,15 @@ QMovie* QLabel::movie() const
 
 #endif  // QT_NO_MOVIE
 
+/*!
+    \property QLabel::backgroundMode
+    \brief the label's background mode
+
+    Get this property with backgroundMode().
+
+    \sa QWidget::setBackgroundMode()
+*/
+
 /*!
     \property QLabel::textFormat
     \brief the label's text format
