@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qpushbutton.cpp   3.0.5   edited Jun 7 16:13 $
+** $Id: qt/qpushbutton.cpp   3.3.4   edited Apr 16 2004 $
 **
 ** Implementation of QPushButton class
 **
@@ -53,174 +53,191 @@
 #endif
 
 /*!
-  \class QPushButton qpushbutton.h
-  \brief The QPushButton widget provides a command button.
+    \class QPushButton qpushbutton.h
+    \brief The QPushButton widget provides a command button.
 
-  \ingroup basic
-  \mainclass
+    \ingroup basic
+    \mainclass
 
-  The push button, or command button, is perhaps the most commonly
-  used widget in any graphical user interface. Push (click) a button
-  to command the computer to perform some action, or to answer a
-  question. Typical buttons are OK, Apply, Cancel, Close, Yes, No and
-  Help.
+    The push button, or command button, is perhaps the most commonly
+    used widget in any graphical user interface. Push (click) a button
+    to command the computer to perform some action, or to answer a
+    question. Typical buttons are OK, Apply, Cancel, Close, Yes, No
+    and Help.
 
-  A command button is rectangular and typically displays a text label
-  describing its action. An underscored character in the label
-  (signified by preceding it with an ampersand in the text) indicates
-  an accelerator key, e.g.
-  \code
-    QPushButton *pb = new QPushButton( "&Download", this );
-  \endcode
-  In this example the accelerator is \e{Alt+D}, and the label text
-  will be displayed as <b><u>D</u>ownload</b>.
+    A command button is rectangular and typically displays a text
+    label describing its action. An underlined character in the label
+    (signified by preceding it with an ampersand in the text)
+    indicates an accelerator key, e.g.
+    \code
+	QPushButton *pb = new QPushButton( "&Download", this );
+    \endcode
+    In this example the accelerator is \e{Alt+D}, and the label text
+    will be displayed as <b><u>D</u>ownload</b>.
 
-   Push buttons can display a textual label or a pixmap, and
-   optionally a small icon. These can be set using the constructors
-   and changed later using setText(), setPixmap() and setIconSet(). If
-   the button is disabled the appearance of the text or pixmap and
-   iconset will be manipulated with respect to the GUI style to make
-   the button look "disabled".
+    Push buttons can display a textual label or a pixmap, and
+    optionally a small icon. These can be set using the constructors
+    and changed later using setText(), setPixmap() and setIconSet().
+    If the button is disabled the appearance of the text or pixmap and
+    iconset will be manipulated with respect to the GUI style to make
+    the button look "disabled".
 
-  A push button emits the signal clicked() when it is activated
-  by the mouse, the Spacebar or by a keyboard accelerator.
-  Connect to this signal to perform the button's action. Push buttons
-  also provide less commonly used signals, for example, pressed()
-  and released().
+    A push button emits the signal clicked() when it is activated by
+    the mouse, the Spacebar or by a keyboard accelerator. Connect to
+    this signal to perform the button's action. Push buttons also
+    provide less commonly used signals, for example, pressed() and
+    released().
 
-  Command buttons in dialogs are by default auto-default buttons,
-  i.e. they become the default push button automatically when they
-  receive the keyboard input focus. A default button is a push
-  button that is activated when the user hits the Enter or Return key
-  in a dialog. You can change this with setAutoDefault(). Note that
-  auto-default buttons reserve a little extra space which is necessary
-  to draw a default-button indicator. If you do not want this space
-  around your buttons, call setAutoDefault(FALSE).
+    Command buttons in dialogs are by default auto-default buttons,
+    i.e. they become the default push button automatically when they
+    receive the keyboard input focus. A default button is a push
+    button that is activated when the user presses the Enter or Return
+    key in a dialog. You can change this with setAutoDefault(). Note
+    that auto-default buttons reserve a little extra space which is
+    necessary to draw a default-button indicator. If you do not want
+    this space around your buttons, call setAutoDefault(FALSE).
 
-  Being so central, the button widget has grown to accommodate a great many
-  variations in the past decade. The Microsoft style guide now
-  shows about ten different states of Windows push buttons and the
-  text implies that there are dozens more when all the combinations of
-  features are taken into consideration.
+    Being so central, the button widget has grown to accommodate a
+    great many variations in the past decade. The Microsoft style
+    guide now shows about ten different states of Windows push buttons
+    and the text implies that there are dozens more when all the
+    combinations of features are taken into consideration.
 
-  The most important modes or states are:
-  \list
-  \i Available or not (grayed out, disabled).
-  \i Standard push button, toggling push button or menu button.
-  \i On or off (only for toggling push buttons).
-  \i Default or normal.  The default button in a dialog can
-  generally be "clicked" using the Enter or Return key.
-  \i Auto-repeat or not.
-  \i Pressed down or not.
-  \endlist
+    The most important modes or states are:
+    \list
+    \i Available or not (grayed out, disabled).
+    \i Standard push button, toggling push button or menu button.
+    \i On or off (only for toggling push buttons).
+    \i Default or normal. The default button in a dialog can generally
+       be "clicked" using the Enter or Return key.
+    \i Auto-repeat or not.
+    \i Pressed down or not.
+    \endlist
 
-  As a general rule, use a push button when the application or dialog
-  window performs an action when the user clicks on it (such as Apply,
-  Cancel, Close and Help) \e and when the widget is supposed to have
-  a wide, rectangular shape with a text label.  Small, typically
-  square buttons that change the state of the window rather than
-  performing an action (such as the buttons in the top-right
-  corner of the QFileDialog) are not command buttons, but tool
-  buttons. Qt provides a special class (QToolButton) for these buttons.
+    As a general rule, use a push button when the application or
+    dialog window performs an action when the user clicks on it (such
+    as Apply, Cancel, Close and Help) \e and when the widget is
+    supposed to have a wide, rectangular shape with a text label.
+    Small, typically square buttons that change the state of the
+    window rather than performing an action (such as the buttons in
+    the top-right corner of the QFileDialog) are not command buttons,
+    but tool buttons. Qt provides a special class (QToolButton) for
+    these buttons.
 
-  If you need toggle behavior (see setToggleButton()) or a button
-  that auto-repeats the activation signal when being pushed down like
-  the arrows in a scroll bar (see setAutoRepeat()), a command button is
-  probably not what you want. When in doubt, use a tool button.
+    If you need toggle behavior (see setToggleButton()) or a button
+    that auto-repeats the activation signal when being pushed down
+    like the arrows in a scroll bar (see setAutoRepeat()), a command
+    button is probably not what you want. When in doubt, use a tool
+    button.
 
-  A variation of a command button is a menu button. These provide not
-  just one command, but several, since when they are clicked they pop
-  up a menu of options. Use the method setPopup() to associate a popup
-  menu with a push button.
+    A variation of a command button is a menu button. These provide
+    not just one command, but several, since when they are clicked
+    they pop up a menu of options. Use the method setPopup() to
+    associate a popup menu with a push button.
 
-  Other classes of buttons are option buttons (see QRadioButton) and
-  check boxes (see QCheckBox).
+    Other classes of buttons are option buttons (see QRadioButton) and
+    check boxes (see QCheckBox).
 
-  <img src="qpushbt-m.png"> <img src="qpushbt-w.png">
+    <img src="qpushbt-m.png"> <img src="qpushbt-w.png">
 
-  In Qt, the QButton base class provides most of the modes and other
-  API, and QPushButton provides GUI logic.  See QButton for more
-  information about the API.
+    In Qt, the QButton abstract base class provides most of the modes
+    and other API, and QPushButton provides GUI logic. See QButton for
+    more information about the API.
 
-  \important text, setText, text, pixmap, setPixmap, accel, setAccel,
-  isToggleButton, setDown, isDown, isOn, state, autoRepeat,
-  isExclusiveToggle, group, setAutoRepeat, toggle, pressed, released,
-  clicked, toggled, state stateChanged
+    \important text, setText, text, pixmap, setPixmap, accel, setAccel,
+    isToggleButton, setDown, isDown, isOn, state, autoRepeat,
+    isExclusiveToggle, group, setAutoRepeat, toggle, pressed, released,
+    clicked, toggled, state stateChanged
 
-  \sa QToolButton, QRadioButton QCheckBox
-  \link guibooks.html#fowler GUI Design Handbook: Push Button\endlink
+    \sa QToolButton, QRadioButton QCheckBox
+    \link guibooks.html#fowler GUI Design Handbook: Push Button\endlink
 */
 
-/*! \property QPushButton::autoDefault
+/*!
+    \property QPushButton::autoDefault
     \brief whether the push button is the auto default button
 
     If this property is set to TRUE then the push button is the auto
     default button in a dialog.
 
-    An auto-default button automatically becomes the default push button
-    in a dialog when it receives the keyboard input focus.
-
     In some GUI styles a default button is drawn with an extra frame
-    around it, up to 3 pixels or more. Qt automatically keeps this space
-    free around auto-default buttons, i.e., auto-default buttons may have
-    a slightly larger size hint.
+    around it, up to 3 pixels or more. Qt automatically keeps this
+    space free around auto-default buttons, i.e. auto-default buttons
+    may have a slightly larger size hint.
 
-  This property's default is TRUE  for buttons that have a QDialog parent,
-  else FALSE.
+    This property's default is TRUE for buttons that have a QDialog
+    parent; otherwise it defaults to FALSE.
+
+    See the \l default property for details of how \l default and
+    auto-default interact.
 */
 
-/*! \property QPushButton::autoMask
+/*!
+    \property QPushButton::autoMask
     \brief whether the button is automatically masked
 
     \sa QWidget::setAutoMask()
 */
 
-/*! \property QPushButton::default
+/*!
+    \property QPushButton::default
     \brief whether the push button is the default button
 
-  If this property is set to TRUE then the push button will be pressed if the
-  user hits the Enter key in a dialog.
+    If this property is set to TRUE then the push button will be
+    pressed if the user presses the Enter (or Return) key in a dialog.
 
-    The current default button gets clicked when the user presses the
-    Enter key, independently of which widget in the dialog currently
-    has the keyboard input focus.  Only one push button can at any time
-    be the default button.  This button is then displayed with an
-    additional frame (depending on the GUI style).
+    Regardless of focus, if the user presses Enter: If there is a
+    default button the default button is pressed; otherwise, if
+    there are one or more \l autoDefault buttons the first \l autoDefault
+    button that is next in the tab order is pressed. If there are no
+    default or \l autoDefault buttons only pressing Space on a button
+    with focus, mouse clicking, or using an accelerator will press a
+    button.
+
+    In a dialog, only one push button at a time can be the default
+    button. This button is then displayed with an additional frame
+    (depending on the GUI style).
 
     The default button behavior is provided only in dialogs. Buttons
-    can always be clicked from the keyboard by pressing Return or the
-    spacebar when the button has focus.
+    can always be clicked from the keyboard by pressing Enter (or
+    Return) or the Spacebar when the button has focus.
 
-  This property's default is FALSE.
+    This property's default is FALSE.
 */
 
-/*! \property QPushButton::flat
+/*!
+    \property QPushButton::flat
     \brief whether the border is disabled
 
     This property's default is FALSE.
 */
 
-/*! \property QPushButton::iconSet
+/*!
+    \property QPushButton::iconSet
     \brief the icon set on the push button
 
-  This property will return 0 if the push button has no iconset
+    This property will return 0 if the push button has no iconset.
 */
 
-/*! \property QPushButton::on
+/*!
+    \property QPushButton::on
     \brief whether the push button is toggled
 
-  This property should only be set for toggle push buttons. The
-  default value is FALSE.
-  \sa isOn(), toggle(), toggled(), isToggleButton()
+    This property should only be set for toggle push buttons. The
+    default value is FALSE.
+
+    \sa isOn(), toggle(), toggled(), isToggleButton()
 */
 
-/*! \property QPushButton::toggleButton
+/*!
+    \property QPushButton::toggleButton
     \brief whether the button is a toggle button
 
-  Toggle buttons have an on/off state similar to \link QCheckBox check
-  boxes. \endlink A push button is initially not a toggle button.
+    Toggle buttons have an on/off state similar to \link QCheckBox
+    check boxes. \endlink A push button is initially not a toggle
+    button.
 
-  \sa setOn(), toggle(), isToggleButton() toggled()
+    \sa setOn(), toggle(), isToggleButton() toggled()
 */
 
 /*! \property QPushButton::menuButton
@@ -240,17 +257,22 @@ public:
     {}
     ~QPushButtonPrivate()
     {
+#ifndef QT_NO_ICONSET
 	delete iconset;
+#endif
     }
+#ifndef QT_NO_POPUPMENU
     QGuardedPtr<QPopupMenu> popup;
+#endif
     QIconSet* iconset;
 };
 
 
 /*!
-  Constructs a push button with no text.
+    Constructs a push button with no text.
 
-  The \a parent and \a name arguments are sent to the QWidget constructor.
+    The \a parent and \a name arguments are sent on to the QWidget
+    constructor.
 */
 
 QPushButton::QPushButton( QWidget *parent, const char *name )
@@ -260,8 +282,8 @@ QPushButton::QPushButton( QWidget *parent, const char *name )
 }
 
 /*!
-  Constructs a push button called \a name with the parent \a parent
-  and the text \a text.
+    Constructs a push button called \a name with the parent \a parent
+    and the text \a text.
 */
 
 QPushButton::QPushButton( const QString &text, QWidget *parent,
@@ -274,14 +296,15 @@ QPushButton::QPushButton( const QString &text, QWidget *parent,
 
 
 /*!
-  Constructs a push button with an \a icon and a \a text.
+    Constructs a push button with an \a icon and a \a text.
 
-  Note that you can also pass a QPixmap object as an icon (thanks to
-  the implicit type conversion provided by C++).
+    Note that you can also pass a QPixmap object as an icon (thanks to
+    the implicit type conversion provided by C++).
 
-  The \a parent and \a name arguments are sent to the QWidget constructor.
+    The \a parent and \a name arguments are sent to the QWidget
+    constructor.
 */
-
+#ifndef QT_NO_ICONSET
 QPushButton::QPushButton( const QIconSet& icon, const QString &text,
 			  QWidget *parent, const char *name )
 	: QButton( parent, name )
@@ -290,12 +313,12 @@ QPushButton::QPushButton( const QIconSet& icon, const QString &text,
     setText( text );
     setIconSet( icon );
 }
-
+#endif
 
 
 /*!
-  Destroys the push button
- */
+    Destroys the push button.
+*/
 QPushButton::~QPushButton()
 {
     delete d;
@@ -309,7 +332,7 @@ void QPushButton::init()
     hasMenuArrow = FALSE;
     flt = FALSE;
 #ifndef QT_NO_DIALOG
-    autoDefButton = topLevelWidget()->inherits("QDialog");
+    autoDefButton = ::qt_cast<QDialog*>(topLevelWidget()) != 0;
 #else
     autoDefButton = FALSE;
 #endif
@@ -363,18 +386,18 @@ void QPushButton::setDefault( bool enable )
 	return;					// no change
     defButton = enable;
 #ifndef QT_NO_DIALOG
-    if ( defButton && topLevelWidget()->inherits( "QDialog" ) )
- 	((QDialog*)topLevelWidget())->setDefault( this );
+    if ( defButton && ::qt_cast<QDialog*>(topLevelWidget()) )
+ 	((QDialog*)topLevelWidget())->setMainDefault( this );
 #endif
     update();
 #if defined(QT_ACCESSIBILITY_SUPPORT)
     QAccessible::updateAccessibility( this, 0, QAccessible::StateChanged );
 #endif
-
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 QSize QPushButton::sizeHint() const
 {
@@ -383,13 +406,14 @@ QSize QPushButton::sizeHint() const
     int w = 0, h = 0;
 
     // calculate contents size...
+#ifndef QT_NO_ICONSET
     if ( iconSet() && !iconSet()->isNull() ) {
 	int iw = iconSet()->pixmap( QIconSet::Small, QIconSet::Normal ).width() + 4;
 	int ih = iconSet()->pixmap( QIconSet::Small, QIconSet::Normal ).height();
 	w += iw;
 	h = QMAX( h, ih );
     }
-
+#endif
     if ( isMenuButton() )
 	w += style().pixelMetric(QStyle::PM_MenuButtonIndicator, this);
 
@@ -415,49 +439,56 @@ QSize QPushButton::sizeHint() const
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QPushButton::move( int x, int y )
 {
     QWidget::move( x, y );
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QPushButton::move( const QPoint &p )
 {
     move( p.x(), p.y() );
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QPushButton::resize( int w, int h )
 {
     QWidget::resize( w, h );
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QPushButton::resize( const QSize &s )
 {
     resize( s.width(), s.height() );
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QPushButton::setGeometry( int x, int y, int w, int h )
 {
     QWidget::setGeometry( x, y, w, h );
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QPushButton::setGeometry( const QRect &r )
 {
     QWidget::setGeometry( r );
 }
 
-/*!\reimp
+/*!
+    \reimp
  */
 void QPushButton::resizeEvent( QResizeEvent * )
 {
@@ -465,7 +496,8 @@ void QPushButton::resizeEvent( QResizeEvent * )
 	updateMask();
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QPushButton::drawButton( QPainter *paint )
 {
@@ -489,10 +521,10 @@ void QPushButton::drawButton( QPainter *paint )
 					x(), y() );
 		paint->drawTiledPixmap( 0, height()-diw, width(), diw,
 					*parentWidget()->backgroundPixmap(),
-					x(), y() );
+					x(), y()+height() );
 		paint->drawTiledPixmap( width()-diw, 0, diw, height(),
 					*parentWidget()->backgroundPixmap(),
-					x(), y() );
+					x()+width(), y() );
 	    } else {
 		paint->fillRect( 0, 0, width(), diw,
 				 colorGroup().brush(QColorGroup::Background) );
@@ -528,7 +560,8 @@ void QPushButton::drawButton( QPainter *paint )
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QPushButton::drawButtonLabel( QPainter *paint )
 {
@@ -553,7 +586,8 @@ void QPushButton::drawButtonLabel( QPainter *paint )
 }
 
 
-/*!\reimp
+/*!
+    \reimp
  */
 void QPushButton::updateMask()
 {
@@ -568,45 +602,51 @@ void QPushButton::updateMask()
     setMask( bm );
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QPushButton::focusInEvent( QFocusEvent *e )
 {
-    if ( autoDefButton )
-	setDefault( TRUE );
-#if 0
-    else {
-	if ( topLevelWidget()->inherits("QDialog") )
-	    ((QDialog*)topLevelWidget())->hideDefault();
-    }
+    if (autoDefButton && !defButton) {
+	defButton = TRUE;
+#ifndef QT_NO_DIALOG
+	if ( defButton && ::qt_cast<QDialog*>(topLevelWidget()) )
+ 	    ((QDialog*)topLevelWidget())->setDefault( this );
 #endif
+    }
     QButton::focusInEvent( e );
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QPushButton::focusOutEvent( QFocusEvent *e )
 {
 #ifndef QT_NO_DIALOG
     if ( defButton && autoDefButton ) {
-	if ( topLevelWidget()->inherits("QDialog") )
+	if ( ::qt_cast<QDialog*>(topLevelWidget()) )
 	    ((QDialog*)topLevelWidget())->setDefault( 0 );
     }
 #endif
 
     QButton::focusOutEvent( e );
+#ifndef QT_NO_POPUPMENU
     if ( popup() && popup()->isVisible() )	// restore pressed status
 	setDown( TRUE );
+#endif
 }
 
+
+#ifndef QT_NO_POPUPMENU
 /*!
-  Associates the popup menu \a popup with this push button and
-  thus turns it into a menu button.
+    Associates the popup menu \a popup with this push button. This
+    turns the button into a menu button.
 
-  Ownership of the popup menu is \e not transferred to the push button.
+    Ownership of the popup menu is \e not transferred to the push
+    button.
 
-  \sa popup()
- */
+    \sa popup()
+*/
 void QPushButton::setPopup( QPopupMenu* popup )
 {
     if ( !d )
@@ -615,11 +655,10 @@ void QPushButton::setPopup( QPopupMenu* popup )
 	connect( this, SIGNAL( pressed() ), this, SLOT( popupPressed() ) );
 
     d->popup = popup;
-    autoDefButton = FALSE;
     setIsMenuButton( popup != 0 );
 }
-
-
+#endif //QT_NO_POPUPMENU
+#ifndef QT_NO_ICONSET
 void QPushButton::setIconSet( const QIconSet& icon )
 {
     if ( !d )
@@ -643,12 +682,13 @@ QIconSet* QPushButton::iconSet() const
 {
     return d ? d->iconset : 0;
 }
-
+#endif // QT_NO_ICONSET
+#ifndef QT_NO_POPUPMENU
 /*!
-  Returns the button's associated popup menu or 0 if no popup menu has
-  been defined.
+    Returns the button's associated popup menu or 0 if no popup menu
+    has been set.
 
-  \sa setPopup()
+    \sa setPopup()
 */
 QPopupMenu* QPushButton::popup() const
 {
@@ -662,10 +702,9 @@ void QPushButton::popupPressed()
 	bool horizontal = TRUE;
 	bool topLeft = TRUE;			// ### always TRUE
 #ifndef QT_NO_TOOLBAR
-	if ( parentWidget() && parentWidget()->inherits("QToolBar") ) {
-	    if ( ( (QToolBar*) parentWidget() )->orientation() == Vertical )
-		horizontal = FALSE;
-	}
+	QToolBar *tb = ::qt_cast<QToolBar*>(parentWidget());
+	if ( tb && tb->orientation() == Vertical )
+	    horizontal = FALSE;
 #endif
 	if ( horizontal ) {
 	    if ( topLeft ) {
@@ -696,6 +735,7 @@ void QPushButton::popupPressed()
 	setDown( FALSE );
     }
 }
+#endif
 
 void QPushButton::setFlat( bool f )
 {
