@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qsplashscreen.cpp   3.2.3   edited Nov 4 12:11 $
+** $Id: qt/qsplashscreen.cpp   3.3.4   edited Jan 10 15:42 $
 **
 ** Definition of QSplashScreen class
 **
@@ -64,8 +64,9 @@ public:
    networking applications that take time to establish connections) to
    provide the user with feedback that the application is loading.
 
-   The splash screen appears centered on the screen, on top of all
-   other windows.
+   The splash screen appears centered on the screen. It may be useful to add
+   the \c WStyle_StaysOnTop if you desire to keep above all the windows in the
+   GUI.
 
    Some X11 window managers do not support the "stays on top" flag. A
    solution is to set up a timer that periodically calls raise() on
@@ -106,7 +107,8 @@ public:
    call QApplication::processEvents() to receive the mouse clicks.
 
    \code
-   QSplashScreen *splash = new QSplashScreen( "splash.png" );
+   QPixmap pixmap( "splash.png" );
+   QSplashScreen *splash = new QSplashScreen( pixmap );
    splash->show();
    ... // Loading some items
    splash->message( "Loaded modules" );
@@ -122,7 +124,7 @@ public:
     Construct a splash screen that will display the \a pixmap.
 
     There should be no need to set the widget flags, \a f, except
-    perhaps \c WDestructiveClose.
+    perhaps \c WDestructiveClose or \c WStyle_StaysOnTop.
 */
 QSplashScreen::QSplashScreen( const QPixmap &pixmap, WFlags f )
     : QWidget( 0, 0, WStyle_Customize | WStyle_Splash | f )
@@ -202,8 +204,8 @@ void QSplashScreen::clear()
 }
 
 /*!
-    Makes the splash screen wait until the widget \a mainWin is
-    displayed before calling close() on itself.
+    Makes the splash screen wait until the widget \a mainWin is displayed
+    before calling close() on itself.
 */
 void QSplashScreen::finish( QWidget *mainWin )
 {
