@@ -1,9 +1,9 @@
 /****************************************************************************
-** $Id:  qt/qtabbar.cpp   3.0.5   edited Jul 3 16:41 $
+** $Id: qt/qtabbar.cpp   3.3.4   edited Feb 10 2004 $
 **
 ** Implementation of QTab and QTabBar classes
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -45,24 +45,25 @@
 #include "qiconset.h"
 #include "qcursor.h"
 #include "../kernel/qinternal_p.h"
-
-#include <ctype.h>
+#if defined(QT_ACCESSIBILITY_SUPPORT)
+#include "qaccessible.h"
+#endif
 
 
 /*!
-  \class QTab qtabbar.h
-  \ingroup advanced
-  \brief The QTab class provides the structures in a QTabBar.
+    \class QTab qtabbar.h
+    \brief The QTab class provides the structures in a QTabBar.
 
+    \ingroup advanced
 
-  This class is used for custom QTabBar tab headings.
+    This class is used for custom QTabBar tab headings.
 
-  \sa QTabBar
+    \sa QTabBar
 */
 
 
-/*! \fn QTab::QTab()
-  Constructs an empty tab.  All fields are set to empty.
+/*!
+    Constructs an empty tab. All fields are set to empty.
 */
 
 QTab::QTab()
@@ -73,8 +74,8 @@ QTab::QTab()
 {
 }
 
-/*! \fn QTab::QTab( const QString& text )
-  Constructs a tab with the text, \a text.
+/*!
+    Constructs a tab with the text \a text.
 */
 
 QTab::QTab( const QString &text )
@@ -86,8 +87,8 @@ QTab::QTab( const QString &text )
 {
 }
 
-/*! \fn QTab::QTab( const QIconSet& icon, const QString& text )
-  Constructs a tab with an \a icon and the text, \a text.
+/*!
+    Constructs a tab with an \a icon and the text, \a text.
 */
 
 QTab::QTab( const QIconSet& icon, const QString& text )
@@ -100,49 +101,59 @@ QTab::QTab( const QIconSet& icon, const QString& text )
 }
 
 /*!
-  \fn QString QTab::text() const
-  Return the text of the QTab label.
+    \fn QString QTab::text() const
+
+    Returns the text of the QTab label.
 */
 
 /*!
-  \fn QIconSet QTab::iconSet() const
-  Return the QIconSet of the QTab.
+    \fn QIconSet QTab::iconSet() const
+
+    Return the QIconSet of the QTab.
 */
 
 /*!
-  \fn void QTab::setRect( const QRect &rect )
-  Set the QTab QRect to \a rect.
+    \fn void QTab::setRect( const QRect &rect )
+
+    Set the QTab QRect to \a rect.
 */
 
 /*!
-  \fn QRect QTab::rect() const
-  Return the QRect for the QTab.
+    \fn QRect QTab::rect() const
+
+    Return the QRect for the QTab.
 */
 
 /*!
-  \fn void QTab::setEnabled( bool enable )
-  If \a enable is TRUE enable the QTab, otherwise disable it.
+    \fn void QTab::setEnabled( bool enable )
+
+    If \a enable is TRUE enable the QTab, otherwise disable it.
 */
 
 /*!
-  \fn bool QTab::isEnabled() const
-  Returns TRUE if the QTab is enabled, otherwise return FALSE.
+    \fn bool QTab::isEnabled() const
+
+    Returns TRUE if the QTab is enabled; otherwise returns FALSE.
 */
 
 /*!
-  \fn void QTab::setIdentifier( int i )
-  Set the identifier for the QTab to \a i.
-  Each identifier for a QTabBar must be unique
+    \fn void QTab::setIdentifier( int i )
+
+    Set the identifier for the QTab to \a i. Each QTab's identifier
+    within a QTabBar must be unique.
 */
 
 /*!
-  \fn int QTab::identifier() const
-  Return the identifier for the QTab.
+    \fn int QTab::identifier() const
+
+    Return the QTab's identifier.
 */
 
 
 
-/*! Destroys the tab and frees up all allocated resources */
+/*!
+    Destroys the tab and frees up all allocated resources.
+*/
 
 QTab::~QTab()
 {
@@ -151,65 +162,67 @@ QTab::~QTab()
 }
 
 /*!
-  \class QTabBar qtabbar.h
+    \class QTabBar qtabbar.h
+    \brief The QTabBar class provides a tab bar, e.g. for use in tabbed dialogs.
 
-  \brief The QTabBar class provides a tab bar, e.g. for use in tabbed dialogs.
+    \ingroup advanced
 
-  \ingroup advanced
+    QTabBar is straightforward to use; it draws the tabs using one of
+    the predefined \link QTabBar::Shape shapes\endlink, and emits a
+    signal when a tab is selected. It can be subclassed to tailor the
+    look and feel. Qt also provides a ready-made \l{QTabWidget} and a
+    \l{QTabDialog}.
 
-  QTabBar is straightforward to use; it draws the tabs using one of
-  the predefined \link QTabBar::Shape shapes\endlink, and emits a
-  signal when a tab is selected.  It can be subclassed to tailor the
-  look and feel.
+    The choice of tab shape is a matter of taste, although tab dialogs
+    (for preferences and similar) invariably use \c RoundedAbove;
+    nobody uses \c TriangularAbove. Tab controls in windows other than
+    dialogs almost always use either \c RoundedBelow or \c
+    TriangularBelow. Many spreadsheets and other tab controls in which
+    all the pages are essentially similar use \c TriangularBelow,
+    whereas \c RoundedBelow is used mostly when the pages are
+    different (e.g. a multi-page tool palette).
 
-  The choice of tab shape is a matter of taste, although tab dialogs
-  (preferences and the like) invariably use \c RoundedAbove, and
-  nobody uses \c TriangularAbove.  Tab controls in windows other than
-  dialogs almost always use either \c RoundedBelow or \c
-  TriangularBelow.  Many spreadsheets and other tab controls in which
-  all the pages are essentially similar use \c TriangularBelow,
-  whereas \c RoundedBelow is used mostly when the pages are different
-  (e.g. a multi-page tool palette).
+    The most important part of QTabBar's API is the selected() signal.
+    This is emitted whenever the selected page changes (even at
+    startup, when the selected page changes from 'none'). There is
+    also a slot, setCurrentTab(), which can be used to select a page
+    programmatically.
 
-  The most important part of QTabBar's API is the signal selected().
-  This is emitted whenever the selected page changes (even at startup,
-  when the selected page changes from 'none').  There is also a slot,
-  setCurrentTab(), which can be used to select a page
-  programmatically.
+    QTabBar creates automatic accelerator keys in the manner of
+    QButton; e.g. if a tab's label is "\&Graphics", Alt+G becomes an
+    accelerator key for switching to that tab.
 
-  QTabBar creates automatic accelerator keys in the manner of QButton;
-  e.g. if a tab's label is "\&Graphics", Alt+G becomes an accelerator
-  key for switching to that tab.
+    The following virtual functions may need to be reimplemented:
+    \list
+    \i paint() paints a single tab. paintEvent() calls paint() for
+    each tab so that any overlap will look right.
+    \i addTab() creates a new tab and adds it to the bar.
+    \i selectTab() decides which tab, if any, the user selects with the mouse.
+    \endlist
 
-  The following virtual functions may need to be reimplemented:
-  \list
-  \i paint() paints a single tab.  paintEvent() calls paint() for
-  each tab so that any overlap will look right.
-  \i addTab() creates a new tab and adds it to the bar.
-  \i selectTab() decides which tab, if any, the user selects with the mouse.
-  \endlist
+    The index of the current tab is returned by currentTab(). The tab
+    with a particular index is returned by tabAt(), the tab with a
+    particular id is returned by tab(). The index of a tab is returned
+    by indexOf(). The current tab can be set by index or tab pointer
+    using one of the setCurrentTab() functions.
 
-  The index of the current tab is returned by currentTab(). The tab
-  with a particular index is returned by tabAt(), the tab with a
-  particular id is returned by tab(). The index of a tab is returned
-  by indexOf(). The current tab can be set by index or tab pointer
-  using one of the setCurrentTab() functions.
-
-  <img src=qtabbar-m.png> <img src=qtabbar-w.png>
+    <img src=qtabbar-m.png> <img src=qtabbar-w.png>
 */
 
-/*! \enum QTabBar::Shape
-  This enum type lists the built-in shapes supported by QTabBar:
+/*!
+    \enum QTabBar::Shape
 
-  \value RoundedAbove  the normal rounded look above the pages
+    This enum type lists the built-in shapes supported by QTabBar:
 
-  \value RoundedBelow  the normal rounded look below the pages
+    \value RoundedAbove  the normal rounded look above the pages
 
-  \value TriangularAbove  triangular tabs above the pages (very
-  unusual; included for completeness)
+    \value RoundedBelow  the normal rounded look below the pages
 
-  \value TriangularBelow  triangular tabs similar to those used in
-  the spreadsheet Excel, for example
+    \value TriangularAbove  triangular tabs above the pages (very
+    unusual; included for completeness)
+
+    \value TriangularBelow  triangular tabs similar to those used in
+    the Excel spreadsheet, for example
 */
 
 class QTabBarToolTip;
@@ -224,6 +237,7 @@ struct QTabPrivate {
     QTabBar::Shape s;
     QToolButton* rightB;
     QToolButton* leftB;
+    int btnWidth;
     bool  scrolls;
     QTabBarToolTip * toolTips;
 };
@@ -291,25 +305,30 @@ private:
 #endif
 
 /*!
-  \fn void QTabBar::selected( int id )
+    \fn void QTabBar::selected( int id )
 
-  QTabBar emits this signal whenever any tab is selected, whether by
-  the program or by the user.  The argument \a id is the id of the tab
-  as returned by addTab().
+    QTabBar emits this signal whenever any tab is selected, whether by
+    the program or by the user. The argument \a id is the id of the
+    tab as returned by addTab().
 
-  show() is guaranteed to emit this signal; you can display
-  your page in a slot connected to this signal.
+    show() is guaranteed to emit this signal; you can display your
+    page in a slot connected to this signal.
 */
 
+/*!
+    \fn void QTabBar::layoutChanged()
+
+    QTabBar emits the signal whenever the layout of the tab bar has
+    been recalculated, for example when the contents of a tab change.
+*/
 
 /*!
-  Constructs a new, empty tab bar; the \a parent and \a name arguments
-  are passed on to the QWidget constructor.
+    Constructs a new, empty tab bar; the \a parent and \a name
+    arguments are passed on to the QWidget constructor.
 */
 
 QTabBar::QTabBar( QWidget * parent, const char *name )
-    : QWidget( parent, name, WRepaintNoErase | WResizeNoErase |
-	       WNoMousePropagation  )
+    : QWidget( parent, name, WNoAutoErase | WNoMousePropagation  )
 {
     d = new QTabPrivate;
     d->pressed = 0;
@@ -319,6 +338,7 @@ QTabBar::QTabBar( QWidget * parent, const char *name )
 #ifndef QT_NO_ACCEL
     d->a = new QAccel( this, "tab accelerators" );
     connect( d->a, SIGNAL(activated(int)), this, SLOT(setCurrentTab(int)) );
+    connect( d->a, SIGNAL(activatedAmbiguously(int)), this, SLOT(setCurrentTab(int)) );
 #endif
     d->s = RoundedAbove;
     d->scrolls = FALSE;
@@ -328,6 +348,7 @@ QTabBar::QTabBar( QWidget * parent, const char *name )
     d->rightB = new QToolButton( RightArrow, this, "qt_right_btn" );
     connect( d->rightB, SIGNAL( clicked() ), this, SLOT( scrollTabs() ) );
     d->rightB->hide();
+    d->btnWidth = style().pixelMetric(QStyle::PM_TabBarScrollButtonWidth, this);
     l = new QPtrList<QTab>;
     lstatic = new QPtrList<QTab>;
     lstatic->setAutoDelete( TRUE );
@@ -337,7 +358,7 @@ QTabBar::QTabBar( QWidget * parent, const char *name )
 
 
 /*!
-  Destroys the tab control, freeing memory used.
+    Destroys the tab control, freeing memory used.
 */
 
 QTabBar::~QTabBar()
@@ -356,16 +377,16 @@ QTabBar::~QTabBar()
 
 
 /*!
-  Adds the tab, \a newTab, to the tab control.
+    Adds the tab, \a newTab, to the tab control.
 
-  Sets \a newTab's id to a new id and places the tab just to the right of the
-  existing tabs. If the tab's label contains an ampersand, the letter
-  following the ampersand is used as an accelerator for the tab, e.g.
-  if the label is "Bro&wse" then Alt+W becomes an accelerator which
-  will move the focus to this tab.
-  Returns the id.
+    Sets \a newTab's id to a new id and places the tab just to the
+    right of the existing tabs. If the tab's label contains an
+    ampersand, the letter following the ampersand is used as an
+    accelerator for the tab, e.g. if the label is "Bro\&wse" then
+    Alt+W becomes an accelerator which will move the focus to this
+    tab. Returns the id.
 
-  \sa insertTab()
+    \sa insertTab()
 */
 
 int QTabBar::addTab( QTab * newTab )
@@ -375,23 +396,26 @@ int QTabBar::addTab( QTab * newTab )
 
 
 /*!
-  Inserts the tab, \a newTab, into the tab control.
+    Inserts the tab, \a newTab, into the tab control.
 
-  If \a index is not specified, the tab is simply added. Otherwise
-  it's inserted at the specified position.
+    If \a index is not specified, the tab is simply appended.
+    Otherwise it's inserted at the specified position.
 
-  Sets \a newTab's id to a new id. If the tab's label contains an
-  ampersand, the letter following the ampersand is used as an
-  accelerator for the tab, e.g. if the label is "Bro&wse" then Alt+W
-  becomes an accelerator which will move the focus to this tab.
-  Returns the id.
+    Sets \a newTab's id to a new id. If the tab's label contains an
+    ampersand, the letter following the ampersand is used as an
+    accelerator for the tab, e.g. if the label is "Bro\&wse" then
+    Alt+W becomes an accelerator which will move the focus to this
+    tab. Returns the id.
 
-  \sa addTab()
+    \sa addTab()
 */
 
 int QTabBar::insertTab( QTab * newTab, int index )
 {
     newTab->id = d->id++;
+    if ( !tab( d->focus ) )
+	d->focus = newTab->id;
+
     newTab->setTabBar( this );
     l->insert( 0, newTab );
     if ( index < 0 || index > int(lstatic->count()) )
@@ -414,7 +438,7 @@ int QTabBar::insertTab( QTab * newTab, int index )
 
 
 /*!
-  Removes tab \a t from the tab control, and deletes the tab.
+    Removes tab \a t from the tab control, and deletes the tab.
 */
 void QTabBar::removeTab( QTab * t )
 {
@@ -427,28 +451,31 @@ void QTabBar::removeTab( QTab * t )
     if ( d->a )
 	d->a->removeItem( t->id );
 #endif
+    bool updateFocus = t->id == d->focus;
     // remove the TabBar Reference
     if(d->pressed == t)
-	d->pressed = NULL;
+	d->pressed = 0;
     t->setTabBar( 0 );
     l->remove( t );
     lstatic->remove( t );
     layoutTabs();
     updateArrowButtons();
     makeVisible( tab( currentTab() ) );
+    if ( updateFocus )
+	d->focus = currentTab();
     update();
 }
 
 
 /*!
-  Enables tab \a id if \a enabled is TRUE or disables it if \a enabled is
-  FALSE.  If \a id is currently selected, setTabEnabled(FALSE) makes
-  another tab selected.
+    Enables tab \a id if \a enabled is TRUE or disables it if \a
+    enabled is FALSE. If \a id is currently selected,
+    setTabEnabled(FALSE) makes another tab selected.
 
-  setTabEnabled() updates the display if this causes a
-  change in \a id's status.
+    setTabEnabled() updates the display if this causes a change in \a
+    id's status.
 
-  \sa update(), isTabEnabled()
+    \sa update(), isTabEnabled()
 */
 
 void QTabBar::setTabEnabled( int id, bool enabled )
@@ -485,7 +512,7 @@ void QTabBar::setTabEnabled( int id, bool enabled )
 			emit selected( t->id );
 		    }
 		}
-		repaint( r );
+		repaint( r, FALSE );
 	    }
 	    return;
 	}
@@ -494,52 +521,55 @@ void QTabBar::setTabEnabled( int id, bool enabled )
 
 
 /*!
-  Returns TRUE if the tab with id \a id is enabled, or FALSE if it
-  is disabled or there is no such tab.
+    Returns TRUE if the tab with id \a id exists and is enabled;
+    otherwise returns FALSE.
 
-  \sa setTabEnabled()
+    \sa setTabEnabled()
 */
 
 bool QTabBar::isTabEnabled( int id ) const
 {
-    QTab * t;
-    for( t = l->first(); t; t = l->next() ) {
-	if ( t && t->id == id )
-	    return t->enabled;
-    }
+    QTab * t = tab( id );
+    if ( t )
+	return t->enabled;
     return FALSE;
 }
 
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 QSize QTabBar::sizeHint() const
 {
-    QTab * t = l->first();
-    if ( t ) {
+    QSize sz(0, 0);
+    if ( QTab * t = l->first() ) {
 	QRect r( t->r );
 	while ( (t = l->next()) != 0 )
 	    r = r.unite( t->r );
-	return r.size().expandedTo( QApplication::globalStrut() );
-    } else {
-	return QSize( 0, 0 ).expandedTo( QApplication::globalStrut() );
+	sz = r.size();
     }
+    return sz.expandedTo(QApplication::globalStrut());
 }
 
-/*! \reimp */
+/*!
+    \reimp
+*/
 
 QSize QTabBar::minimumSizeHint() const
 {
+    if(style().styleHint( QStyle::SH_TabBar_PreferNoArrows, this ))
+	return sizeHint();
     return QSize( d->rightB->sizeHint().width() * 2 + 75, sizeHint().height() );
 }
 
-/*!  Paints the tab \a t using painter \a p.  If and only if \a selected
-  is TRUE, \a t is drawn currently selected.
+/*!
+    Paints the tab \a t using painter \a p. If and only if \a selected
+    is TRUE, \a t is drawn currently selected.
 
-  This virtual function may be reimplemented to change the look of
-  QTabBar.  If you decide to reimplement it, you may also need to
-  reimplement sizeHint().
+    This virtual function may be reimplemented to change the look of
+    QTabBar. If you decide to reimplement it, you may also need to
+    reimplement sizeHint().
 */
 
 void QTabBar::paint( QPainter * p, QTab * t, bool selected ) const
@@ -579,8 +609,8 @@ void QTabBar::paint( QPainter * p, QTab * t, bool selected ) const
 }
 
 /*!
-  Paints the label of tab \a t centered in rectangle \a br using
-  painter \a p. A focus indication is drawn if \a has_focus is TRUE.
+    Paints the label of tab \a t centered in rectangle \a br using
+    painter \a p. A focus indication is drawn if \a has_focus is TRUE.
 */
 
 void QTabBar::paintLabel( QPainter* p, const QRect& br,
@@ -599,10 +629,13 @@ void QTabBar::paintLabel( QPainter* p, const QRect& br,
 	int pixh = pixmap.height();
 	r.setLeft( r.left() + pixw + 4 );
 	r.setRight( r.right() + 2 );
-	// ### the pixmap shift should probably not be hardcoded..
-	p->drawPixmap( br.left() + 2 + ((selected == TRUE) ? 0 : 2),
-		       br.center().y()-pixh/2 + ((selected == TRUE) ? 0 : 2),
-		       pixmap );
+
+	int xoff = 0, yoff = 0;
+	if(!selected) {
+	    xoff = style().pixelMetric(QStyle::PM_TabBarTabShiftHorizontal, this);
+	    yoff = style().pixelMetric(QStyle::PM_TabBarTabShiftVertical, this);
+	}
+	p->drawPixmap( br.left() + 2 + xoff, br.center().y()-pixh/2 + yoff, pixmap );
     }
 
     QStyle::SFlags flags = QStyle::Style_Default;
@@ -611,6 +644,12 @@ void QTabBar::paintLabel( QPainter* p, const QRect& br,
 	flags |= QStyle::Style_Enabled;
     if (has_focus)
 	flags |= QStyle::Style_HasFocus;
+    if ( selected )
+	flags |= QStyle::Style_Selected;
+    else if(t == d->pressed)
+	flags |= QStyle::Style_Sunken;
+    if(t->rect().contains(mapFromGlobal(QCursor::pos())))
+	flags |= QStyle::Style_MouseOver;
     style().drawControl( QStyle::CE_TabBarLabel, p, this, r,
 			 t->isEnabled() ? colorGroup(): palette().disabled(),
 			 flags, QStyleOption(t) );
@@ -618,11 +657,11 @@ void QTabBar::paintLabel( QPainter* p, const QRect& br,
 
 
 /*!
-  Repaints the tab row.  All the painting is done by paint();
-  paintEvent() only decides which tabs need painting and in what
-  order. The event is passed in \a e.
+    Repaints the tab row. All the painting is done by paint();
+    paintEvent() only decides which tabs need painting and in what
+    order. The event is passed in \a e.
 
-  \sa paint()
+    \sa paint()
 */
 
 void QTabBar::paintEvent( QPaintEvent * e )
@@ -632,9 +671,6 @@ void QTabBar::paintEvent( QPaintEvent * e )
 
     QSharedDoubleBuffer buffer( this, e->rect() );
 
-    buffer.painter()->setBrushOrigin( rect().bottomLeft() );
-    buffer.painter()->fillRect( 0, 0, width(), height(),
-				colorGroup().brush( QColorGroup::Background ) );
     QTab * t;
     t = l->first();
     do {
@@ -669,12 +705,13 @@ void QTabBar::paintEvent( QPaintEvent * e )
 
 
 /*!
-  This virtual function is called by the mouse event handlers to
-  determine which tab is pressed.  The default implementation returns
-  a pointer to the tab whose bounding rectangle contains \a p, if
-  exactly one tab's bounding rectangle contains \a p.  Otherwise it returns 0.
+    This virtual function is called by the mouse event handlers to
+    determine which tab is pressed. The default implementation returns
+    a pointer to the tab whose bounding rectangle contains \a p, if
+    exactly one tab's bounding rectangle contains \a p. Otherwise it
+    returns 0.
 
-  \sa mousePressEvent() mouseReleaseEvent()
+    \sa mousePressEvent() mouseReleaseEvent()
 */
 
 QTab * QTabBar::selectTab( const QPoint & p ) const
@@ -699,7 +736,8 @@ QTab * QTabBar::selectTab( const QPoint & p ) const
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QTabBar::mousePressEvent( QMouseEvent * e )
 {
@@ -718,7 +756,8 @@ void QTabBar::mousePressEvent( QMouseEvent * e )
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 
 void QTabBar::mouseMoveEvent ( QMouseEvent *e )
@@ -738,7 +777,8 @@ void QTabBar::mouseMoveEvent ( QMouseEvent *e )
     }
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 
 void QTabBar::mouseReleaseEvent( QMouseEvent *e )
@@ -746,15 +786,16 @@ void QTabBar::mouseReleaseEvent( QMouseEvent *e )
     if ( e->button() != LeftButton )
 	e->ignore();
     if(d->pressed) {
-	QTab *t = selectTab( e->pos() ) == d->pressed ? d->pressed : NULL;
-	d->pressed = NULL;
-	if(t && e->type() == style().styleHint( QStyle::SH_TabBar_SelectMouseType, this ))
+	QTab *t = selectTab( e->pos() ) == d->pressed ? d->pressed : 0;
+	d->pressed = 0;
+	if(t && t->enabled && e->type() == style().styleHint( QStyle::SH_TabBar_SelectMouseType, this ))
 	    setCurrentTab( t );
     }
 }
 
 
-/*!  \reimp
+/*!
+    \reimp
 */
 void QTabBar::show()
 {
@@ -766,20 +807,20 @@ void QTabBar::show()
 	emit selected( t->id );
 }
 
-/*! \property QTabBar::currentTab
-    \brief the id of the currently visible tab in the tab bar
+/*!
+    \property QTabBar::currentTab
+    \brief the id of the tab bar's visible tab
 
-  If no tab page is currently visible, -1 will be the current value
-  for this property.
-  Even if the property value is not -1, you cannot assume that
-  the user can see the relevant page, or that the tab is enabled.
-  When you need to display something the value of this property
-  represents the best page to display.
+    If no tab page is currently visible, the property's value is -1.
+    Even if the property's value is not -1, you cannot assume that the
+    user can see the relevant page, or that the tab is enabled. When
+    you need to display something the value of this property
+    represents the best page to display.
 
-  When this property is set to \e id, it will raise the tab with the
-  id \e id and emit the selected() signal.
+    When this property is set to \e id, it will raise the tab with the
+    id \e id and emit the selected() signal.
 
-  \sa selected() isTabEnabled()
+    \sa selected() isTabEnabled()
 */
 
 int QTabBar::currentTab() const
@@ -797,10 +838,11 @@ void QTabBar::setCurrentTab( int id )
 
 /*!
     \overload
-    Raises \a tab and emits the selected() signal unless the tab was
-  already current.
 
-  \sa currentTab() selected()
+    Raises \a tab and emits the selected() signal unless the tab was
+    already current.
+
+    \sa currentTab() selected()
 */
 
 void QTabBar::setCurrentTab( QTab * tab )
@@ -818,23 +860,26 @@ void QTabBar::setCurrentTab( QTab * tab )
 	setMicroFocusHint( tab->rect().x(), tab->rect().y(), tab->rect().width(), tab->rect().height(), FALSE );
 
 	if ( tab->r.intersects( r ) ) {
-	    repaint( r.unite( tab->r ) );
+	    repaint( r.unite( tab->r ), FALSE );
 	} else {
-	    repaint( r );
-	    repaint( tab->r );
+	    repaint( r, FALSE );
+	    repaint( tab->r, FALSE );
 	}
 	makeVisible( tab );
 	emit selected( tab->id );
+
+#ifdef QT_ACCESSIBILITY_SUPPORT
+	QAccessible::updateAccessibility( this, indexOf(tab->id)+1, QAccessible::Focus );
+#endif
     }
 }
 
-/*! \property QTabBar::keyboardFocusTab
-    \brief the id of the tab that currently has the keyboard focus
-
-  This property contains the id of the tab that currently has the
-  keyboard focus. If the tab bar does not have keyboard focus, the
-  value of this property will be -1.
+/*!
+    \property QTabBar::keyboardFocusTab
+    \brief the id of the tab that has the keyboard focus
 
+    This property contains the id of the tab that has the keyboard
+    focus or -1 if the tab bar does not have the keyboard focus.
 */
 
 int QTabBar::keyboardFocusTab() const
@@ -843,7 +888,8 @@ int QTabBar::keyboardFocusTab() const
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QTabBar::keyPressEvent( QKeyEvent * e )
 {
@@ -893,10 +939,10 @@ void QTabBar::keyPressEvent( QKeyEvent * e )
 }
 
 
-/*!  Returns a pointer to the tab with id \a id or 0 if there is no
-  such tab.
+/*!
+    Returns the tab with id \a id or 0 if there is no such tab.
 
-  \sa count()
+    \sa count()
 */
 
 QTab * QTabBar::tab( int id ) const
@@ -909,9 +955,10 @@ QTab * QTabBar::tab( int id ) const
 }
 
 
-/*! Returns a pointer to the tab at the position \a index.
+/*!
+    Returns the tab at position \a index.
 
-  \sa indexOf()
+    \sa indexOf()
 */
 
 QTab * QTabBar::tabAt( int index ) const
@@ -923,10 +970,11 @@ QTab * QTabBar::tabAt( int index ) const
 
 
 /*!
-  Returns the position index of the tab with id \a id.
+    Returns the position index of the tab with id \a id or -1 if no
+    tab has this \a id.
 
-  \sa tabAt()
- */
+    \sa tabAt()
+*/
 int QTabBar::indexOf( int id ) const
 {
     QTab * t;
@@ -940,10 +988,11 @@ int QTabBar::indexOf( int id ) const
 }
 
 
-/*! \property QTabBar::count
+/*!
+    \property QTabBar::count
     \brief the number of tabs in the tab bar
 
-  \sa tab()
+    \sa tab()
 */
 int QTabBar::count() const
 {
@@ -952,7 +1001,16 @@ int QTabBar::count() const
 
 
 /*!
-  The list of QTab objects in the tab bar.
+    The list of QTab objects in the tab bar.
+
+    This list is unlikely to be in the order that the QTab elements
+    appear visually. One way of iterating over the tabs is like this:
+    \code
+    for ( uint i = 0; i < myTabBar->count(); ++i ) {
+	nextTab = myTabBar->tabAt( i );
+	// do something with nextTab
+    }
+    \endcode
 */
 QPtrList<QTab> * QTabBar::tabList()
 {
@@ -960,11 +1018,12 @@ QPtrList<QTab> * QTabBar::tabList()
 }
 
 
-/*! \property QTabBar::shape
+/*!
+    \property QTabBar::shape
     \brief the shape of the tabs in the tab bar
 
-    The value of this property can be one of the following:
-    \c RoundedAbove (default), \c RoundedBelow, \c TriangularAbove or \c
+    The value of this property is one of the following: \c
+    RoundedAbove (default), \c RoundedBelow, \c TriangularAbove or \c
     TriangularBelow.
 
     \sa Shape
@@ -984,21 +1043,29 @@ void QTabBar::setShape( Shape s )
 }
 
 /*!
-  Lays out all existing tabs according to their label and their
-  iconset.
+    Lays out all existing tabs according to their label and their
+    iconset.
  */
 void QTabBar::layoutTabs()
 {
     if ( lstatic->isEmpty() )
 	return;
 
+    QSize oldSh(0, 0);
+    if ( QTab * t = l->first() ) {
+	QRect r( t->r );
+	while ( (t = l->next()) != 0 )
+	    r = r.unite( t->r );
+	oldSh = r.size();
+    }
+
+    d->btnWidth = style().pixelMetric(QStyle::PM_TabBarScrollButtonWidth, this);
     int hframe, vframe, overlap;
     hframe  = style().pixelMetric( QStyle::PM_TabBarTabHSpace, this );
     vframe  = style().pixelMetric( QStyle::PM_TabBarTabVSpace, this );
     overlap = style().pixelMetric( QStyle::PM_TabBarTabOverlap, this );
 
     QFontMetrics fm = fontMetrics();
-    int x = 0;
     QRect r;
     QTab *t;
     bool reverse = QApplication::reverseLayout();
@@ -1006,6 +1073,8 @@ void QTabBar::layoutTabs()
 	t = lstatic->last();
     else
 	t = lstatic->first();
+    int x = 0;
+    int offset = (t && d->scrolls) ? t->r.x() : 0;
     while ( t ) {
 	int lw = fm.width( t->label );
 	lw -= t->label.contains('&') * fm.width('&');
@@ -1020,8 +1089,9 @@ void QTabBar::layoutTabs()
 	h = QMAX( h, QApplication::globalStrut().height() );
 
 	h += vframe;
-	t->r.setRect( x, 0, QMAX( lw + hframe + iw,
-				  QApplication::globalStrut().width() ), h );
+	t->r = QRect(QPoint(x, 0), style().sizeFromContents(QStyle::CT_TabBarTab, this,
+   	             QSize( QMAX( lw + hframe + iw, QApplication::globalStrut().width() ), h ),
+		     QStyleOption(t) ));
 	x += t->r.width() - overlap;
 	r = r.unite( t->r );
 	if ( reverse )
@@ -1029,8 +1099,37 @@ void QTabBar::layoutTabs()
 	else
 	    t = lstatic->next();
     }
-    for ( t = lstatic->first(); t; t = lstatic->next() )
+    x += overlap;
+    int w = (d->scrolls) ? d->leftB->x() : width();
+    if (x + offset < w)
+	offset = w - x;
+    if (offset > 0)
+	offset = 0;
+
+    for ( t = lstatic->first(); t; t = lstatic->next() ) {
+	t->r.moveBy( offset, 0 );
 	t->r.setHeight( r.height() );
+    }
+
+    if ( sizeHint() != oldSh )
+	updateGeometry();
+
+    emit layoutChanged();
+}
+
+/*!
+    \reimp
+*/
+
+bool QTabBar::event( QEvent *e )
+{
+    if ( e->type() == QEvent::LanguageChange ) {
+	layoutTabs();
+	updateArrowButtons();
+	makeVisible( tab( currentTab() ));
+    }
+
+    return QWidget::event( e );
 }
 
 /*!
@@ -1039,44 +1138,37 @@ void QTabBar::layoutTabs()
 
 void QTabBar::styleChange( QStyle& old )
 {
-	layoutTabs();
-	QWidget::styleChange( old );
+    layoutTabs();
+    updateArrowButtons();
+    QWidget::styleChange( old );
 }
 
 /*!
-  \reimp
+    \reimp
 */
 void QTabBar::focusInEvent( QFocusEvent * )
 {
-    QTab *t = l->first();
-    for ( ; t; t = l->next() ) {
-	if ( t->id == d->focus ) {
-	    repaint( t->r );
-	    break;
-	}
-    }
+    QTab *t = tab( d->focus );
+    if ( t )
+	repaint( t->r, FALSE );
 }
 
 /*!
-  \reimp
+    \reimp
 */
 void QTabBar::focusOutEvent( QFocusEvent * )
 {
-    QTab *t = l->first();
-    for ( ; t; t = l->next() ) {
-	if ( t->id == d->focus ) {
-	    repaint( t->r );
-	    break;
-	}
-    }
+    QTab *t = tab( d->focus );
+    if ( t )
+	repaint( t->r, FALSE );
 }
 
 /*!
-  \reimp
+    \reimp
 */
 void QTabBar::resizeEvent( QResizeEvent * )
 {
-    const int arrowWidth = 16;
+    const int arrowWidth = QMAX( d->btnWidth, QApplication::globalStrut().width() );;
     d->rightB->setGeometry( width() - arrowWidth, 0, arrowWidth, height() );
     d->leftB->setGeometry( width() - 2*arrowWidth, 0, arrowWidth, height() );
     layoutTabs();
@@ -1109,20 +1201,25 @@ void QTabBar::makeVisible( QTab* tab  )
     if ( !d->scrolls || ( !tooFarLeft && ! tooFarRight ) )
 	return;
 
+    bool bs = signalsBlocked();
+    blockSignals(TRUE);
     layoutTabs();
+    blockSignals(bs);
 
     int offset = 0;
 
-    if ( tooFarLeft )
-	offset = tab == lstatic->first() ? 0 : tab->r.left() - 8;
-    else if ( tooFarRight ) {
+    if ( tooFarLeft ) {
+	offset = tab->r.left();
+	if (tab != lstatic->first())
+	    offset -= 8;
+    } else if ( tooFarRight ) {
 	offset = tab->r.right() - d->leftB->x() + 1;
     }
 
     for ( QTab* t = lstatic->first(); t; t = lstatic->next() )
 	t->r.moveBy( -offset, 0 );
 
-    d->leftB->setEnabled( offset != 0 );
+    d->leftB->setEnabled( lstatic->first()->r.left() < 0);
     d->rightB->setEnabled( lstatic->last()->r.right() >= d->leftB->x() );
 
     // Make sure disabled buttons pop up again
@@ -1132,25 +1229,40 @@ void QTabBar::makeVisible( QTab* tab  )
 	d->rightB->setDown( FALSE );
 
     update();
+    emit layoutChanged();
 }
 
 void QTabBar::updateArrowButtons()
 {
-    bool b = lstatic->last() &&	( lstatic->last()->r.right() > width() );
-    d->scrolls = b;
+    if (lstatic->isEmpty()) {
+	d->scrolls = FALSE;
+    } else {
+	d->scrolls = (lstatic->last()->r.right() - lstatic->first()->r.left() > width());
+    }
     if ( d->scrolls ) {
-	d->leftB->setEnabled( FALSE );
-	d->rightB->setEnabled( TRUE );
+	const int arrowWidth = QMAX( d->btnWidth, QApplication::globalStrut().width() );
+	if ( QApplication::reverseLayout() ) {
+	    d->rightB->setGeometry( arrowWidth, 0, arrowWidth, height() );
+	    d->leftB->setGeometry( 0, 0, arrowWidth, height() );
+	} else {
+	    d->rightB->setGeometry( width() - arrowWidth, 0, arrowWidth, height() );
+	    d->leftB->setGeometry( width() - 2*arrowWidth, 0, arrowWidth, height() );
+	}
+
+	d->leftB->setEnabled( lstatic->first()->r.left() < 0);
+	d->rightB->setEnabled( lstatic->last()->r.right() >= d->leftB->x() );
 	d->leftB->show();
 	d->rightB->show();
     } else {
 	d->leftB->hide();
 	d->rightB->hide();
+	layoutTabs();
     }
 }
 
-/*! Removes the tool tip for the tab at index \a index.
- */
+/*!
+    Removes the tool tip for the tab at index position \a index.
+*/
 void QTabBar::removeToolTip( int index )
 {
 #ifndef QT_NO_TOOLTIP
@@ -1161,8 +1273,10 @@ void QTabBar::removeToolTip( int index )
 #endif
 }
 
-/*! Sets the tool tip for the tab at index \a index to \a tip.
- */
+/*!
+    Sets the tool tip for the tab at index position \a index to \a
+    tip.
+*/
 void QTabBar::setToolTip( int index, const QString & tip )
 {
 #ifndef QT_NO_TOOLTIP
@@ -1175,8 +1289,9 @@ void QTabBar::setToolTip( int index, const QString & tip )
 #endif
 }
 
-/*! Returns the tool tip for the tab at index \a index.
- */
+/*!
+    Returns the tool tip for the tab at index position \a index.
+*/
 QString QTabBar::toolTip( int index ) const
 {
 #ifndef QT_NO_TOOLTIP
@@ -1188,8 +1303,8 @@ QString QTabBar::toolTip( int index ) const
 }
 
 /*!
-  Sets the text of the tab to \a text.
- */
+    Sets the text of the tab to \a text.
+*/
 void QTab::setText( const QString& text )
 {
     label = text;
@@ -1201,12 +1316,16 @@ void QTab::setText( const QString& text )
 	    tb->d->a->insertItem( p, id );
 #endif
 	tb->layoutTabs();
-	tb->repaint();
+	tb->repaint(FALSE);
+
+#if defined(QT_ACCESSIBILITY_SUPPORT)
+	QAccessible::updateAccessibility( tb, tb->indexOf(id)+1, QAccessible::NameChanged );
+#endif
     }
 }
 
 /*!
-  Sets the tab iconset to \a icon
+    Sets the tab's iconset to \a icon
 */
 void QTab::setIconSet( const QIconSet &icon )
 {
