@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qwhatsthis.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/qwhatsthis.cpp   3.3.4   edited Feb 27 2004 $
 **
 ** Implementation of QWhatsThis class
 **
@@ -225,7 +225,7 @@ private slots:
 static QWhatsThisPrivate * wt = 0;
 
 // shadowWidth not const, for XP drop-shadow-fu turns it to 0
-int shadowWidth = 6;   // also used as '5' and '6' and even '8' below
+static int shadowWidth = 6;   // also used as '5' and '6' and even '8' below
 const int vMargin = 8;
 const int hMargin = 12;
 
@@ -275,7 +275,7 @@ QWhatsThat::QWhatsThat( QWidget* w, const QString& txt, QWidget* parent, const c
 					text );
     }
 #if defined(Q_WS_WIN)
-    if ( qWinVersion() == WV_XP ) {
+    if ( (qWinVersion()&WV_NT_based) > WV_2000 ) {
 	BOOL shadow;
 	SystemParametersInfo( SPI_GETDROPSHADOW, 0, &shadow, 0 );
 	shadowWidth = shadow ? 0 : 6;
@@ -363,7 +363,7 @@ void QWhatsThat::paintEvent( QPaintEvent* )
 {
     bool drawShadow = TRUE;
 #if defined(Q_WS_WIN)
-    if ( qWinVersion() == WV_XP ) {
+    if ( (qWinVersion()&WV_NT_based) > WV_2000 ) {
 	BOOL shadow;
 	SystemParametersInfo( SPI_GETDROPSHADOW, 0, &shadow, 0 );
 	drawShadow = !shadow;
