@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qworkspace.cpp   3.3.3   edited Jul 16 16:43 $
+** $Id: qt/qworkspace.cpp   3.3.4   edited Jan 4 15:13 $
 **
 ** Implementation of the QWorkspace class
 **
@@ -532,7 +532,7 @@ void QWorkspace::activateWindow( QWidget* w, bool change_focus )
     if ( d->maxWindow && d->maxWindow != d->active && d->active->windowWidget() &&
 	 d->active->windowWidget()->testWFlags( WStyle_MinMax ) &&
 	 !d->active->windowWidget()->testWFlags( WStyle_Tool ) ) {
-	d->active->windowWidget()->showMaximized();
+        d->active->showMaximized();
 	if ( d->maxtools ) {
 #ifndef QT_NO_WIDGET_TOPEXTRA
 	    if ( w->icon() ) {
@@ -1273,6 +1273,10 @@ QWidgetList QWorkspace::windowList() const
     had been inserted into the workspace. If \a order is StackingOrder
     the windows are listed in their stacking order, with the topmost window
     being the last window in the list.
+
+    QWidgetList is the same as QPtrList<QWidget>.
+
+    \sa QPtrList
 */
 QWidgetList QWorkspace::windowList( WindowOrder order ) const
 {
@@ -1869,7 +1873,6 @@ void QWorkspace::cascade()
 
     setUpdatesEnabled( FALSE );
     QPtrListIterator<QWorkspaceChild> it( widgets );
-    int children = d->windows.count() - 1;
     while ( it.current () ) {
 	QWorkspaceChild *child = it.current();
 	++it;
@@ -1983,11 +1986,14 @@ void QWorkspace::tile()
 	    c->showNormal();
 	    qApp->sendPostedEvents( 0, QEvent::ShowNormal );
 	    used[row*cols+col] = TRUE;
-            QSize sz(w, h + add ? h : 0);
+            QSize sz(w, h);
             QSize bsize(c->baseSize());
             sz = sz.expandedTo(c->windowWidget()->minimumSize()).boundedTo(c->windowWidget()->maximumSize());
             sz += bsize;
+
 	    if ( add ) {
+                if (sz.height() == h + bsize.height()) // no relevant constrains
+                    sz.rheight() *= 2;
 		c->setGeometry(col*w + col*bsize.width(), row*h + row*bsize.height(), sz.width(), sz.height());
 		used[(row+1)*cols+col] = TRUE;
 		add--;
@@ -2001,7 +2007,7 @@ void QWorkspace::tile()
 		    row++;
 		}
 	    }
-	}
+        }
     }
     delete [] used;
 
@@ -2122,7 +2128,10 @@ QWorkspaceChild::QWorkspaceChild( QWidget* window, QWorkspace *parent,
     widgetResizeHandler->setSizeProtection( !parent->scrollBarsEnabled() );
     connect( widgetResizeHandler, SIGNAL( activate() ),
 	     this, SLOT( activate() ) );
-    widgetResizeHandler->setExtraHeight( th + contentsRect().y() - 2*frameWidth() );
+    if ( !style().styleHint( QStyle::SH_TitleBar_NoBorder, titlebar ) )
+	widgetResizeHandler->setExtraHeight( th + contentsRect().y() - 2*frameWidth() );
+    else
+	widgetResizeHandler->setExtraHeight( th + contentsRect().y() - frameWidth() );
     if(parent->windowMode() == QWorkspace::TopLevel && isTopLevel()) {
 	move(0, 0);
 	widgetResizeHandler->setActive( FALSE );
@@ -2139,8 +2148,12 @@ QWorkspaceChild::~QWorkspaceChild()
     QWorkspace *workspace = ::qt_cast<QWorkspace*>(parentWidget());
     if ( workspace ) {
         workspace->d->focus.removeRef(this);
-	if ( workspace->d->active == this )
+	if ( workspace->d->active == this ) {
 	    workspace->activatePrevWindow();
+            if (workspace->d->active == this) {
+                workspace->activateWindow(0);
+            }
+        }
 	if ( workspace->d->maxWindow == this ) {
 	    workspace->hideMaximizeControls();
 	    workspace->d->maxWindow = 0;
@@ -2386,7 +2399,7 @@ bool QWorkspaceChild::eventFilter( QObject * o, QEvent * e)
 		pm.setMask(pm.createHeuristicMask());
 	    }
 	    titlebar->setIcon( pm );
-	    if ( iconw ) 		
+	    if ( iconw )
 		iconw->setIcon( pm );
 
 	    if ( ws->d->maxWindow != this )
