@@ -47,6 +47,7 @@
 #include <qprogressbar.h>
 #include <qptrlist.h>
 #include <qptrstack.h>
+#include <qpushbutton.h>
 #include <qregexp.h>
 #include <qsettings.h>
 #include <qstatusbar.h>
@@ -67,6 +68,18 @@ static QString stripAmpersand( const QString &str )
     return s;
 }
 
+static bool verifyDirectory(const QString &str)
+{
+    QFileInfo dirInfo(str);
+    if (!dirInfo.exists())
+	return QDir().mkdir(str);
+    if (!dirInfo.isDir()) {
+	qWarning("'%s' exists but is not a directory", str.latin1());
+	return FALSE;
+    }
+    return TRUE;
+}
+
 struct IndexKeyword {
     IndexKeyword( const QString &kw, const QString &l )
 	: keyword( kw ), link( l ) {}
@@ -215,9 +228,9 @@ void HelpDialog::initialize()
     termsEdit->setValidator( new SearchValidator( termsEdit ) );
 
     itemPopup = new QPopupMenu( this );
-    itemPopup->insertItem( tr( "Open Link in Current Page" ), 0 );
+    itemPopup->insertItem( tr( "Open Link in Current Tab" ), 0 );
     itemPopup->insertItem( tr( "Open Link in New Window" ), 1 );
-    itemPopup->insertItem( tr( "Open Link in New Page" ), 2 );
+    itemPopup->insertItem( tr( "Open Link in New Tab" ), 2 );
 
     contentList.setAutoDelete( TRUE );
     contentList.clear();
@@ -247,7 +260,7 @@ void HelpDialog::lastWinClosed()
 void HelpDialog::removeOldCacheFiles()
 {
     QString dir = cacheFilesPath; // ### remove the last '/' ?
-    if ( !QFile::exists( dir ) && !QDir().mkdir( dir ) ) {
+    if (!verifyDirectory(cacheFilesPath)) {
 	qWarning( "Failed to created assistant directory" );
 	return;
     }
@@ -264,6 +277,17 @@ void HelpDialog::removeOldCacheFiles()
     }
 }
 
+void HelpDialog::timerEvent(QTimerEvent *e)
+{
+    static int opacity = 255;
+    // To allow patching of 3.2.3 assistant for qsa.
+#if QT_VERSION >= 0x030300
+    help->setWindowOpacity((opacity-=4)/255.0);
+    if (opacity<=0)
+#endif
+	qApp->quit();
+}
+
 
 void HelpDialog::loadIndexFile()
 {
@@ -289,9 +313,18 @@ void HelpDialog::loadIndexFile()
 	processEvents();
 	if( lwClosed )
 	    return;
-	indexFile.open( IO_ReadOnly );
+	if (!indexFile.open(IO_ReadOnly)) {
+	    QMessageBox::warning(help, tr("Qt Assistant"), tr("Failed to load keyword index file\n"
+							      "Assistant will not work!"));
+#if defined Q_WS_WIN || defined Q_WS_MACX
+	    startTimer(50);
+#endif
+	    return;
+	}
     }
 
+    editIndex->setEnabled(FALSE);
+
     QDataStream ds( &indexFile );
     Q_UINT32 fileAges;
     ds >> fileAges;
@@ -301,6 +334,7 @@ void HelpDialog::loadIndexFile()
 	if ( !indexFile.open( IO_ReadOnly ) ) {
 	    QMessageBox::warning( help, tr( "Qt Assistant" ),
 		tr( "Cannot open the index file %1" ).arg( QFileInfo( indexFile ).absFilePath() ) );
+	    editIndex->setEnabled(TRUE);
 	    return;
 	}
 	ds.setDevice( &indexFile );
@@ -325,6 +359,7 @@ void HelpDialog::loadIndexFile()
     framePrepare->hide();
     showInitDoneMessage();
     setCursor( arrowCursor );
+    editIndex->setEnabled(TRUE);
 }
 
 Q_UINT32 HelpDialog::getFileAges()
@@ -404,7 +439,7 @@ void HelpDialog::buildKeywordDB()
 	qHeapSort( lst );
 
     QFile indexout( cacheFilesPath + "indexdb." + Config::configuration()->profileName() );
-    if ( indexout.open( IO_WriteOnly ) ) {
+    if ( verifyDirectory(cacheFilesPath) && indexout.open( IO_WriteOnly ) ) {
 	QDataStream s( &indexout );
 	s << fileAges;
 	s << lst;
@@ -502,7 +537,10 @@ void HelpDialog::buildContentDict()
 	}
     }
 
-    QFile contentOut( cacheFilesPath + "contentdb." + Config::configuration()->profileName() );
+    QString pname = Config::configuration()->profileName();
+    QFile contentOut( cacheFilesPath + "contentdb." + pname );
+    QFile::remove( cacheFilesPath + "indexdb.dict." + pname );
+    QFile::remove( cacheFilesPath + "indexdb.doc." + pname );
     if ( contentOut.open( IO_WriteOnly ) ) {
 	QDataStream s( &contentOut );
 	s << fileAges;
@@ -609,6 +647,7 @@ void HelpDialog::searchInIndex( const QString &s )
 	if ( t.length() >= sl.length() &&
 	     i->text().left(s.length()).lower() == sl ) {
 	    listIndex->setCurrentItem( i );
+	    listIndex->setTopItem(listIndex->index(i));
 	    break;
 	}
 	i = i->next();
@@ -760,7 +799,7 @@ void HelpDialog::insertContents()
     for ( ; lstIt.current(); ++lstIt ) {
 	HelpNavigationContentsItem *newEntry;
 
-	HelpNavigationContentsItem *contentEntry;
+	HelpNavigationContentsItem *contentEntry = 0;
 	QPtrStack<HelpNavigationContentsItem> stack;
 	stack.clear();
 	int depth = 0;
@@ -777,7 +816,7 @@ void HelpDialog::insertContents()
 	    ContentItem item = *it;
 	    if( item.depth == 0 ){
 		newEntry = new HelpNavigationContentsItem( listContents, 0 );
-		newEntry->setPixmap( 0, QPixmap::fromMimeSource( "book.png" ) );	
+		newEntry->setPixmap( 0, QPixmap::fromMimeSource( "book.png" ) );
 		newEntry->setText( 0, item.title );
 		newEntry->setLink( item.reference );
 		stack.push( newEntry );
@@ -873,6 +912,16 @@ void HelpDialog::setupFullTextIndex()
 
     QString pname = Config::configuration()->profileName();
     fullTextIndex = new Index( documentList, QDir::homeDirPath() ); // ### Is this correct ?
+    if (!verifyDirectory(cacheFilesPath)) {
+	QMessageBox::warning(help, tr("Qt Assistant"),
+			     tr("Failed to save fulltext search index\n"
+				"Assistant will not work!"));
+	return;
+    }
+    searchButton->setEnabled(FALSE);
+    helpButton->setEnabled(FALSE);
+    termsEdit->setEnabled(FALSE);
+
     fullTextIndex->setDictionaryFile( cacheFilesPath + "indexdb.dict." + pname );
     fullTextIndex->setDocListFile( cacheFilesPath + "indexdb.doc." + pname );
     processEvents();
@@ -889,13 +938,13 @@ void HelpDialog::setupFullTextIndex()
 	progressPrepare->show();
 	framePrepare->show();
 	processEvents();
-	if ( fullTextIndex->makeIndex() == -1 )
-	    return;
-	fullTextIndex->writeDict();
-	progressPrepare->setProgress( 100 );
-	framePrepare->hide();
-	setCursor( arrowCursor );
-	showInitDoneMessage();
+	if ( fullTextIndex->makeIndex() != -1 ) {
+	    fullTextIndex->writeDict();
+	    progressPrepare->setProgress( 100 );
+	    framePrepare->hide();
+	    setCursor( arrowCursor );
+	    showInitDoneMessage();
+	}
     } else {
 	setCursor( waitCursor );
 	help->statusBar()->message( tr( "Reading dictionary..." ) );
@@ -904,6 +953,9 @@ void HelpDialog::setupFullTextIndex()
 	help->statusBar()->message( tr( "Done" ), 3000 );
 	setCursor( arrowCursor );
     }
+    searchButton->setEnabled(TRUE);
+    termsEdit->setEnabled(TRUE);
+    helpButton->setEnabled(TRUE);
 }
 
 void HelpDialog::setIndexingProgress( int prog )
