@@ -39,8 +39,17 @@
 
 QPtrList<MainWindow> *MainWindow::windows = 0;
 
+#if defined(Q_WS_WIN)
+extern Q_EXPORT int qt_ntfs_permission_lookup;
+#endif
+
 void MainWindow::init()
 {
+#if defined(Q_WS_WIN)
+    // Workaround for QMimeSourceFactory failing in QFileInfo::isReadable() for
+    // certain user configs. See task: 34372
+    qt_ntfs_permission_lookup = 0;
+#endif
     setupCompleted = FALSE;
 
     goActions = new QPtrList<QAction>;
@@ -86,9 +95,15 @@ void MainWindow::init()
     if ( config->sideBarHidden() )
 	dw->hide();
 
-    setObjectsEnabled( FALSE );
     tabs->setup();
     QTimer::singleShot( 0, this, SLOT( setup() ) );
+#if defined(Q_OS_MACX)
+    // Use the same forward and backward browser shortcuts as Safari and Internet Explorer do
+    // on the Mac. This means that if you have access to one of those cool Intellimice, the thing
+    // works just fine, since that's how Microsoft hacked it.
+    actionGoPrevious->setAccel(QKeySequence(Qt::CTRL|Qt::Key_Left));
+    actionGoNext->setAccel(QKeySequence(Qt::CTRL|Qt::Key_Right));
+#endif
 }
 
 void MainWindow::setup()
@@ -96,6 +111,8 @@ void MainWindow::setup()
     if( setupCompleted )
 	return;
 
+    qApp->setOverrideCursor( QCursor( Qt::WaitCursor ) );
+    statusBar()->message( tr( "Initializing Qt Assistant..." ) );
     setupCompleted = TRUE;
     helpDock->initialize();
     connect( actionGoPrevious, SIGNAL( activated() ), tabs, SLOT( backward() ) );
@@ -143,7 +160,7 @@ void MainWindow::setup()
     PopupMenu->insertItem( tr( "Vie&ws" ), createDockWindowMenu() );
     helpDock->tabWidget->setCurrentPage( config->sideBarPage() );
 
-    setObjectsEnabled( TRUE );
+    qApp->restoreOverrideCursor();
     actionGoPrevious->setEnabled( FALSE );
     actionGoNext->setEnabled( FALSE );
 }
@@ -200,31 +217,6 @@ bool MainWindow::insertActionSeparator()
     return TRUE;
 }
 
-void MainWindow::setObjectsEnabled( bool b )
-{
-    if ( b ) {
-	qApp->restoreOverrideCursor();
-    } else {
-	qApp->setOverrideCursor( QCursor( Qt::WaitCursor ) );
-	statusBar()->message( tr( "Initializing Qt Assistant..." ) );
-    }
-    QObjectList *l = queryList( "QAction" );
-    QObject *obj;
-    QObjectListIt it( *l );
-    while ( (obj = it.current()) != 0 ) {
-        ++it;
-        ((QAction*)obj)->setEnabled( b );
-    }
-    delete l;
-    QPtrListIterator<QAction> ait( *goActions );
-    while ( ait.current() != 0 ) {
-	(*ait)->setEnabled( b );
-        ++ait;
-    }
-    menubar->setEnabled( b );
-    helpDock->setEnabled( b );
-}
-
 bool MainWindow::close( bool alsoDelete )
 {
     saveSettings();
@@ -253,7 +245,7 @@ void MainWindow::about()
 		 " licensed to you under the terms of the Qt Commercial License"
 		 " Agreement. For details, see the file LICENSE that came with"
 		 " this software distribution.</p><p></p>"
-		 "<p>Qt Free Edition users: This program is licensed to you"
+		 "<p>Qt Open Source Edition users: This program is licensed to you"
 		 " under the terms of the GNU General Public License Version 2."
 		 " For details, see the file LICENSE.GPL that came with this"
 		 " software distribution.</p><p>The program is provided AS IS"
@@ -296,6 +288,24 @@ void MainWindow::find()
     findDialog->show();
 }
 
+void MainWindow::findAgain()
+{
+    if (!findDialog || !findDialog->hasFindExpression()) {
+	find();
+	return;
+    }
+    findDialog->doFind(TRUE);
+}
+
+void MainWindow::findAgainPrev()
+{
+    if (!findDialog || !findDialog->hasFindExpression()) {
+	find();
+	return;
+    }
+    findDialog->doFind(FALSE);
+}
+
 void MainWindow::goHome()
 {
     showLink( Config::configuration()->homePage() );
@@ -317,25 +327,24 @@ void MainWindow::print()
 	QTextBrowser *browser = tabs->currentBrowser();
 	int dpiy = metrics.logicalDpiY();
 	int margin = (int) ( (2/2.54)*dpiy );
-	QRect body( margin,
+	QRect view( margin,
 		    margin,
 		    metrics.width() - 2 * margin,
 		    metrics.height() - 2 * margin );
 	QSimpleRichText richText( browser->text(), browser->QWidget::font(), browser->context(),
 				  browser->styleSheet(), browser->mimeSourceFactory(),
-				  body.height(), Qt::black, FALSE );
-	richText.setWidth( &p, body.width() );
-	QRect view( body );
+				  view.height(), Qt::black, FALSE );
+	richText.setWidth( &p, view.width() );
 	int page = 1;
 	do {
 	    qApp->eventLoop()->processEvents( QEventLoop::ExcludeUserInput );
 
-	    richText.draw( &p, body.left(), body.top(), view, colorGroup() );
-	    view.moveBy( 0, body.height() );
-	    p.translate( 0 , -body.height() );
+	    richText.draw( &p, margin, margin, view, colorGroup() );
+	    view.moveBy( 0, view.height() );
+	    p.translate( 0 , -view.height() );
 	    p.drawText( view.right() - p.fontMetrics().width( QString::number(page) ),
 			view.bottom() + p.fontMetrics().ascent() + 5, QString::number(page) );
-	    if ( view.top() >= richText.height() )
+	    if ( view.top() - margin >= richText.height() )
 		break;
 	    printer.newPage();
 	    page++;
@@ -379,6 +388,7 @@ void MainWindow::showBookmark( int id )
 
 void MainWindow::showLinkFromClient( const QString &link )
 {
+    setWindowState(windowState() & ~WindowMinimized);
     raise();
     setActiveWindow();
     showLink( link );
@@ -413,6 +423,48 @@ void MainWindow::showLink( const QString &link )
     }
 }
 
+void MainWindow::showLinks( const QStringList &links )
+{
+    if ( links.size() == 0 ) {
+	qWarning( "MainWindow::showLinks() - Empty link" );
+	return;
+    }
+
+    if ( links.size() == 1 ) {
+	showLink( links.first() );
+	return;
+    }
+
+    pendingLinks = links;
+
+    QStringList::ConstIterator it = pendingLinks.begin();
+    // Initial showing, The tab is empty so update that without creating it first
+    if ( tabs->currentBrowser()->source().isEmpty() ) {
+	pendingBrowsers.append(tabs->currentBrowser());
+	tabs->setTitle(tabs->currentBrowser(), pendingLinks.first());
+    }
+    ++it;
+
+    while( it != pendingLinks.end() ) {
+	pendingBrowsers.append( tabs->newBackgroundTab(*it) );
+	++it;
+    }
+
+    startTimer(50);
+    return;
+}
+
+void MainWindow::timerEvent(QTimerEvent *e)
+{
+    QString link = pendingLinks.first();
+    HelpWindow *win = pendingBrowsers.first();
+    pendingLinks.pop_front();
+    pendingBrowsers.removeFirst();
+    if (pendingLinks.size() == 0)
+	killTimer(e->timerId());
+    win->setSource(link);
+}
+
 void MainWindow::showQtHelp()
 {
     showLink( QString( qInstallPathDocs() ) + "/html/index.html" );
@@ -434,8 +486,10 @@ void MainWindow::showSettingsDialog( int page )
 	settingsDia = new SettingsDialog( this );
     }
     QFontDatabase fonts;
+    settingsDia->fontCombo->clear();
     settingsDia->fontCombo->insertStringList( fonts.families() );
     settingsDia->fontCombo->lineEdit()->setText( tabs->browserFont().family() );
+    settingsDia->fixedfontCombo->clear();
     settingsDia->fixedfontCombo->insertStringList( fonts.families() );
     settingsDia->fixedfontCombo->lineEdit()->setText( tabs->styleSheet()->item( "pre" )->fontFamily() );
     settingsDia->linkUnderlineCB->setChecked( tabs->linkUnderline() );
@@ -510,10 +564,19 @@ void MainWindow::saveSettings()
     config->setFontFixedFamily( tabs->styleSheet()->item( "pre" )->fontFamily() );
     config->setLinkUnderline( tabs->linkUnderline() );
     config->setLinkColor( tabs->palette().color( QPalette::Active, QColorGroup::Link ).name() );
-    config->setSource( tabs->currentBrowser()->source() );
     config->setSideBarPage( helpDock->tabWidget->currentPageIndex() );
     config->setGeometry( QRect( x(), y(), width(), height() ) );
     config->setMaximized( isMaximized() );
+
+    // Create list of the tab urls
+    QStringList lst;
+    QPtrList<HelpWindow> browsers = tabs->browsers();
+    HelpWindow *browser = browsers.first();
+    while (browser) {
+	lst << browser->source();
+	browser = browsers.next();
+    }
+    config->setSource(lst);
     config->save();
 }
 
