@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/connection.h   3.2.3   edited May 13 09:08 $
+** $Id: qt/connection.h   3.3.4   edited Nov 14 2003 $
 **
 ** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
@@ -9,8 +9,8 @@
 *****************************************************************************/
 
 // Add your own connection parameters here
-#define DB_BOOKS_DRIVER "QMYSQL3"
-#define DB_BOOKS 	""
+#define DB_BOOKS_DRIVER "QSQLITE"
+#define DB_BOOKS 	":memory:"
 #define DB_BOOKS_USER 	""
 #define DB_BOOKS_PASSWD ""
 #define DB_BOOKS_HOST 	""
