@@ -9,10 +9,16 @@
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.
 **
+** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
+** licenses may use this file in accordance with the Qt Commercial License
+** Agreement provided with the Software.
+**
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** See http://www.trolltech.com/gpl/ for GPL licensing information.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+**   information about Qt Commercial License Agreements.
 **
 ** Contact info@trolltech.com if any conditions of this licensing are
 ** not clear to you.
@@ -121,26 +127,17 @@ static QMap<int, QMap<QString, int > > *wordMap = 0;
 SyntaxHighlighter_CPP::SyntaxHighlighter_CPP()
     : QTextPreProcessor(), lastFormat( 0 ), lastFormatId( -1 )
 {
-    int normalSize =  qApp->font().pointSize();
-    QString normalFamily = qApp->font().family();
-    QString commentFamily = "times";
-    int normalWeight = qApp->font().weight();
-    addFormat( Standard,
-	       new QTextFormat( QFont( normalFamily, normalSize, normalWeight ), Qt::black ) );
-    addFormat( Comment,
-	       new QTextFormat( QFont( commentFamily, normalSize, normalWeight, TRUE ), Qt::red ) );
-    addFormat( Number,
-	       new QTextFormat( QFont( normalFamily, normalSize, normalWeight ), Qt::darkBlue ) );
-    addFormat( String,
-	       new QTextFormat( QFont( normalFamily, normalSize, normalWeight ), Qt::darkGreen ) );
-    addFormat( Type,
-	       new QTextFormat( QFont( normalFamily, normalSize, normalWeight ), Qt::darkMagenta ) );
-    addFormat( Keyword,
-	       new QTextFormat( QFont( normalFamily, normalSize, normalWeight ), Qt::darkYellow ) );
-    addFormat( PreProcessor,
-	       new QTextFormat( QFont( normalFamily, normalSize, normalWeight ), Qt::darkBlue ) );
-    addFormat( Label,
-	       new QTextFormat( QFont( normalFamily, normalSize, normalWeight ), Qt::darkRed ) );
+    QFont f( qApp->font() );
+
+    addFormat( Standard, new QTextFormat( f, Qt::black ) );
+    addFormat( Number, new QTextFormat( f, Qt::darkBlue ) );
+    addFormat( String, new QTextFormat( f, Qt::darkGreen ) );
+    addFormat( Type, new QTextFormat( f, Qt::darkMagenta ) );
+    addFormat( Keyword, new QTextFormat( f, Qt::darkYellow ) );
+    addFormat( PreProcessor, new QTextFormat( f, Qt::darkBlue ) );
+    addFormat( Label, new QTextFormat( f, Qt::darkRed ) );
+    f.setFamily( "times" );
+    addFormat( Comment, new QTextFormat( f, Qt::red ) );
 
     if ( wordMap )
 	return;
@@ -261,7 +258,7 @@ void SyntaxHighlighter_CPP::process( QTextDocument *doc, QTextParagraph *string,
 	    process( doc, string->prev(), 0, FALSE );
 	state = string->prev()->endState();
     }
-    int input;
+    int input = -1;
     int i = 0;
     bool lastWasBackSlash = FALSE;
     bool makeLastStandard = FALSE;
@@ -279,7 +276,7 @@ void SyntaxHighlighter_CPP::process( QTextDocument *doc, QTextParagraph *string,
     QChar lastChar;
     for (;;) {
 	QChar c = string->at( i )->c;
-	
+
 	if ( lastWasBackSlash ) {
 	    input = InputSep;
 	} else {
@@ -358,12 +355,12 @@ void SyntaxHighlighter_CPP::process( QTextDocument *doc, QTextParagraph *string,
 	    } break;
 	    }
 	}
-	
+
 	lastWasBackSlash = !lastWasBackSlash && c == '\\';
-	
+
 	if ( input == InputAlpha )
 	    buffer += c;
-	
+
     	state = table[ state ][ input ];
 
 	switch ( state ) {
@@ -497,6 +494,7 @@ void SyntaxHighlighter_CPP::process( QTextDocument *doc, QTextParagraph *string,
 
     string->setExtraData( paragData );
 
+    int oldEndState = string->endState();
     if ( state == StateCComment ||
 	 state == StateCCommentEnd1 ) {
 	string->setEndState( StateCComment );
@@ -510,9 +508,9 @@ void SyntaxHighlighter_CPP::process( QTextDocument *doc, QTextParagraph *string,
 
     string->setFirstPreProcess( FALSE );
 
-    if ( invalidate && string->next() &&
-	 !string->next()->firstPreProcess() && string->next()->endState() != -1 ) {
-	QTextParagraph *p = string->next();
+    QTextParagraph *p = string->next();
+    if ( (!!oldEndState || !!string->endState()) && oldEndState != string->endState() &&
+	 invalidate && p && !p->firstPreProcess() && p->endState() != -1 ) {
 	while ( p ) {
 	    if ( p->endState() == -1 )
 		return;
