@@ -1,21 +1,28 @@
-/****************************************************************************
+/**********************************************************************
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
-** Qt/Embedded virtual framebuffer
+** This file is part of Qt/Embedded virtual framebuffer.
 **
-** Created : 20000605
+** This file may be distributed and/or modified under the terms of the
+** GNU General Public License version 2 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
+** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
-** This file is part of the Qt GUI Toolkit.
+** See http://www.trolltech.com/gpl/ for GPL licensing information.
 **
-** Licensees holding valid Qt Professional Edition licenses may use this
-** file in accordance with the Qt Professional Edition License Agreement
-** provided with the Qt Professional Edition.
+** Contact info@trolltech.com if any conditions of this licensing are
+** not clear to you.
 **
-** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
-** information about the Professional Edition licensing.
-**
-*****************************************************************************/
+**********************************************************************/
+
+#include "qvfb.h"
+#include "qvfbview.h"
+#include "qvfbratedlg.h"
+#include "config.h"
+#include "skin.h"
 
 #include <qmenubar.h>
 #include <qpopupmenu.h>
@@ -30,100 +37,27 @@
 #include <qimage.h>
 #include <qpixmap.h>
 #include <qcursor.h>
-
-
-#include "qvfb.h"
-#include "qvfbview.h"
-#include "qvfbratedlg.h"
-#include "config.h"
-#include "skin.h"
-
-
-static const char * logo[] = {
-/* width height ncolors chars_per_pixel */
-"50 50 17 1",
-/* colors */
-"  c #000000",
-". c #495808",
-"X c #2A3304",
-"o c #242B04",
-"O c #030401",
-"+ c #9EC011",
-"@ c #93B310",
-"# c #748E0C",
-"$ c #A2C511",
-"% c #8BA90E",
-"& c #99BA10",
-"* c #060701",
-"= c #181D02",
-"- c #212804",
-"; c #61770A",
-": c #0B0D01",
-"/ c None",
-/* pixels */
-"/$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$/",
-"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$",
-"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$",
-"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$",
-"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$",
-"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$",
-"$$$$$$$$$$$$$$$$$$$$$$$+++$$$$$$$$$$$$$$$$$$$$$$$$",
-"$$$$$$$$$$$$$$$$$$$@;.o=::=o.;@$$$$$$$$$$$$$$$$$$$",
-"$$$$$$$$$$$$$$$$+#X*         **X#+$$$$$$$$$$$$$$$$",
-"$$$$$$$$$$$$$$$#oO*         O  **o#+$$$$$$$$$$$$$$",
-"$$$$$$$$$$$$$&.* OO              O*.&$$$$$$$$$$$$$",
-"$$$$$$$$$$$$@XOO            * OO    X&$$$$$$$$$$$$",
-"$$$$$$$$$$$@XO OO  O  **:::OOO OOO   X@$$$$$$$$$$$",
-"$$$$$$$$$$&XO      O-;#@++@%.oOO      X&$$$$$$$$$$",
-"$$$$$$$$$$.O  :  *-#+$$$$$$$$+#- : O O*.$$$$$$$$$$",
-"$$$$$$$$$#*OO  O*.&$$$$$$$$$$$$+.OOOO **#$$$$$$$$$",
-"$$$$$$$$+-OO O *;$$$$$$$$$$$&$$$$;*     o+$$$$$$$$",
-"$$$$$$$$#O*  O .+$$$$$$$$$$@X;$$$+.O    *#$$$$$$$$",
-"$$$$$$$$X*    -&$$$$$$$$$$@- :;$$$&-    OX$$$$$$$$",
-"$$$$$$$@*O  *O#$$$$$$$$$$@oOO**;$$$#    O*%$$$$$$$",
-"$$$$$$$;     -+$$$$$$$$$@o O OO ;+$$-O   *;$$$$$$$",
-"$$$$$$$.     ;$$$$$$$$$@-OO OO  X&$$;O    .$$$$$$$",
-"$$$$$$$o    *#$$$$$$$$@o  O O O-@$$$#O   *o$$$$$$$",
-"$$$$$$+=    *@$$$$$$$@o* OO   -@$$$$&:    =$$$$$$$",
-"$$$$$$+:    :+$$$$$$@-      *-@$$$$$$:    :+$$$$$$",
-"$$$$$$+:    :+$$$$$@o* O    *-@$$$$$$:    :+$$$$$$",
-"$$$$$$$=    :@$$$$@o*OOO      -@$$$$@:    =+$$$$$$",
-"$$$$$$$-    O%$$$@o* O O    O O-@$$$#*   OX$$$$$$$",
-"$$$$$$$. O *O;$$&o O*O* *O      -@$$;    O.$$$$$$$",
-"$$$$$$$;*   Oo+$$;O*O:OO--      Oo@+=    *;$$$$$$$",
-"$$$$$$$@*  O O#$$$;*OOOo@@-O     Oo;O*  **@$$$$$$$",
-"$$$$$$$$X* OOO-+$$$;O o@$$@-    O O     OX$$$$$$$$",
-"$$$$$$$$#*  * O.$$$$;X@$$$$@-O O        O#$$$$$$$$",
-"$$$$$$$$+oO O OO.+$$+&$$$$$$@-O         o+$$$$$$$$",
-"$$$$$$$$$#*    **.&$$$$$$$$$$@o      OO:#$$$$$$$$$",
-"$$$$$$$$$+.   O* O-#+$$$$$$$$+;O    OOO:@$$$$$$$$$",
-"$$$$$$$$$$&X  *O    -;#@++@#;=O    O    -@$$$$$$$$",
-"$$$$$$$$$$$&X O     O*O::::O      OO    Oo@$$$$$$$",
-"$$$$$$$$$$$$@XOO                  OO    O*X+$$$$$$",
-"$$$$$$$$$$$$$&.*       **  O      ::    *:#$$$$$$$",
-"$$$$$$$$$$$$$$$#o*OO       O    Oo#@-OOO=#$$$$$$$$",
-"$$$$$$$$$$$$$$$$+#X:* *     O**X#+$$@-*:#$$$$$$$$$",
-"$$$$$$$$$$$$$$$$$$$%;.o=::=o.#@$$$$$$@X#$$$$$$$$$$",
-"$$$$$$$$$$$$$$$$$$$$$$$$+++$$$$$$$$$$$+$$$$$$$$$$$",
-"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$",
-"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$",
-"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$",
-"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$",
-"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$",
-"/$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$/",
-};
+#include <qdragobject.h>
+#include <qcheckbox.h>
 
 QVFb::QVFb( int display_id, int w, int h, int d, const QString &skin, QWidget *parent,
 	    const char *name, uint flags )
     : QMainWindow( parent, name, flags )
 {
-    imagesave = new QFileDialog(".", "*.png", this, 0, TRUE);
-    imagesave->setSelection("snapshot.png");
-    setIcon(QPixmap(logo));
+    const QMimeSource *m = QMimeSourceFactory::defaultFactory()->data( "logo.png" );
+    if ( m ) {
+	QPixmap pix;
+	QImageDrag::decode( m, pix );
+	setIcon( pix );
+    }
+
+    imagesave = new QFileDialog( this, 0, TRUE );
+
     rateDlg = 0;
     view = 0;
     init( display_id, w, h, d, skin );
     createMenu( menuBar() );
+    adjustSize();
 }
 
 QVFb::~QVFb()
@@ -155,7 +89,7 @@ void QVFb::init( int display_id, int w, int h, int d, const QString &skin_name )
 	view->setFixedSize( w, h );
 	setCentralWidget( skin );
 	adjustSize();
-	view->show(); 
+	view->show();
 	if ( vis ) show();
     } else {
 	if ( !currentSkin.isEmpty() ) {
@@ -167,8 +101,8 @@ void QVFb::init( int display_id, int w, int h, int d, const QString &skin_name )
 	view->setMargin( 0 );
 	view->setFrameStyle( QFrame::NoFrame );
 	setCentralWidget( view );
-	resize(w,menuBar()->height()+h+2);
-	view->show(); 
+	resize( sizeHint() );
+	view->show();
     }
 
     currentSkin = skin_name;
@@ -193,7 +127,7 @@ void QVFb::createMenu(QMenuData *menu)
     menu->insertItem( "&File", file );
 
     viewMenu = new QPopupMenu( this );
-    viewMenu->setCheckable( true );
+    viewMenu->setCheckable( TRUE );
     cursorId = viewMenu->insertItem( "Show &Cursor", this, SLOT(toggleCursor()) );
     enableCursor(TRUE);
     viewMenu->insertItem( "&Refresh Rate...", this, SLOT(changeRate()) );
@@ -208,6 +142,7 @@ void QVFb::createMenu(QMenuData *menu)
 
     QPopupMenu *help = new QPopupMenu( this );
     help->insertItem("&About", this, SLOT(about()));
+    help->insertItem("About &Qt", this, SLOT(aboutQt()));
     menu->insertSeparator();
     menu->insertItem( "&Help", help );
 }
@@ -245,11 +180,9 @@ void QVFb::setZoom4()
 void QVFb::saveImage()
 {
     QImage img = view->image();
-    if ( imagesave->exec() ) {
-	QString filename = imagesave->selectedFile();
-	if ( !!filename )
-	    img.save(filename,"PNG");
-    }
+    QString filename = imagesave->getSaveFileName("snapshot.png", "*.png", this, "", "Save Image");
+    if ( !!filename )
+	img.save(filename,"PNG");
 }
 
 void QVFb::toggleAnimation()
@@ -257,7 +190,7 @@ void QVFb::toggleAnimation()
     if ( view->animating() ) {
 	view->stopAnimation();
     } else {
-	QString filename = imagesave->getSaveFileName("animation.mng", "*.mng", this, "", "Save animation...");
+	QString filename = imagesave->getSaveFileName("animation.mng", "*.mng", this, "", "Save animation");
 	if ( !filename ) {
 	    view->stopAnimation();
 	} else {
@@ -283,18 +216,38 @@ void QVFb::changeRate()
 
 void QVFb::about()
 {
+#if defined( Q_WS_MAC )
+    QString platform("Mac OS X");
+    QString qt("Mac");
+#elif defined( Q_WS_WIN )
+    QString platform("Windows");
+    QString qt("Windows");
+#else
+    QString platform("X11");
+    QString qt("X11");
+#endif
     QMessageBox::about(this, "About QVFB",
-	"<h2>The Qt/Embedded Virtual X11 Framebuffer</h2>"
-	"<p>This application runs under Qt/X11, emulating a simple framebuffer, "
-	"which the Qt/Embedded server and clients can attach to just as if "
-	"it was a hardware Linux framebuffer. "
-	"<p>With the aid of this development tool, you can develop Qt/Embedded "
-	"applications under X11 without having to switch to a virtual console. "
-	"This means you can comfortably use your other development tools such "
-	"as GUI profilers and debuggers."
+	"<p><b><font size=+2>Qt/Embedded Virtual " + platform + " Framebuffer</font></b></p>"
+	"<p></p>"
+	"<p>Version 1.0</p>"
+	"<p>Copyright (C) 2001-2002 Trolltech AS. All rights reserved.</p>"
+	"<p></p>"
+	"<p>This program is licensed to you under the terms of the GNU General "
+	"Public License Version 2 as published by the Free Software Foundation. This "
+	"gives you legal permission to copy, distribute and/or modify this software "
+	"under certain conditions. For details, see the file 'LICENSE.GPL' that came with "
+	"this software distribution. If you did not get the file, send email to "
+	"info@trolltech.com.</p>\n\n<p>The program is provided AS IS with NO WARRANTY "
+	"OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS "
+	"FOR A PARTICULAR PURPOSE.</p>"
     );
 }
 
+void QVFb::aboutQt()
+{
+    QMessageBox::aboutQt( this, tr("QVFB") );
+}
+
 void QVFb::configure()
 {
     config = new Config(this,0,TRUE);
@@ -313,6 +266,7 @@ void QVFb::configure()
 	config->size_skin->setChecked(TRUE);
 	config->skin->setEditText(currentSkin);
     }
+    config->touchScreen->setChecked(view->touchScreenEmulation());
     config->depth_1->setChecked(view->displayDepth()==1);
     config->depth_4gray->setChecked(view->displayDepth()==4);
     config->depth_8->setChecked(view->displayDepth()==8);
@@ -366,9 +320,10 @@ void QVFb::configure()
 	else
 	    d=32;
 
-	if ( w != view->displayWidth() || h != view->displayHeight() 
+	if ( w != view->displayWidth() || h != view->displayHeight()
 		|| d != view->displayDepth() || skin != currentSkin )
 	    init( id, w, h, d, skin );
+	view->setTouchscreenEmulation( config->touchScreen->isChecked() );
     } else {
 	view->setGamma(ogr, ogg, ogb);
     }
@@ -419,9 +374,3 @@ void QVFb::updateGammaLabels()
     config->glabel->setText(QString::number(view->gammaGreen(),'g',2));
     config->blabel->setText(QString::number(view->gammaBlue(),'g',2));
 }
-
-QSize QVFb::sizeHint() const
-{
-    return QSize(view->displayWidth()*view->zoom(),
-	    menuBar()->height()+view->displayHeight()*view->zoom());
-}
